
#include "DynamicFunctions.h"
#include <algorithm>
#include <math.h>

//Author(s) : Li Zhang 
//lzhan126@syr.edu Feb 2021

//#include <boost/math/distributions/normal.hpp>


double DynamicFunctions::ecdf(const std::vector<double>& DataRecord, double data)
{
    //Number of elements < event data
    double num=0;
    double ecdf = 0.00;

    for (auto it = DataRecord.begin(); it != DataRecord.end(); ++it) {
        if (*it <= data) { ++num; }
    }
    //ecdf range from 0-1
    ecdf = num / DataRecord.size();
    return ecdf;
    //test
    //cout << ecdf << endl;
}


double DynamicFunctions::quantile(const std::vector<double>& ECVector, double percentile, Inputs* input)
{
    //https://www.rdocumentation.org/packages/stats/versions/3.6.2/topics/quantile
    double m = 0.0;
    double j = 0.0;
    int n;
    double g = 0.0;
    double Q = 0.0;

    //n is the sample size
    n = ECVector.size();
    //For types 4 through 9, m given below
    //type4: linear interpolation of the empirical cdf
    if (input->ECDynamicVariables["QuantileType"] == 4) {
        m = 0;
    }
    //type5: piecewise linear function where the knots are the values midway through the steps of the empirical cdf
    if (input->ECDynamicVariables["QuantileType"] == 5) {
        m = 0.5;
    }
    //type6: used by Minitaband by SPSS.
    if (input->ECDynamicVariables["QuantileType"] == 6) {
        m = percentile;
    }
    //type7: This is used by S.
    if (input->ECDynamicVariables["QuantileType"] == 7) {
        m = 1 - percentile;
    }
    //type8: The resulting quantile estimates are approximately median-unbiased regardless of the distribution of x
    if (input->ECDynamicVariables["QuantileType"] == 8) {
        m = (percentile + 1)/3;
    }
    //type9: The resulting quantile estimates are approximately unbiased for the expected order statistics if x is normally distributed
    if (input->ECDynamicVariables["QuantileType"] == 9) {
        m = percentile/4 + 3/8;
    }
    
    j = floor(n * percentile + m);
    g = n * percentile + m - j;
    
    //j = order number; 
    //insert one more data (0) at the beginning of the vector, so we can use the same order as R code: j & j+1
    //if we remove this line, since the vector begin from j=0, j & j+1 in the equation-> j-1 & j
    //VectorSorted.insert(VectorSorted.begin(), 0);
    //Q = (1 - g) * VectorSorted[j] + g * VectorSorted[j+1];
    if (j == ECVector.size()) {
        Q = (1 - g) * ECVector[j-1];
    }
    else Q = (1 - g) * ECVector[j-1] + g * ECVector[j];

     return Q;
}

vector< vector<double> > DynamicFunctions::pollutant_landcover_list;
void DynamicFunctions::Interpolation(const std::vector<double>& DataRecord, Inputs* input) {
    double Q1st, Q2nd, Qlast, Qlast2nd,slope_Q_begin, slope_Q_end,EC_begin, EC_end;
    std::vector<double> DataRecordSorted(DataRecord.size());
    //sort the discharge record
    partial_sort_copy(DataRecord.begin(), DataRecord.end() , DataRecordSorted.begin(), DataRecordSorted.end());
    //use the quantile function to get the Q at the tails
    Q1st= quantile(DataRecordSorted, 0.0003, input);//0th
    Qlast = quantile(DataRecordSorted, 0.9997, input);//100th
    Q2nd = quantile(DataRecordSorted,0.05, input);//5th->0.05
    Qlast2nd = quantile(DataRecordSorted, 0.95, input);//95th->0.95
    //find the slope at teh tails
    if (Q2nd == 0) {
        slope_Q_begin = 0;
    }
    else slope_Q_begin = Q1st / Q2nd;

    if (Qlast2nd == 0) {
        slope_Q_end = 0;
    }
    else slope_Q_end = Qlast / Qlast2nd;

    /*vector<string> pollutant_landcover_vec{ "TP21", "TP22", "TP23", "TP24", "TP41", "TP42", "TP43", "TP71", "TP81", "TP82",
        "TN21", "TN22", "TN23", "TN24", "TN41", "TN42", "TN43", "TN71", "TN81", "TN82",
        "TSS21", "TSS22", "TSS23", "TSS24", "TSS41", "TSS42", "TSS43", "TSS71", "TSS81", "TSS82" };*/
    //list< vector<double> >::iterator list_it;
    //vector< vector<double> > DynamicFunctions::pollutant_landcover_list;
    /*pollutant_landcover_list.emplace_back(input->TP21, input->TP22, input->TP23, input->TP24, input->TP41, input->TP42, input->TP43, input->TP71, input->TP81, input->TP82,
        input->TN21, input->TN22, input->TP23, input->TN24, input->TN41, input->TN42, input->TN43, input->TN71, input->TN81, input->TN82,
        input->TSS21, input->TSS22, input->TSS23, input->TSS24, input->TSS41, input->TSS42, input->TSS43, input->TSS71, input->TSS81, input->TSS82);*/
    
    pollutant_landcover_list.push_back(input->TP21);
    pollutant_landcover_list.push_back(input->TP22);
    pollutant_landcover_list.push_back(input->TP23);
    pollutant_landcover_list.push_back(input->TP24);
    pollutant_landcover_list.push_back(input->TP41);
    pollutant_landcover_list.push_back(input->TP42);
    pollutant_landcover_list.push_back(input->TP43);
    pollutant_landcover_list.push_back(input->TP71);
    pollutant_landcover_list.push_back(input->TP81);
    pollutant_landcover_list.push_back(input->TP82);

    pollutant_landcover_list.push_back(input->TN21);
    pollutant_landcover_list.push_back(input->TN22);
    pollutant_landcover_list.push_back(input->TN23);
    pollutant_landcover_list.push_back(input->TN24);
    pollutant_landcover_list.push_back(input->TN41);
    pollutant_landcover_list.push_back(input->TN42);
    pollutant_landcover_list.push_back(input->TN43);
    pollutant_landcover_list.push_back(input->TN71);
    pollutant_landcover_list.push_back(input->TN81);
    pollutant_landcover_list.push_back(input->TN82);

    pollutant_landcover_list.push_back(input->TSS21);
    pollutant_landcover_list.push_back(input->TSS22);
    pollutant_landcover_list.push_back(input->TSS23);
    pollutant_landcover_list.push_back(input->TSS24);
    pollutant_landcover_list.push_back(input->TSS41);
    pollutant_landcover_list.push_back(input->TSS42);
    pollutant_landcover_list.push_back(input->TSS43);
    pollutant_landcover_list.push_back(input->TSS71);
    pollutant_landcover_list.push_back(input->TSS81);
    pollutant_landcover_list.push_back(input->TSS82);


    for (auto list_it = pollutant_landcover_list.begin(); list_it != pollutant_landcover_list.end(); ++list_it) {
        //vector<double> &V = *list_it;
        int num_insert = 0;
        EC_begin = (*list_it)[0] * slope_Q_begin;
        EC_end = (*list_it).back() * slope_Q_end;
        (*list_it).insert((*list_it).begin(), EC_begin);
        (*list_it).insert((*list_it).end(), EC_end);
        //now we have 0th, 5th, 10th, 25th, 50th, 75th, 90th, 95th, 100th (9 in total)
        //insert 12 new values using linear interpolation, to have 5th interval (21 intotal)
        int interval = 5;
        //filling the 1st gap
        double x_increment1 = 15;//input->percentileVec[2] - input->percentileVec[1];//25-10
        double y_increment1 = (*list_it)[3] - (*list_it)[2];
        double slope1 = y_increment1 / x_increment1;
        for (auto i = 0; i < x_increment1 / interval-1; ++i) {//insert 2 times
            //inserts EC at (3+i)-th index, for this loop, we add 2 values, add at begin() + 3 and begin() + 4
            auto it = (*list_it).insert((*list_it).begin() + 3+i, (*list_it)[2]+slope1* interval*(i+1));
        }
        num_insert += x_increment1 / interval - 1;//=2
        
        //filling the 2nd gap
        double x_increment2 = 25;// input->percentileVec[3] - input->percentileVec[2];//50-25
        double y_increment2 = (*list_it)[3 + num_insert+1] - (*list_it)[3+num_insert ];
        double slope2 = y_increment2 / x_increment2;
        for (auto i = 0; i < x_increment2 / interval - 1; ++i) {//insert 4 times
            auto it = (*list_it).insert((*list_it).begin() + 4+ num_insert + i, (*list_it)[3+ num_insert]+ slope2 * interval * (i + 1));
        }
        num_insert += x_increment2 / interval - 1;//=6

        //filling the 3rd gap
        double x_increment3 = 25;//input->percentileVec[4] - input->percentileVec[3];//75-50
        double y_increment3 = (*list_it)[4+ num_insert+1] - (*list_it)[4 + num_insert];
        double slope3 = y_increment3 / x_increment3;
        for (auto i = 0; i < x_increment3 / interval - 1; ++i) {//insert 4 times
            auto it = (*list_it).insert((*list_it).begin() + 5 + num_insert + i, (*list_it)[4+ num_insert] + slope3 * interval * (i + 1));
        }
        num_insert += x_increment3 / interval - 1;//=10

        //filling the 4rd gap
        double x_increment4 = 15;// input->percentileVec[5] - input->percentileVec[4];//90-75
        double y_increment4 = (*list_it)[5 + num_insert + 1] - (*list_it)[5 + num_insert];
        double slope4 = y_increment4 / x_increment4;
        for (auto i = 0; i < x_increment4 / interval - 1; ++i) {//insert 2 times
            auto it = (*list_it).insert((*list_it).begin() + 6 + num_insert + i, (*list_it)[5 + num_insert] + slope4 * interval * (i + 1));
        }
        num_insert += x_increment4 / interval - 1;//=12
    }

}



double DynamicFunctions::LCEventload(const std::vector<double>& DataRecord, const std::vector<double>& ECVector, double Eventdata, double cell_num, Inputs* input) {
    double LCEC = 0.0;
    double EventECDF = 0.0;
    double EventTotal = 0.0;
    //get the ecdf for this event, function returns cumulative dristribution probability between 0 and 1, it is (1- exceedance probability)
    //DataRecord is a vector that reading the daily discharge, Eventdata is a scaler of that day's discharge value, cfs
    EventECDF = ecdf(DataRecord, Eventdata);
    //use the quantile funciton to find the EC of this event in this landcover
    LCEC = quantile(ECVector, EventECDF, input);
    //EventTotal = EC for this landcover * cell number
    EventTotal = LCEC * cell_num;
    return EventTotal; 


}



void DynamicFunctions::YearAverage(const std::vector<double>& DataRecord, const std::vector<int>& YearRecord, Inputs* input) {

    double Eventdata = 0.000;
    int year;
    std::vector<int> uniqueyear;
    std::vector<double>AverageYearLoad_TP;
    std::vector<double>AverageYearLoad_TN;
    std::vector<double>AverageYearLoad_TSS;

    double sum;//a temp value
    double averageEC;//a temp value
    double sdr = input->ECDynamicVariables["SDR"];//sediment delivery ratio 
    double area = 0.0;
    double cell_size = 0.0;
    double cell_count = 0.0;
    double Threshold = input->ECDynamicVariables["Threshold"];

    std::vector<double>DataRecord_th;
    std::vector<int>YearRecord_th;

    //seletct the data greater than the Threshold
    auto YearRecord_th_it = YearRecord.begin();
    for (auto DataRecord_th_it = DataRecord.begin(); DataRecord_th_it != DataRecord.end(); ++DataRecord_th_it) {
        if (*DataRecord_th_it >= Threshold) {
            DataRecord_th.push_back(*DataRecord_th_it);
            YearRecord_th.push_back(*YearRecord_th_it);
        }
        ++YearRecord_th_it;
    }
    //Linear Interpolation
    Interpolation(DataRecord_th, input);

    //The cell size of the blockgroup map is 370m
    cell_size = input -> cellsize;
    cell_count = input->cell_count;
    //watershed area is cell number(m)*cell_size (m)^2/10000 ha
    area = cell_count * (cell_size * cell_size) / 10000;

    //use the sort and unique function within algorithm library to find out the unique years we are simulating
    uniqueyear = YearRecord_th;
    sort(uniqueyear.begin(), uniqueyear.end());
    uniqueyear.erase(unique(uniqueyear.begin(), uniqueyear.end()), uniqueyear.end());

    //for each year
    for (auto it = uniqueyear.begin(); it != uniqueyear.end(); ++it) {
        std::vector<double>datarecord_year;//a temp value
        //set an iterator at the begining of the discharge data vector, recordit will interate with yearit      
        auto yearit = YearRecord_th.begin();
        for (auto recordit = DataRecord_th.begin(); recordit != DataRecord_th.end(); ++recordit) {
            if (*yearit == *it) {
                datarecord_year.push_back(*recordit);
            }
            ++yearit;
        }


        std::vector<double> EventEC_TP;
        std::vector<double> EventEC_TN;
        std::vector<double> EventEC_TSS;
        //for each event in this year
        //if use all data record to build the probability distribution, rather than using event > th, change DataRecord_th to DataRecord for the follwing line within the loop 
        for (auto recordit = datarecord_year.begin(); recordit != datarecord_year.end(); ++recordit) {
            Eventdata = *recordit;
            double ECTP21 = 0.0, ECTP22 = 0.0, ECTP23 = 0.0, ECTP24 = 0.0, ECTP41 = 0.0, ECTP42 = 0.0, ECTP43 = 0.0, ECTP71 = 0.0, ECTP81 = 0.0, ECTP82 = 0.0;
            double ECTN21 = 0.0, ECTN22 = 0.0, ECTN23 = 0.0, ECTN24 = 0.0, ECTN41 = 0.0, ECTN42 = 0.0, ECTN43 = 0.0, ECTN71 = 0.0, ECTN81 = 0.0, ECTN82 = 0.0;
            double ECTSS21 = 0.0, ECTSS22 = 0.0, ECTSS23 = 0.0, ECTSS24 = 0.0, ECTSS41 = 0.0, ECTSS42 = 0.0, ECTSS43 = 0.0, ECTSS71 = 0.0, ECTSS81 = 0.0, ECTSS82 = 0.0;

            
            ECTP21 = LCEventload(DataRecord_th, pollutant_landcover_list[0], Eventdata, input->Cell_number[0], input);
            ECTP22 = LCEventload(DataRecord_th, pollutant_landcover_list[1], Eventdata, input->Cell_number[1], input);
            ECTP23 = LCEventload(DataRecord_th, pollutant_landcover_list[2], Eventdata, input->Cell_number[2], input);
            ECTP24 = LCEventload(DataRecord_th, pollutant_landcover_list[3], Eventdata, input->Cell_number[3], input);
            ECTP41 = LCEventload(DataRecord_th, pollutant_landcover_list[4], Eventdata, input->Cell_number[4], input);
            ECTP42 = LCEventload(DataRecord_th, pollutant_landcover_list[5], Eventdata, input->Cell_number[5], input);
            ECTP43 = LCEventload(DataRecord_th, pollutant_landcover_list[6], Eventdata, input->Cell_number[6], input);
            ECTP71 = LCEventload(DataRecord_th, pollutant_landcover_list[7], Eventdata, input->Cell_number[7], input);
            ECTP81 = LCEventload(DataRecord_th, pollutant_landcover_list[8], Eventdata, input->Cell_number[8], input);
            ECTP82 = LCEventload(DataRecord_th, pollutant_landcover_list[9], Eventdata, input->Cell_number[9], input);

            ECTN21 = LCEventload(DataRecord_th, pollutant_landcover_list[10], Eventdata, input->Cell_number[0], input);
            ECTN22 = LCEventload(DataRecord_th, pollutant_landcover_list[11], Eventdata, input->Cell_number[1], input);
            ECTN23 = LCEventload(DataRecord_th, pollutant_landcover_list[12], Eventdata, input->Cell_number[2], input);
            ECTN24 = LCEventload(DataRecord_th, pollutant_landcover_list[13], Eventdata, input->Cell_number[3], input);
            ECTN41 = LCEventload(DataRecord_th, pollutant_landcover_list[14], Eventdata, input->Cell_number[4], input);
            ECTN42 = LCEventload(DataRecord_th, pollutant_landcover_list[15], Eventdata, input->Cell_number[5], input);
            ECTN43 = LCEventload(DataRecord_th, pollutant_landcover_list[16], Eventdata, input->Cell_number[6], input);
            ECTN71 = LCEventload(DataRecord_th, pollutant_landcover_list[17], Eventdata, input->Cell_number[7], input);
            ECTN81 = LCEventload(DataRecord_th, pollutant_landcover_list[18], Eventdata, input->Cell_number[8], input);
            ECTN82 = LCEventload(DataRecord_th, pollutant_landcover_list[19], Eventdata, input->Cell_number[9], input);

            ECTSS21 = LCEventload(DataRecord_th, pollutant_landcover_list[20], Eventdata, input->Cell_number[0], input);
            ECTSS22 = LCEventload(DataRecord_th, pollutant_landcover_list[21], Eventdata, input->Cell_number[1], input);
            ECTSS23 = LCEventload(DataRecord_th, pollutant_landcover_list[22], Eventdata, input->Cell_number[2], input);
            ECTSS24 = LCEventload(DataRecord_th, pollutant_landcover_list[23], Eventdata, input->Cell_number[3], input);
            ECTSS41 = LCEventload(DataRecord_th, pollutant_landcover_list[14], Eventdata, input->Cell_number[4], input);
            ECTSS42 = LCEventload(DataRecord_th, pollutant_landcover_list[25], Eventdata, input->Cell_number[5], input);
            ECTSS43 = LCEventload(DataRecord_th, pollutant_landcover_list[26], Eventdata, input->Cell_number[6], input);
            ECTSS71 = LCEventload(DataRecord_th, pollutant_landcover_list[27], Eventdata, input->Cell_number[7], input);
            ECTSS81 = LCEventload(DataRecord_th, pollutant_landcover_list[28], Eventdata, input->Cell_number[8], input);
            ECTSS82 = LCEventload(DataRecord_th, pollutant_landcover_list[29], Eventdata, input->Cell_number[9], input);

            EventEC_TP.push_back((ECTP21 + ECTP22 + ECTP23 + ECTP24 + ECTP41 + ECTP42 + ECTP43 + ECTP71 + ECTP81 + ECTP82) / cell_count);
            EventEC_TN.push_back((ECTN21 + ECTN22 + ECTP23 + ECTN24 + ECTN41 + ECTN42 + ECTN43 + ECTN71 + ECTN81 + ECTN82) / cell_count);
            EventEC_TSS.push_back((ECTSS21 + ECTSS22 + ECTSS23 + ECTSS24 + ECTSS41 + ECTSS42 + ECTSS43 + ECTSS71 + ECTSS81 + ECTSS82) / cell_count);


                
            
            
            /*if (recordit == datarecord_year.begin()) {
                cout << "LC71:" << endl;
                cout << "ECTP71: " << ECTP71 << "  ECTN71: " << ECTN71 << "  ECTSS71: " << ECTSS71 << endl;
                cout << "LC81:" << endl;
                cout << "ECTP81: " << ECTP81 << "  ECTN81: " << ECTN81 << "  ECTSS81: " << ECTSS81 << endl;
                cout << "LC21:" << endl;
                cout << "ECTP21: " << ECTP21 << "  ECTN21: " << ECTN21 << "  ECTSS21: " << ECTSS21 << endl;

            }*/
        }
        sum = std::accumulate(EventEC_TP.begin(), EventEC_TP.end(), 0.0);
        averageEC = sum / EventEC_TP.size();
        //convert from kg/ha/yr to metric ton/ha/yr
        AverageYearLoad_TP.push_back(averageEC * area * sdr / 1000);

        sum = std::accumulate(EventEC_TN.begin(), EventEC_TN.end(), 0.0);
        averageEC = sum / EventEC_TN.size();
        //convert from kg/ha/yr to metric ton/ha/yr
        AverageYearLoad_TN.push_back(averageEC * area * sdr / 1000);

        sum = std::accumulate(EventEC_TSS.begin(), EventEC_TSS.end(), 0.0);
        averageEC = sum / EventEC_TSS.size();
        //convert from kg/ha/yr to metric ton/ha/yr
        AverageYearLoad_TSS.push_back(averageEC * area * sdr / 1000);
    }


    ECDynamicOutputwriter(AverageYearLoad_TP, uniqueyear, input, "TP");
    ECDynamicOutputwriter(AverageYearLoad_TN, uniqueyear, input, "TN");
    ECDynamicOutputwriter(AverageYearLoad_TSS, uniqueyear, input, "TSS");
    cout << "Finish Writing" << endl;

    //print the new EC
    /*for (auto i = 0; i < 30; ++i) {
        for (auto it = pollutant_landcover_list[i].begin(); it != pollutant_landcover_list[i].end(); ++it)
        {
            std::cout << *it << ' ';
        }
        std::cout << endl;
    }*/

    /*for (auto it = AverageYearLoad_TP.begin(); it != AverageYearLoad_TP.end(); ++it)
    {
        std::cout << *it << ' ';
    }
    std::cout << endl;
    for (auto it = AverageYearLoad_TN.begin(); it != AverageYearLoad_TN.end(); ++it)
    {
        std::cout << *it << ' ';
    }
    std::cout << endl;
    for (auto it = AverageYearLoad_TSS.begin(); it != AverageYearLoad_TSS.end(); ++it)
    {
        std::cout << *it << ' ';
    }
    std::cout << endl;
    
    for (auto it = input->Cell_number.begin(); it != input->Cell_number.end(); ++it)
    {
        std::cout << *it << ' ';
    }
    std::cout << endl;

    
    for (auto it = input->TP71.begin(); it != input->TP71.end(); ++it) {
        cout << *it << " ";
    }
    cout << endl;
    for (auto it = input->TN71.begin(); it != input->TN71.end(); ++it) {
        cout << *it << " ";
    }
    cout << endl;
    for (auto it = input->TSS71.begin(); it != input->TSS71.end(); ++it) {
        cout << *it << " ";
    }
    cout << endl;*/
}
   

 void DynamicFunctions::ECDynamicOutputwriter(const std::vector<double>& result, const std::vector<int>& Year, Inputs* input, const std::string& pollutant) {
     ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "AverageYearLoad_"+ pollutant + ".csv");
     if (!outfile.good())        
         return;
     //else
         //cout << "writing to" << endl;
         //cout << (input->SimulationStringParams["OutputDirectory"] + "AverageYearLoad.csv") << endl;
     
     outfile.precision(15);
     outfile.setf(ios::showpoint);
     
     

     outfile << "Total Area:" << ","<< input ->cell_count *30 *30<< "," << "sq m" << std::endl;
     outfile <<"Cell size:" << "," << input -> cellsize << ","<< "m" << std::endl;
     outfile << "Land Cover Type" << "," << "LC21" << "," << "LC22" << "," << "LC23" << "," << "LC24" << "," << "LC31" << "," 
         << "LC41" << "," << "LC42" << "," << "LC43" << "," << "LC52" << "," << "LC71" << "," << "LC81" << "," << "LC82" << std::endl;
         
     outfile << "Cells" << ",";
     /*for (auto LCit = input->Cell_number.begin(); LCit != input->Cell_number.end(); ++LCit) {
         outfile << *LCit
             << ",";
     }  */
     outfile <<input->LC21 << "," << input->LC22 << "," << input->LC23 << "," << input->LC24 << "," << input->LC31 << ","
         << input->LC41 << "," << input->LC42 << "," << input->LC43 << "," << input->LC52 << ","
         << input->LC71 << "," << input->LC81 << "," << input->LC82 << ",";
     outfile << std::endl;
     outfile << std::endl;
     
	
     outfile << "Year"
         << "," << "Load"
         << "," << "Unit"
         << std::endl;
     
     auto loadit = result.begin();
     for (auto yearit = Year.begin(); yearit != Year.end(); ++yearit) {
 
         outfile << *yearit
             << "," << *loadit
             << "," << "Metric tons"
             << std::endl;
         loadit++;
     }
     outfile.close();
 }