//////////////////////////////////////////////////////////////////////////	
//  TemperatureOutputWriter.cpp - Provides methods to write Temperature	//
//					             related output files				    //
//////////////////////////////////////////////////////////////////////////

#include "TemperatureOutputWriter.h"
#include <fstream>
#include <iomanip>




void TemperatureOutputWriter::timeStepResultsFileProcessor(int timeStep, DataOrganizer* organizer, std::string outDirectory, Inputs* input)
{

	bool print = input->TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"];
	int hr = timeStep % 24;
	int currDay = input->CalendarDates[timeStep] * 100 + hr;
	if (!print)
	{
		for (int iter = 0; iter < input->StartDatesForTimeBasedTempOutput.size(); ++iter)
		{
			if (currDay >= input->StartDatesForTimeBasedTempOutput[iter] && currDay <= input->EndDatesForTimeBasedTempOutput[iter])
			{
				print = true;
				break;
			}
		}
	}

	if (print)
		createTimeStepResultFile(input->TemperatureExecutionParams["PrintTimeBasedResults"], currDay, input);

	//create file names for time-based output, and write the six header rows for GIS output
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"] == 1)
	{
		createTimeStepResultFile("Tair_K", currDay, input);
		createTimeStepResultFile("Tdew_K", currDay, input);
		createTimeStepResultFile("AbsHumidity_kg_p_m3", currDay, input);

		createTimeStepResultFile("NR_total_W_p_m2", currDay, input);
		createTimeStepResultFile("ImpNR_W_p_m2", currDay, input);
		createTimeStepResultFile("TreeNR_W_p_m2", currDay, input);
		createTimeStepResultFile("ShortVegNR_W_p_m2", currDay, input);
		createTimeStepResultFile("SoilNR_W_p_m2", currDay, input);
		createTimeStepResultFile("WaterNR_W_p_m2", currDay, input);
		createTimeStepResultFile("ImperviousH_W_p_m2", currDay, input);
		createTimeStepResultFile("ImperviousLE_W_p_m2", currDay, input);
		createTimeStepResultFile("TreeH_W_p_m2", currDay, input);
		createTimeStepResultFile("TreeLE_W_p_m2", currDay, input);
		createTimeStepResultFile("TreeLEE_W_p_m2", currDay, input);
		createTimeStepResultFile("TreeLET_W_p_m2", currDay, input);
		createTimeStepResultFile("ShortVegH_W_p_m2", currDay, input);
		createTimeStepResultFile("ShortVegLE_W_p_m2", currDay, input);
		createTimeStepResultFile("ShortVegLEE_W_p_m2", currDay, input);
		createTimeStepResultFile("ShortVegLET_W_p_m2", currDay, input);
		createTimeStepResultFile("SoilH_W_p_m2", currDay, input);
		createTimeStepResultFile("SoilLE_W_p_m2", currDay, input);
		createTimeStepResultFile("WaterH_W_p_m2", currDay, input);
		createTimeStepResultFile("WaterLE_W_p_m2", currDay, input);
		createTimeStepResultFile("H_total_W_p_m2", currDay, input);
		createTimeStepResultFile("LE_total_W_p_m2", currDay, input);

		createTimeStepResultFile("DeltaQ_W_p_m2", currDay, input);
		createTimeStepResultFile("ImpDeltaQ_W_p_m2", currDay, input);
		createTimeStepResultFile("TreeDeltaQ_W_p_m2", currDay, input);
		createTimeStepResultFile("ShortDeltaQ_W_p_m2", currDay, input);
		createTimeStepResultFile("SoilDeltaQ_W_p_m2", currDay, input);
		createTimeStepResultFile("WaterDeltaQ_W_p_m2", currDay, input);

	}
	if (print)
	{
		writeToTimeStepResultFile(input->TemperatureExecutionParams["PrintTimeBasedResults"], currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
	}
	//write values to time-based output, added below the six header rows for GIS output
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"])
	{
		writeToTimeStepResultFile("Tair_K", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("Tdew_K", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("AbsHumidity_kg_p_m3", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("NR_total_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImpNR_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeNR_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegNR_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilNR_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterNR_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImperviousH_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImperviousLE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeH_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeLE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeLEE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeLET_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegH_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegLE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegLEE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortVegLET_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilH_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilLE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterH_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterLE_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("H_total_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("LE_total_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("DeltaQ_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ImpDeltaQ_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("TreeDeltaQ_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("ShortDeltaQ_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("SoilDeltaQ_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
		writeToTimeStepResultFile("WaterDeltaQ_W_p_m2", currDay, input->SimulationStringParams["OutputDirectory"], organizer, input);
	}
}

void TemperatureOutputWriter::rowColFileBatchProcessor(int timeStep, int organizerLocationIndex, DataFolder* folder, std::string outDirectory, Inputs* input)
{

	bool locationDataIsToBeOutputted = input->TemperatureCalculationParams["PrintAllRowColOutput"];
	if (!locationDataIsToBeOutputted)
		locationDataIsToBeOutputted = input->LocationIdsToPrintTempOutput.find(organizerLocationIndex) != input->LocationIdsToPrintTempOutput.end();
	if (!locationDataIsToBeOutputted)
		return;
	int row = (int)(organizerLocationIndex / (int)(input->LocationParams["nCols"]) + 1);
	int col = (organizerLocationIndex % (int)(input->LocationParams["nCols"]) + 1);
	bool printLocation = input->TemperatureCalculationParams["RowColOutputAllTimeSteps"];

	//control to write out 1st time step, connects with TestSimulation.cpp statement: if (timeStep > 0)
	if (timeStep > 0) {
		int hr = timeStep % 24;
		int currDay = input->CalendarDates[timeStep] * 100 + hr;

		if (!printLocation)
		{
			for (int iter = 0; iter < input->StartDatesForRowColTempOutput.size(); ++iter)
			{
				if (currDay >= input->StartDatesForRowColTempOutput[iter] && currDay <= input->EndDatesForRowColTempOutput[iter])
				{
					printLocation = true;
					break;
				}
			}
		}
		if (printLocation)
		{
			writeToHeatRowColFile(currDay, row, col, folder, input->SimulationStringParams["OutputDirectory"]);
			writeToHydroRowColFile(currDay, row, col, folder, input->SimulationStringParams["OutputDirectory"]);
		}
	}
}




void TemperatureOutputWriter::createHydroRowColFile(int i, int j, std::string outDirectory, Inputs* input)
{
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Hydro.txt");
	ofstream outfile(outDirectory + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to " + outDirectory + str);
		return;
	}
	//  std::cout<<"Writing outputs.txt file\n"<<std::endl;
	int col = input->LocationParams["nCols"];
	outfile << "Elevation:  " << input->DEM[i * col + j] << endl;
	outfile << "Slope:  " << input->Slope[i * col + j] << endl;
	outfile << "Aspect:  " << input->Aspect[i * col + j] << endl;
	outfile << "AverageIndex: " << input->AverageIndex << endl;
	outfile << "TI:  " << input->Index[i * col + j] << endl;
	outfile << "Land cover type:  " << input->NLCDLandCoverData[i * col + j] << endl;
	outfile << "Tree cover percentage:  " << input->TreeCoverdata[i * col + j] << endl;
	//outfile <<"Short vegetation cover percentage:  "<<repo->input->getSpData()->getImpCover(i*col+j)<<endl;
	outfile << "Impervious cover percentage:  " << input->ImperCoverdata[i * col + j] << endl;
	outfile << "WOnImp    " << "WOnTree    " << "WOnShort    " <<
		"Cumfil    " << "FilExQ    " << "RootStoreD    " << "OverLandQ    " <<
		"LocalSMD    " << "uszStore    " <<
		"Tp    " << "Ponding    " << "ImpDrunoff    " << endl;
	outfile.close();
	LogMessage::log("Created " + outDirectory + str);
}


void TemperatureOutputWriter::writeToHydroRowColFile(int timestemp, int i, int j, DataFolder* folder, std::string outDirectory) {
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Hydro.txt");
	ofstream outfile(outDirectory + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to " + outDirectory + str);
		return;
	}
	//outfile << folder->VarDict["WaterOnImpervious"] << "  "
	//  << folder->VarDict["WaterOnTree"] << "  " <<
	//  folder->VarDict["WaterOnShortVeg"] << "  " <<
	//  folder->VarDict["Cumfil"] << "  " <<
	//  folder->VarDict["FilExcessQ"] << "  " <<
	//  folder->VarDict["RootZoneStorageD"] << "  " <<
	//  folder->VarDict["OverlandQ"] << "  " <<
	//  folder->VarDict["localSMD"] << "  " <<
	//  folder->VarDict["uszStorage"] << "  " <<
	//  folder->VarDict["TP"] << "  " <<
	//  folder->VarDict["ponding"] << "  " <<
	//  folder->VarDict["ImperDirRunoff"] << endl;

//	outfile << folder->VarDict["waterOnImperviousArea"] << " ";
	outfile << folder->VarDict["impDepStor_m"] << " ";
	outfile << folder->VarDict["treeStoredW"] << "  ";
	outfile << folder->VarDict["shortVegStoredW"] << "  ";
	outfile << folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] << "  ";
	outfile << folder->VarDict["Runoff_PervInfilEx_m"] << "  ";
	outfile << folder->VarVecDict["Storage_unUsed_RootZone_TI_m"].at(0) << "  ";
	outfile << folder->VarVecDict["Runoff_SatExcess_TI_m"].at(0) << "  ";
	outfile << folder->VarVecDict["Storage_Max_UnsatZone_TI_m"].at(0) << "  ";
	outfile << folder->VarVecDict["Storage_Used_UnsatZone_TI_m"].at(0) << "  ";
	outfile << folder->VarDict["pondTime"] << "  ";
	outfile << folder->VarDict["ponding"] << "  ";
	outfile << folder->VarDict["ImperDirRunoff"] << endl;
	outfile.close();
}


void TemperatureOutputWriter::createBlockGroupHourlyFile(std::string outDirectory, int blockGroupIndex)
{
	std::string str = "BlockGroup" + std::to_string(blockGroupIndex) + ".txt";
	ofstream outfile(outDirectory + "\\" + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Could not create BlockGroup" + std::to_string(blockGroupIndex) + ".txt");
		return;
	}

	outfile << "YYYYMMDDHH    " << "Tair_K    " << "Tdew_K" << endl;

	outfile.close();

}

void TemperatureOutputWriter::writeToBlockGroupHourlyFile(std::string outDirectory, std::string date, DataFolder* blockGroupFolder)
{
	int bgId = static_cast<int>(blockGroupFolder->ParamDict["blockGroupIndex"]);
	std::string str = "BlockGroup" + std::to_string(bgId) + ".txt";

	ofstream outfile(outDirectory + "//" + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write " + str + " file" << std::endl;
		LogMessage::log("Could not write to " + str);
		return;
	}

	outfile << date << "   " << blockGroupFolder->VarDict["Tair_K"] / blockGroupFolder->ParamDict["folderCount"] << "  "
		<< blockGroupFolder->VarDict["Tdew_K"] / blockGroupFolder->ParamDict["folderCount"] << endl;

}

void TemperatureOutputWriter::writeToBlockGroupDailyFile(std::string outDirectory, std::string date, DataFolder* blockGroupFolder, int timeStep)
{
	int bgId = static_cast<int>(blockGroupFolder->ParamDict["blockGroupIndex"]);
	std::string str = "BlockGroupDaily" + std::to_string(bgId) + ".txt";
	ofstream outfile(outDirectory + "//" + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write " + str + " file" << std::endl;
		LogMessage::log("Cannot write " + str);
		return;
	}
	int avgDenom = (timeStep != 23) ? 24 : 23;

	//HeatIndex2 terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009)
	//HeatIndex2bAvg calculated with daily average Tair and Tdew
	//HeatIndex2Avg is daily average heat index calculated as average of hourly heat index values, and is within 0.1% of HeatIndex2bAvg 
	double HeatIndex2bAvg = -2.653 + 0.994 * ((blockGroupFolder->VarDict["AirTAvg"] / avgDenom) - 273.15) + 0.0153 * pow(((blockGroupFolder->VarDict["TdAvg"] / avgDenom) - 273.15), 2);
	outfile << date << "   "
		<< blockGroupFolder->VarDict["AirTMax"] << "  "
		<< blockGroupFolder->VarDict["AirTMin"] << "  "
		<< blockGroupFolder->VarDict["AirTAvg"] / avgDenom << "  "
		<< blockGroupFolder->VarDict["TdAvg"] / avgDenom << "  "
		<< blockGroupFolder->VarDict["HeatIndexMax"] << "  "
		<< blockGroupFolder->VarDict["HeatIndexAvg"] / avgDenom << "  "
		<< blockGroupFolder->VarDict["HeatIndex2Max"] << "  "
		<< blockGroupFolder->VarDict["HeatIndex2Avg"] / avgDenom << "  "
		<< (HeatIndex2bAvg + 273.15) << "  "
		<< blockGroupFolder->VarDict["WindChillMin"] << "  "
		<< blockGroupFolder->VarDict["WindChillAvg"] / avgDenom << endl;

	outfile.close();

}

void TemperatureOutputWriter::createBlockGroupDailyFile(std::string outDirectory, int blockGroupIndex)
{
	std::string str = "BlockGroupDaily" + std::to_string(blockGroupIndex) + ".txt";
	ofstream outfile(outDirectory + "\\" + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to BlockGroupDaily" + std::to_string(blockGroupIndex) + ".txt");
		return;
	}

	//HeatIndex terms are from Rothfusz, Lans P. National Weather Service Technical Attachment SR 90-23 (1990) 
	//HeatIndex2 terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009)
	//HeatIndex2Avg_Lin is daily average heat index calculated as average of hourly heat index values
	//HeatIndex2bAvg_Lin calculated with daily average Tair and Tdew
	outfile << "YYYYMMDD    " << "RawTairMax_K    "	<< "RawTairMin_K    " << "RawTairAvg_K    "
		<< "RawTdewAvg_K    " << "HeatIndexMax_NWS_K "	<< "HeatIndexAvg_NWS " << "HeatIndex2Max_Lin_K "
		<< "HtIdx2AvgDlyHI_Lin_K " << "HtIdx2DlyAvgTaTd_Lin_K " << "WindchillMin_K    "	<< "WindchillAvg_K" 
		<< endl;

	outfile.close();

}

void TemperatureOutputWriter::createHeatRowColFile(int i, int j, std::string outDirectory) {
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Heat.txt");
	ofstream outfile(outDirectory + str);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot create " + outDirectory + str);
		return;
	}
	//	std::cout<<"Writing outputs.txt file\n"<<std::endl;


	outfile << "ts    " << "Tair_K    " << "Tdew_K   " << "AbsHumidity_kg_p_m3  "
		<< "ImpNR_W_p_m2      " << "ImpH_W_p_m2    " << "ImpLE_W_p_m2    " << "ImpDeltQ_W_p_m2   " 
		<< "TreeNR_W_p_m2     " << "TreeH_W_p_m2   " << "TreeLE_W_p_m2   " << "TreeLEE_W_p_m2    " << "TreeLET_W_p_m2    " << "TreeDeltQ_W_p_m2  "
		<< "ShortVegNR_W_p_m2 " << "ShortH_W_p_m2  " << "ShortLE_W_p_m2  " << "ShortLEE_W_p_m2   " << "ShortLET_W_p_m2   " << "ShortDeltQ_W_p_m2 "
		<< "SoilNR_W_p_m2     " << "SoilH_W_p_m2   " << "SoilLE_W_p_m2   " << "SoilDeltQ_W_p_m2  "
		<< "WaterNR_W_p_m2    " << "WaterH_W_p_m2  " << "WaterLE_W_p_m2  " << "WaterDeltQ_W_p_m2 "
		<< "NR_W_p_m2         " << "H_total_W_p_m2       " << "LE_total_W_p_m2       " << "DeltaQ_W_p_m2" << endl;
	outfile.close();
}

void TemperatureOutputWriter::generateRowColFiles(DataOrganizer* organizer, Inputs* input)
{
	for (int locationIndex = 0; locationIndex < organizer->DataDrawers.size(); ++locationIndex)
	{
		for (int folderIndex = 0; folderIndex < organizer->DataDrawers[locationIndex].size(); ++folderIndex)
		{
			int row = (int)(locationIndex / (int)(input->LocationParams["nCols"]) + 1);
			int col = (locationIndex % (int)(input->LocationParams["nCols"]) + 1);
			if (organizer->DataDrawers[locationIndex][folderIndex]->ParamStringDict["Type"] == "NODATA")
			{
				continue;
			}
			bool locationDataIsToBeOutputted = input->TemperatureCalculationParams["PrintAllRowColOutput"];
			if (!locationDataIsToBeOutputted)
				locationDataIsToBeOutputted = input->LocationIdsToPrintTempOutput.find(locationIndex) != input->LocationIdsToPrintTempOutput.end();
			if (locationDataIsToBeOutputted)
			{
				createHeatRowColFile(row, col, input->SimulationStringParams["OutputDirectory"] + "\\");
				createHydroRowColFile(row, col, input->SimulationStringParams["OutputDirectory"] + "\\", input);
			}
		}
	}
}




void TemperatureOutputWriter::writeToHeatRowColFile(int timeStep, int i, int j, DataFolder* folder, std::string outDirectory)
{
	i -= 1;
	j -= 1;
	char str[40];
	strcpy(str, "\\Row");
	char str1[10];
	itoa(i, str1, 10);
	strcat(str, str1);
	strcat(str, "Col");
	char str2[10];
	itoa(j, str2, 10);
	strcat(str, str2);
	strcat(str, "Heat.txt");
	ofstream outfile(outDirectory + str, ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write heat outputs.txt file" << std::endl;
		LogMessage::log("Cannot write to " + outDirectory + str);
		return;
	}

	//write variables to cell-based output
	outfile << timeStep << "   "
		<< folder->VarDict["Tair_K"] << "  "
		<< folder->VarDict["Tdew_K"] << "   "
		<< folder->VarDict["AbsHumidity_kg_p_m3"] << "   "

		<< folder->VarDict["ImpNR_W_p_m2"] << "  "
		<< folder->VarDict["ImperviousH_W_p_m2"] << "  "
		<< folder->VarDict["ImperviousLE_W_p_m2"] << "  "
		<< folder->VarDict["ImpDeltaQ_W_p_m2"] << "  "

		<< folder->VarDict["TreeNR_W_p_m2"] << "  "
		<< folder->VarDict["TreeH_W_p_m2"] << "  "
		<< folder->VarDict["TreeLE_W_p_m2"] << "  "
		<< folder->VarDict["TreeLEE_W_p_m2"] << "  "
		<< folder->VarDict["TreeLET_W_p_m2"] << "  "
		<< folder->VarDict["TreeDeltaQ_W_p_m2"] << "  "

		<< folder->VarDict["ShortVegNR_W_p_m2"] << "  "
		<< folder->VarDict["ShortVegH_W_p_m2"] << "  "
		<< folder->VarDict["ShortVegLE_W_p_m2"] << "  "
		<< folder->VarDict["ShortVegLEE_W_p_m2"] << "  "
		<< folder->VarDict["ShortVegLET_W_p_m2"] << "  "
		<< folder->VarDict["ShortVegDeltaQ"] << "  "

		<< folder->VarDict["SoilNR_W_p_m2"] << "  "
		<< folder->VarDict["SoilH_W_p_m2"] << "  "
		<< folder->VarDict["SoilLE_W_p_m2"] << "  "
		<< folder->VarDict["SoilDeltaQ_W_p_m2"] << "  "

		<< folder->VarDict["WaterNR_W_p_m2"] << "  "
		<< folder->VarDict["WaterH_W_p_m2"] << "  "
		<< folder->VarDict["WaterLE_W_p_m2"] << "  "
		<< folder->VarDict["WaterDeltaQ_W_p_m2"] << "  "

		<< folder->VarDict["NR_total_W_p_m2"] << "  "
		<< folder->VarDict["H_total_W_p_m2"] << "  "
		<< folder->VarDict["LE_total_W_p_m2"] << "  "
		<< folder->VarDict["DeltaQ_W_p_m2"]  << "  "
		<< endl;

	outfile.close();
}

void TemperatureOutputWriter::createTimeStepResultFile(std::string outputVarName, int ts, Inputs* input)
{


	std::string fName = input->SimulationStringParams["OutputDirectory"] + "\\" + outputVarName + "_" + std::to_string(ts) + ".txt";
	std::ofstream outfile(fName);
	if (!outfile.good())
	{
		std::cout << "Cannot write file: " + fName << std::endl;
		LogMessage::log("Cannot write file: " + fName);
		return;
	}

	outfile << "ncols" << std::setw(20) << input->LocationParams["nCols"] << endl;
	outfile << "nrows" << std::setw(20) << input->LocationParams["nRows"] << endl;
	outfile << "xllcorner" << std::setw(20) << input->LocationParams["xllcorner"] << endl;
	outfile << "yllcorner" << std::setw(20) << input->LocationParams["yllcorner"] << endl;
	outfile << "cellsize" << std::setw(16) << input->LocationParams["cellsize"] << endl;
	outfile << "NODATA_value" << std::setw(15) << input->LocationParams["NODATA"] << endl;
	outfile.close();
}

void TemperatureOutputWriter::writeToTimeStepResultFile(std::string outputVarName, int ts, std::string& outDirectory, DataOrganizer* organizer, Inputs* input)
{
	std::string fName = outDirectory + "\\" + outputVarName + "_" + std::to_string(ts) + ".txt";
	std::ofstream outfile(fName, std::ios::app);
	if (!outfile.good())
	{
		std::cout << "Cannot write file: " + fName << std::endl;
		LogMessage::log("Cannot write file: " + fName);
		return;
	}
	for (int i = 0; i < organizer->DataDrawers.size(); ++i)
	{
		DataFolder* folder = organizer->DataDrawers[i][0];
		int row = (int)(i / (int)(input->LocationParams["nCols"]) + 1);
		int col = (i % (int)(input->LocationParams["nCols"]) + 1);
		bool newLine = (col != (int)(input->LocationParams["nCols"])) ? false : true;
		if (folder->ParamStringDict["Type"] == "NODATA")
		{
			outfile << "-9999";
		}
		else
			outfile << folder->VarDict[outputVarName];
		if (newLine)
		{
			outfile << std::endl;
		}
		else
			outfile << " ";
	}
	outfile.close();
}

void TemperatureOutputWriter::generateBlockFiles(std::map<int, DataFolder*> blockGroupIDToItsLocation, std::string outDirectory, bool needHourlyBGFiles, bool createDailyFiles)
{
	for (std::map<int, DataFolder*>::iterator iter = blockGroupIDToItsLocation.begin(); iter != blockGroupIDToItsLocation.end(); ++iter)
	{
		if (needHourlyBGFiles)
		{
			createBlockGroupHourlyFile(outDirectory, iter->first);
		}
		if (createDailyFiles)
		{
			createBlockGroupDailyFile(outDirectory, iter->first);
		}
	}
}


void TemperatureOutputWriter::outputLandCoverDetails(int index, std::string outDirectory, DataFolder* folder)
{

	string str = outDirectory + std::to_string(index) + "LCScalingDetails.txt";
	std::fstream fs;
	if (folder->ParamDict["TreeP"] == 0 && folder->ParamDict["ImperP"] == 0)
		fs.open(str, std::ios_base::in | std::ios_base::out | std::ios_base::trunc);
	else
		fs.open(str, ios::app);
	fs << "TrP: " << folder->ParamDict["TreeP"]
		<< ",ImpP: " << folder->ParamDict["ImperP"]
		<< ",SoilP: " << folder->ParamDict["SoilP"]
		<< ",ShVgP: " << folder->ParamDict["ShortVegP"]
		<< ",WaterP: " << folder->ParamDict["WaterP"]
		<< ",ImpTr: " << folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"]
		<< ",PeTr: " << folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"]
		//<< ",ShVgFr:" << folder->ParamDict["ShortVegetationCover_frac"]
		//<< ",SoilFr:" << folder->ParamDict["SoilCover_frac"]
		//<< ",DCIA:" << folder->ParamDict["DCIA_frac"]
		<< std::endl;
	fs.close();
}
