#include "ShortVegEvaporationCalc.h"


void ShortVegEvaporationCalc::calculateShortVegEvaporationTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
	//Latent heat of vaporization (J/kg)
	#define lamda 2257000.0
	folder->VarDict["shortVegEvap"] = folder->VarDict["ShortVegLEE_W_p_m2"] / lamda;
	if (folder->VarDict["shortVegEvap"] > folder->VarDict["shortVegStoredW"])
		folder->VarDict["shortVegEvap"] = folder->VarDict["shortVegStoredW"];
	folder->VarDict["shortVegStoredW"] = folder->VarDict["shortVegStoredW"] - folder->VarDict["shortVegEvap"];
	folder->VarDict["total_Evaporation_viaSVeg_m"] += folder->VarDict["shortVegEvap"];
}

void ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	double ShortVegEvapSnow = 0.0;        // Initialize ShortVegEvapSnow (amount of evaporated snow) = 0 
	double ShortVegEvapW = 0.0;           // Initialize ShortVegEvapW (amount of evaporated water) = 0
	double ShortVegEvap = 0.0;                // Initialize ShortVegEP (potential evaporation) = 0

	// ShortVegEvapSnow (the amount of evaporated snow) = PEShortVegSnow (the potential evaporation of snow) 
	ShortVegEvapSnow = input->PETreeSnow[timeStep];
	
	//routine to evaporate liquid water after evaporating any snow water equivalent
	//enter conditional if potential evaporation for snow water exceeds stored snow water
	if (ShortVegEvapSnow > folder->VarDict["shortVegStoredSWE"]) {
		//calcluate fraction of potential evaporation utilzed by snow water evaporation
		double f = folder->VarDict["shortVegStoredSWE"] / ShortVegEvapSnow;
		//evaporate all of the snow water, which may be 0 when ShortVegStoredSWE is 0
		ShortVegEvapSnow = folder->VarDict["shortVegStoredSWE"];
		//convert storMaxTree to value greater than 0 to avoid division by zero
		if (folder->VarDict["storMaxShortVeg"] <= 0) { 
			folder->VarDict["storMaxShortVeg"] = 1.0e-10;
		}
		//evaporate fraction of water stored in shortVeg
		ShortVegEvapW = (1 - f) *input->PETree[timeStep]* pow(folder->VarDict["shortVegStoredW"] / folder->VarDict["storMaxShortVeg"], (2.0 / 3.0));
		if (ShortVegEvapW > folder->VarDict["shortVegStoredW"]) {
			ShortVegEvapW = folder->VarDict["shortVegStoredW"];
		}
		//reduce the shortVeg stored water by the tree evaporation of water
		folder->VarDict["shortVegStoredW"] = folder->VarDict["shortVegStoredW"] - ShortVegEvapW;
	}
	//reduce the shortVeg stored snow by the shortVeg evaporation of snow
	//if shortVegStoredSWE was 0, then ShortVegEvapSnow was set to 0 in the above condition
	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - ShortVegEvapSnow;

	ShortVegEvap = ShortVegEvapSnow + ShortVegEvapW;

	folder->VarDict["shortVegEvap"] = ShortVegEvap;
	folder->VarDict["shortVegEvapW"] = ShortVegEvapW;
	folder->VarDict["shortVegEvapSWE"] = ShortVegEvapSnow;

	folder->VarDict["totalShortVegEvapW"] += folder->VarDict["shortVegEvapW"];
	folder->VarDict["totalShortVegEvapSWE"] += folder->VarDict["shortVegEvapSWE"];
	folder->VarDict["total_Evaporation_viaSVeg_m"] += folder->VarDict["shortVegEvap"];

    folder->VarVecDict["shortVegStoredW"].emplace_back(folder->VarDict["shortVegStoredW"]);
	folder->VarVecDict["shortVegStoredSWE"].emplace_back(folder->VarDict["shortVegStoredSWE"]);
    folder->VarVecDict["shortVegEvapW"].emplace_back(folder->VarDict["shortVegEvapW"]);
	folder->VarVecDict["shortVegEvapSWE"].emplace_back(folder->VarDict["shortVegEvapSWE"]);
	folder->VarVecDict["shortVegEvap"].emplace_back(folder->VarDict["shortVegEvap"]);
	
}





