
#include <iostream>
#include "Inputs\inputs.h"
#include "DataOrganizer.h"
#include "TestSimulation.h"
#include "BuildDataOrganizer.h"
#include <chrono>
#include "LogMessage.h"

typedef std::chrono::high_resolution_clock Clock;


int main(int argc, char* argv[])
{
	try
	{
		//Defining t1 as record of current time with clock function
		auto t1 = Clock::now();
		LogMessage::log("Starting new hydro simulation");
		
		//Command Line Input (CLI) argument is assigned to string called directory; directory should be path to HydroPlusConfig.xml
		std::string directory = argv[1];

		//Calling Inputs and sending CLI argument directory path to HydroPlusConfig.xml
		Inputs *input = new Inputs(directory);

		//cte 20210917 Can we reconfigure BuildSpatialDataOrganizer to more simply maintain multiple folders within a data drawer
		//Calling DataOrganizer.h function to count DataDrawers?
		DataOrganizer *organizer = new DataOrganizer;

		//Calling BulidDataOrganizer and sending pointers to organizer and input
		BuildDataOrganizer::Go(organizer, input);

		BuildDataOrganizer::SetGIMissingParams(organizer, input);

		//Calling TestSimulation and sending pointers to organizer, input, and directory
		TestSimulation::Go(organizer, input, directory);	

		delete input;
		delete organizer;

		//Defining t2 as record of current time with clock function
		auto t2 = Clock::now();
		//Defining run time variable dur = t2 - t1
		int dur = std::chrono::duration_cast<std::chrono::seconds>(t2 - t1).count();
		std::cout << "Program Runtime: "
			<< dur
			<< "seconds" << std::endl;
		LogMessage::log("Program Runtime secs: "+std::to_string(dur));
		LogMessage::log("Simulation Complete.\n\n");

		//cin.get();
	}
	catch (std::exception ex)
	{
		std::cout << "\n  " << ex.what() << "\n\n";
		std::string s(ex.what());
		LogMessage::log("Error in simulation occurred:\n" + s);
		LogMessage::log("\n\n");
	}
	catch (...)
	{
		std::cout << "Error in simulation run.\n\n";
		LogMessage::log("Error in simulation occurred. No exception message:\n");
		LogMessage::log("\n\n");
	}
}