#include "GI_WaterQuality.h"
#include <iomanip>
#include <string>
#include <fstream>

//Author(s) : Li Zhang 
//lzhan126@syr.edu March 2021

using namespace std;
map<std::string, double> variables;

//Runoff Quantity Continuity 
void GI_WaterQuality::Read_GI_Flux(Inputs* input, DataFolder* folder)
{

	//compute the inflow/outflow volume, by multiplying Area with depth.
	//The cumulative depth were got from GI outputwriter
	variables["Volume_inflow_m3"] = folder->ParamDict["Area"] * folder->VarDict["GI_TotalInflow_q0_m"];
	variables["Volume_surfaceOutflow_m3"] = folder->VarDict["GI_TotalSurfaceOutflow_q1_m"]* folder->ParamDict["Area"];
	variables["Volume_storageDrainFlow_m3"] = folder->VarDict["GI_TotalStorageDrainFlow_q3_m"] * folder->ParamDict["Area"];
	variables["Volume_exfilrationToNativeSoil_m3"] = folder->VarDict["GI_TotalExfilrationToNativeSoil_m"] * folder->ParamDict["Area"];
	
	/*cout << " D: " << folder->VarDict["GI_TotalInflow_q0_m"] << " D1: " << folder->VarDict["GI_TotalSurfaceOutflow_q1_m"] << " D2: " << folder->VarDict["GI_TotalStorageDrainFlow_q3_m"] << " D3: " << folder->VarDict["GI_TotalExfilrationToNativeSoil_m"] << endl;
	cout << " V: " << variables["Volume_inflow_m3"] << " V1: " << variables["Volume_surfaceOutflow_m3"] << " V2: " << variables["Volume_storageDrainFlow_m3"] << " V3: " << variables["Volume_exfilrationToNativeSoil_m3"] << endl;*/
}


void GI_WaterQuality::Removal(Inputs* input, DataFolder* folder)
{	
	
	Read_GI_Flux(input, folder);
	//Part 1: Pollution input mass
	//mass (kg) = V (m3) * concentration (mg/L) /1000.
	
	//for each type of pollutants, create variable name by reading the input header, then compute the input mass.
	for (int i = 0; i < input->headerNames.size(); i ++) {
		std::string varname = "inflow_" + input->headerNames[i] +"_kg";
		variables[varname]= input->headermeanValues[i] * variables["Volume_inflow_m3"] / 1000;
	}

	//Part2: Pollution output - remianing in outflow
	//mass(kg) = V(m3) * concentration(mg / L) / 1000 * (1 - removal ratio / 100).

	//2.1 Compute the pollutant remaining in the Surface outflow.
	for (int i = 0; i < input->headerNames.size(); i++) {
		std::string varname = "surfaceOutflow_" + input->headerNames[i] + "_kg";
		variables[varname] = input->headermeanValues[i] * variables["Volume_surfaceOutflow_m3"] / 1000 * (1 - folder->ParamDict[input->headerNames[i]] / 100);
	}
	//2.2 Compute the pollutant remaining in the storageDrain.
	for (int i = 0; i < input->headerNames.size(); i++) {
		std::string varname = "storageDrain_" + input->headerNames[i] + "_kg";
		variables[varname] = input->headermeanValues[i] * variables["Volume_storageDrainFlow_m3"] / 1000 * (1 - folder->ParamDict[input->headerNames[i]] / 100);
	}
	//2.3 Compute the pollutant remaining in the exfilration.
	for (int i = 0; i < input->headerNames.size(); i++) {
		std::string varname = "exfilration_" + input->headerNames[i] + "_kg";
		variables[varname] = input->headermeanValues[i] * variables["Volume_exfilrationToNativeSoil_m3"] / 1000 * (1 - folder->ParamDict[input->headerNames[i]] / 100);
	}

	//Part3: Removal - Pollutant mass removed by GI, computed for each type of outflow
	//mass(kg) = V(m3) * concentration(mg / L) / 1000 * removal ratio / 100.

	//3.1 compute the mass of pollutant removed within surfaceOutflow
	for (int i = 0; i < input->headerNames.size(); i++) {
		std::string varname = "surfaceOutflow_" + input->headerNames[i] + "_removal_kg";
		variables[varname] = input->headermeanValues[i] * variables["Volume_surfaceOutflow_m3"] / 1000 * folder->ParamDict[input->headerNames[i]] / 100;
	}
	//3.2 compute the mass of pollutant removed within storageDrain
	for (int i = 0; i < input->headerNames.size(); i++) {
		std::string varname = "storageDrain_" + input->headerNames[i] + "_removal_kg";
		variables[varname] = input->headermeanValues[i] * variables["Volume_storageDrainFlow_m3"] / 1000 * folder->ParamDict[input->headerNames[i]] / 100;
	}
	//3.2 compute the mass of pollutant removed within exfilration
	for (int i = 0; i < input->headerNames.size(); i++) {
		std::string varname = "exfilration_" + input->headerNames[i] + "_removal_kg";
		variables[varname] = input->headermeanValues[i] * variables["Volume_exfilrationToNativeSoil_m3"] / 1000 * folder->ParamDict[input->headerNames[i]] / 100;
	}


	//Part4: Total removal
	//Total removal = surfaceOutflow_removal_kg + storageDrain_removal_kg + exfilration_removal_kg
	double TotalOutflow_pollutant = 0.0;
	//create variable names, we have 10 pollutant, 3 flow type, it will create 10 * 3 + 3 names, then get/assign the value matches the name within the map
	for (int i = 0; i < input->headerNames.size()-1; i++) {
		std::string varname = input->headerNames[i] + "_removal_kg";
		std::string Surfaceoutflowvarname = "surfaceOutflow_" + input->headerNames[i] + "_removal_kg";
		std::string Drainflowvarname = "storageDrain_" + input->headerNames[i] + "_removal_kg";
		std::string Exfilrationvarname = "exfilration_" + input->headerNames[i] + "_removal_kg";
		TotalOutflow_pollutant = variables[Surfaceoutflowvarname] + variables[Drainflowvarname] + variables[Exfilrationvarname] ;
		variables[varname] = TotalOutflow_pollutant;
	}

	GI_WQ_Outputwriter(input, folder, variables);
}


void GI_WaterQuality::Removal_TS(Inputs* input, DataFolder* folder)
{
	
	//this loop will push back the pollutant input mass, output mass and removed mass for each time step
	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		//create variable names 3 types of flow and 10 types of pollutants 
		for (int j = 0; j < input->headerNames.size(); j++) {
			std::string varname_inflow = "inflow_" + input->headerNames[j] + "_g";

			std::string varname_surfaceflow = "SurfaceOutflow_" + input->headerNames[j] + "_g";
			std::string varname_storagedrain = "StorageDrain_" + input->headerNames[j] + "_g";
			std::string varname_storageexfiltration = "StorageExfiltration_" + input->headerNames[j] + "_g";

			std::string varname_surfaceflow_removal = "SurfaceOutflow_" + input->headerNames[j] + "_g";
			std::string varname_storagedrain_removal = "StorageDrain_" + input->headerNames[j] + "_g";
			std::string varname_storageexfiltration_removal = "StorageExfiltration_" + input->headerNames[j] + "_g";
			std::string varname_total_removal = "TotalRemoval_" + input->headerNames[j] + "_g";

			//input
			folder->VarVecDict[varname_inflow].push_back(folder->VarVecDict["GI_Inflow_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j]);
			//remian in outflow
			folder->VarVecDict[varname_surfaceflow].push_back(folder->VarVecDict["GI_SurfaceOutflow_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j] * (1-folder->ParamDict[input->headerNames[j]] / 100));
			folder->VarVecDict[varname_storagedrain].push_back(folder->VarVecDict["GI_StorageDrainFlow_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j] * (1-folder->ParamDict[input->headerNames[j]] / 100));
			folder->VarVecDict[varname_storageexfiltration].push_back(folder->VarVecDict["GI_StorageExfiltration_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j] * (1-folder->ParamDict[input->headerNames[j]] / 100));

			//removal
			folder->VarVecDict[varname_surfaceflow_removal].push_back(folder->VarVecDict["GI_SurfaceOutflow_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j]* folder->ParamDict[input->headerNames[j]] / 100);
			folder->VarVecDict[varname_storagedrain_removal].push_back(folder->VarVecDict["GI_StorageDrainFlow_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j]*folder->ParamDict[input->headerNames[j]] / 100);
			folder->VarVecDict[varname_storageexfiltration_removal].push_back(folder->VarVecDict["GI_StorageExfiltration_m"][i] * folder->ParamDict["Area"] * input->headermeanValues[j] * folder->ParamDict[input->headerNames[j]] / 100);	
			folder->VarVecDict[varname_total_removal].push_back((folder->VarVecDict["GI_SurfaceOutflow_m"][i] + folder->VarVecDict["GI_StorageDrainFlow_m"][i] + folder->VarVecDict["GI_StorageExfiltration_m"][i]) * folder->ParamDict["Area"] * input->headermeanValues[j] * folder->ParamDict[input->headerNames[j]] / 100);
		}
	}
	GI_WQ_Outputwriter_TS(input, folder);
}



void GI_WaterQuality::GI_WQ_Outputwriter(Inputs* input, DataFolder* folder, std::map<std::string, double>& variables) {
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "GI_WaterQuality.dat");
	if (!outfile.good())
		return;
	else
		//cout << "writing to" << endl;
		//cout << (input->SimulationStringParams["OutputDirectory"] + "GI_WaterQuality.dat") << endl;

	outfile.precision(2);
	outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	std::string surfacevarname_remain;
	std::string drainvarname_remian;
	std::string exfilvarname_remian;
	std::string surfacevarname_removal;
	std::string drainvarname_removal;
	std::string exfilvarname_removal;


	for (int i = 0; i < input->headerNames.size(); i++) {
		outfile << " Type:   "<<input->headerNames[i] <<  endl;
		outfile << " Removal rate:  " << folder->ParamDict[input->headerNames[i]] << " % " << endl;
		std::string inflowvarname = "Inflow_" + input->headerNames[i] + "_kg";
		outfile <<  " Inflow Concentration  :   " << input->headermeanValues[i] <<" mg/L"<< "           Mass:  "<< variables[inflowvarname] <<" kg"<< endl;

		std::string totalremovalvarname = input->headerNames[i] + "_removal_kg";
		outfile << " Outflow Concentration :   " << input->headermeanValues[i] * folder->ParamDict[input->headerNames[i]]/100 << " mg/L" <<"            Total Removal:  " << variables[totalremovalvarname] << " kg" << endl;
		outfile << " -----------------------------------------------------------------------------------" << endl;

		surfacevarname_remain = "surfaceOutflow_" + input->headerNames[i] + "_kg";
		drainvarname_remian = "storageDrain_" + input->headerNames[i] + "_kg";
		exfilvarname_remian = "exfilration_" + input->headerNames[i] + "_kg";
		surfacevarname_removal = "surfaceOutflow_" + input->headerNames[i] + "_removal_kg";
		drainvarname_removal = "storageDrain_" + input->headerNames[i] + "_removal_kg";
		exfilvarname_removal = "exfiltration_" + input->headerNames[i] + "_removal_kg";

		outfile << " Outflow type              Flow               Remaining                   Removal " << endl;
		outfile << " Surface Outflow             " << variables["Volume_surfaceOutflow_m3"]<<" m^3         "<< variables[surfacevarname_remain] << "kg" << "                      " << variables[surfacevarname_removal] << "kg"  << endl;
		outfile << " Storage Drain               " << variables["Volume_storageDrainFlow_m3"] << " m^3         " << variables[drainvarname_remian] << "kg"<<"                      " << variables[drainvarname_removal] << "kg" << endl;
		outfile << " Exfiltration To Native Soil " << variables["Volume_exfiltrationToNativeSoil_m3"] << " m^3         " << variables[exfilvarname_remian] << "kg" << "                      " << variables[exfilvarname_removal] << "kg" << endl;
		outfile << " ===================================================================================" << endl<<endl;
	}
	outfile.close(); 
}

void GI_WaterQuality::GI_WQ_Outputwriter_TS(Inputs* input, DataFolder* folder) {
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "GI_WaterQuality_TS.csv");
	if (!outfile.good())
		return;
	//else
		//cout << "writing to" << endl;
		//cout << (input->SimulationStringParams["OutputDirectory"] + "GI_WaterQuality_TS.csv") << endl;

	outfile.precision(2);
	outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;
	
	for (auto it = input->headerNames.begin(); it != input->headerNames.end(); ++it)
	{	//write the 1st line ,pollutant type
		outfile << *it
			<< "," << " "
			<< "," << " "
			<< "," << " "
			<< "," << " "
			<< "," << " "
			<< "," << " ";
	}
	outfile << std::endl;
	//write the 2nd line, the header
	for (auto it = input->headerNames.begin(); it != input->headerNames.end(); ++it)
	{
		outfile << "Inflow_g"
			<< "," << "SurfaceOutflow_g"
			<< "," << "StorageDrainFlow_g"
			<< "," << "StorageExfiltration_g"
			<< "," << "Removal_g"
			<< "," << " "
			<< "," << " ";
	}
	outfile << std::endl;
	
	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{	//build the var name first, then in each row, write the result for all pollutant type
		for (int j = 0; j < input->headerNames.size(); j++) {
			std::string varname_inflow = "inflow_" + input->headerNames[j] + "_g";

			std::string varname_surfaceflow = "SurfaceOutflow_" + input->headerNames[j] + "_g";
			std::string varname_storagedrain = "StorageDrain_" + input->headerNames[j] + "_g";
			std::string varname_storageexfiltration = "StorageExfiltration_" + input->headerNames[j] + "_g";

			std::string varname_total_removal = "TotalRemoval_" + input->headerNames[j] + "_g"; 
			outfile << folder->VarVecDict[varname_inflow][i]
				<< "," << folder->VarVecDict[varname_surfaceflow][i]
				<< "," << folder->VarVecDict[varname_storagedrain][i]
				<< "," << folder->VarVecDict[varname_storageexfiltration][i]
				<< "," << folder->VarVecDict[varname_total_removal][i]
				<< "," << " "
				<< "," << " ";
		}
		outfile << std::endl;
	}
	outfile.close();
}