#ifndef INPUTTOP_H
#define INPUTTOP_H
//////////////////////////////////////////////////////////////////////////////////////
//																		            //
// InputTopographyData.h - Calculates and/or stores topography data                 //																		   
// ver 1.0                                                                          //
// Language:    Visual C++ 2010                                                     //
// Platform:    MacBookPro5,4  Win7										            //
// Application: HydroPrime/iHydro Version6, Summer 2012                             //
// Programmer:  				            //																	   	
//////////////////////////////////////////////////////////////////////////////////////
/*
Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author V4/V4.5
Yang Yang, yyang31@syr.edu, V5
*/

#include <string>
#include <utility>
#include <map>
#include <vector>
#include <sstream>
#include "IndexCalculator.h"


class TopographyData
{

public:

	//cte 20210520 adding IA to function
	//void init(double N);
	void init(double N, double IA_Land_NoTC_frac, double IA_Land_TC_frac, double IA_Water_frac, double IA_Map_frac);

	//receiving topographic index average, number of topographic index bins, topographic index scaling parameter, power decay parameter, transmissivity at saturation, initial discharge
	double getAveSoilMoistureDeficit(double topAveIndex, double mM, double nN, double T0, double Q0);

	std::vector<double> elevation1D;
	std::vector<double> slope1D;
	std::vector<double> aspect1D;
	std::vector<double> indexArea;
	std::vector<double> index;
	double averageIndex;
	double indexSize;
	int indexAreaSize;
	bool readSlopeAspectFile();
	int getcols () {return nDemDataCols;}
	int getrows () {return nDemDataRows;}
	double getNODATA () {return NODATA;}
	double getxllcorner(){return nDemDataxllcorner;}
	double getyllcorner(){return nDemDatayllcorner;}

	void setcols(int x) { nDemDataCols = x; }
	void setrows(int x) { nDemDataRows = x; }
	void setxllcorner(double x) { nDemDataxllcorner = x; }
	void setyllcorner(double x) { nDemDatayllcorner = x; }

	//TopographData function receives directory, model type, infiltration type, routing type, number of folders, and no data value
	//Before the compound statement that forms the function body of the constructor begins executing, initialization of all direct base
	TopographyData(std::string directory, std::string simulationMode, std::string routingAlgorithm, double numfolders, double simulationNoDataVal, int NAC) :
		directory(directory), simulationMode(simulationMode), routingAlgorithm(routingAlgorithm),
		numfolders(numfolders), simulationNoDataVal(simulationNoDataVal), NAC(NAC) {}

private:

	double simulationNoDataVal;
	std::string directory;
	std::string simulationMode;
	std::string routingAlgorithm;
	int ncols;
	int nrows;
	int nDemDataCols;
	int nDemDataRows;
	double nDemDataxllcorner;
	double nDemDatayllcorner;
	double numfolders;
	double foldersize;
	double subFarea;
	double folderSize;
	double NODATA;

	//N is Power Decay parameter
	double N;
	//NAC is number of topographic index bins
	int NAC;
	
	std::vector<double> FDorganizer_1D;

	std::map<int, std::pair<int, int>> downstreamfoldermap;
	std::map<int, std::pair<double, double>> downstreamProportionmap;

	bool fileExists(std::string path);
	//cte 20210520 adding IA to function
	//bool calcIndexFile(std::string &directory, double N, std::string simulationMode, std::string routingAlgorithm);
	bool calcIndexFile(std::string& directory, double N, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac);
	//cte 20210520 adding IA to function
	//bool calcIndexFile(std::string &directory, std::string simulationMode, std::string routingAlgorithm);
	bool calcIndexFile(std::string& directory, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac);

	//Power Decay: passing directory, N parameter for power decay, NAC number of topographic index bins, simulation mode , routing algorithm
	//cte 20210520 adding IA to function
	//bool calcDistIndexFile(std::string &directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm);
	bool calcDistIndexFile(std::string& directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac);
	//Exponential Decay: passing directory, NAC number of topographic index bins, simulation mode , routing algorithm
	//cte 20210520 adding IA to function
	//bool calcDistIndexFile(std::string &directory, int NAC, std::string simulationMode, std::string routingAlgorithm);
	bool calcDistIndexFile(std::string& directory, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac);

	bool aveIndex();
	bool distAveIndex();
	void calcPropVector();

};

#endif