#include "GIOutputWriter.h"

//Continuity maintained when adjusting to folder area from cover type area (e.g., pervious or impervious). 
//		Depth_adjusted_CoverType_m = Depth_unadjusted_CoverType_m * Area_CoverType_fraction = Depth_unadjusted_CoverType_m * Area_CoverType_m2 / Area_Folder_m2 
//		Adjustment allows depths reported for folder (e.g., bulk area, GI, etc.) to balance with total inflow 
//		For a folder with Area_Pervious_fraction = 0.8 (fully saturated) and Area_Impervious_fraction = 0.2, each generating Depth_unadjusted_CoverType_m = 0.05 of runoff. 
//		Depth_adjusted_Pervious_m = Depth_unadjusted_Pervious_m * Area_Pervious_fraction and Depth_adjusted_Impervious_m = Depth_unadjusted_Impervious_m * Area_Impervious_fraction

void GIOutputWriter::writeVegCanopyFile_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_m"
		<< "," << "SWE_m"
		<< "," << "Intercept_TC_W_m"
		<< "," << "Intercept_TC_SWE_m"
		<< "," << "Evap_TC_W_m"
		<< "," << "Evap_TC_SWE_m"
		<< "," << "LeafStore_TC_W_m"
		<< "," << "LeafStore_TC_SWE_m"
		<< "," << "ThroughFall_TC_W_m"
		<< "," << "ThroughFall_TC_SWE_m"
		<< "," << "TreeCanopy_WB__m"
		<< "," << "Rain_m"
		<< "," << "SWE_m"
		<< "," << "Intercept_SV_W_m"
		<< "," << "Intercept_SW_SWE_m"
		<< "," << "Evap_SV_W_m"
		<< "," << "Evap_SV_SWE_m"
		<< "," << "LeafStore_SV_W_m"
		<< "," << "LeafStore_SV_SWE"
		<< "," << "ThroughFall_SV_W_m"
		<< "," << "ThroughFall_SV_SWE_m"
		<< "," << "ShortVegCanopy_WB_m"
		<< std::endl;


	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
         
		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(0) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) -
				(folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) -
				(folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) -
				(folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]);

			shortveg_balance = (input->Rain[i] + input->Snow[i]) +
				(folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) -
				(folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) -
				(folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) -
				(folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]);
		}




		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["treeIntW"][i]
			<< "," << folder->VarVecDict["treeIntSWE"][i]
			<< "," << folder->VarVecDict["treeEvapW"][i]
			<< "," << folder->VarVecDict["treeEvapSWE"][i]
			<< "," << folder->VarVecDict["treeStoredW"][i]
			<< "," << folder->VarVecDict["treeStoredSWE"][i]
			<< "," << folder->VarVecDict["treeThroughW"][i]
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i]
			<< "," << tree_balance
			<< "," << input->Rain[i]
			<< "," << input->Snow[i]
			<< "," << folder->VarVecDict["shortVegIntW"][i]
			<< "," << folder->VarVecDict["shortVegIntSWE"][i]
			<< "," << folder->VarVecDict["shortVegEvapW"][i]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i]
			<< "," << folder->VarVecDict["shortVegStoredW"][i]
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i]
			<< "," << folder->VarVecDict["shortVegThroughW"][i]
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i]
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeVegCanopyFile_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "VegCanopyFile_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double tree_balance = 0.0;
	double shortveg_balance = 0.0;

	double TotalTreeCover_m2 = folder->ParamDict["TotalTreeCover"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_m3"
		<< "," << "SWE_m3"
		<< "," << "Rain_TC_m3"
		<< "," << "SWE_TC_m3"
		<< "," << "Intercept_TC_W_m3"
		<< "," << "Intercept_TC_W_m3"
		<< "," << "Evap_TC_W_m3"
		<< "," << "Evap_TC_SWE_m3"
		<< "," << "LeafStore_TC_W_m3"
		<< "," << "LeafStore_TC_SWE_m3"
		<< "," << "ThroughFall_TC_W_m3"
		<< "," << "ThroughFall_TC_SWE_m3"
		<< "," << "TreeCanopy_WB__m3"
		<< "," << "Rain_SV_m3"
		<< "," << "SWE_SV_m3"
		<< "," << "Intercept_SV_W_m3"
		<< "," << "Intercept_SW_SWE_m3"
		<< "," << "Evap_SV_W_m3"
		<< "," << "Evap_SV_SWE_m3"
		<< "," << "LeafStore_SV_W_m3"
		<< "," << "LeafStore_SV_SWE"
		<< "," << "ThroughFall_SV_W_m3"
		<< "," << "ThroughFall_SV_W_m3"
		<< "," << "ShortVegCanopy_WB_m3"
		<< std::endl;


	for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		if (i == 0)
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				(0) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				(0) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Evap - Throughfall - current stored = 0 
			tree_balance = ((input->Rain[i] + input->Snow[i]) * TotalTreeCover_m2) +
				((folder->VarVecDict["treeStoredW"][i-1] + folder->VarVecDict["treeStoredSWE"][i-1]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeEvapW"][i] + folder->VarVecDict["treeEvapSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeThroughW"][i] + folder->VarVecDict["treeThruFlowSWE"][i]) * TotalTreeCover_m2) -
				((folder->VarVecDict["treeStoredW"][i] + folder->VarVecDict["treeStoredSWE"][i]) * TotalTreeCover_m2);

			shortveg_balance = ((input->Rain[i] + input->Snow[i]) * ShortVegCover_m2) +
				((folder->VarVecDict["shortVegStoredW"][i-1] + folder->VarVecDict["shortVegStoredSWE"][i-1]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegEvapW"][i] + folder->VarVecDict["shortVegEvapSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegThroughW"][i] + folder->VarVecDict["shortVegThroughSWE"][i]) * ShortVegCover_m2) -
				((folder->VarVecDict["shortVegStoredW"][i] + folder->VarVecDict["shortVegStoredSWE"][i]) * ShortVegCover_m2);
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * TotalTreeCover_m2
			<< "," << input->Snow[i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeIntSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeStoredSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TotalTreeCover_m2
			<< "," << tree_balance
			<< "," << input->Rain[i] * ShortVegCover_m2
			<< "," << input->Snow[i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegIntSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegStoredSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << shortveg_balance
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeWaterOnPerAreaFile_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnPerAreaFile_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreePerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	double ShortVegCover_frac = folder->ParamDict["ShortVegetationCover_frac"];
	double BareSoilCover_frac = folder->ParamDict["SoilCover_frac"];
	double TotalPerviousCover_frac = BareSoilCover_frac + ShortVegCover_frac + TreePerviousCover_frac;

	double PerArea_WB = 0.0;
	double PerArea_SWEB = 0.0;
	double PerArea_comb_WB = 0.0;
	double PerArea_Flow_To_Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_m"
		<< "," << "Rain_BS_m"
		<< "," << "ThroughFlow_TCPA_W_m"
		<< "," << "ThroughFlow_SV_W_m"
		<< "," << "SnowMelt_TCPA_m"
		<< "," << "SnowMelt_SV_m"
		<< "," << "SnowMelt_BS_m"
		<< "," << "Water_PA_m"
		<< "," << "PerArea_WB_m"
		<< "," << "SWE_m"
		<< "," << "SWE_BareSoil_m"
		<< "," << "ThroughFlow_TCPA_SWE_m"
		<< "," << "ThroughFlow_SV_SWE_m"
		<< "," << "SWE_on_TCPA_m"
		<< "," << "SWE_on_SV_m"
		<< "," << "SWE_on_BS_m"
		<< "," << "SnowSub_TCPA_m"
		<< "," << "SnowSub_SV_m"
		<< "," << "SnowSub_BS_m"
		<< "," << "PerArea_SWEB_m"
		<< "," << "PerArea_comb_WB_m"
		<< "," << "Water_PA_m"
		<< "," << "Run_on_from_IA_m"
		<< "," << "Inflow_to_PerDep_m"
		<< "," << "PerDepStor_PA_m"
		<< "," << "PerDepEvap_PA_m"
		<< "," << "PerFlow_To_Infil_m"
		<< "," << "PA_Flow_To_Infil_WB_m"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{
			PerArea_WB = (input->Rain[i]  * BareSoilCover_frac) +							// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_frac) +			// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_frac) +			// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) +				// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) +		// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) - 	// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac);		// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (0) +															// previously stored SWE BS
				(0) +																		// previously stored SWE TCPA
				(0) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_frac) +										// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_frac) + 		// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_frac) + 		// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_frac) -					// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_frac) -			// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_frac) -			// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) -				// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) -		// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) -		// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_frac) -				// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_frac) -		// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_frac);		// Loss of SnowSub TSV
			
			// combined WB
			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											

			//impRunoffToSoil_adj is depth on pervious area, take product with perCover_frac to get fractional runoff depth from catchment
			PerArea_Flow_To_Infil_WB = (folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac) +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_frac +	
				(0) - 																// Previous PerDep sTorage
				(folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_frac) -																		// Loss to Dep Evap
				(folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_frac) -																	// Current PerDep Stor
				(folder->VarVecDict["Water_forSoilRecharge_m"][i] * TotalPerviousCover_frac);																	// flow to infiltration routine
		}
		else
		{

			PerArea_WB = (input->Rain[i] * BareSoilCover_frac) +						// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_frac) +		// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_frac) +		// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) +			// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) +	// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) - // Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac);	// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * BareSoilCover_frac) +	// previously stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i-1] * TreePerviousCover_frac) +	// previously stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i-1] * ShortVegCover_frac) +	// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_frac) +									// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_frac) + 	// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_frac) - 	// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_frac) -				// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_frac) -		// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_frac) -		// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac) -			// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac) -	// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac) -	// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_frac) -			// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_frac) -	// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_frac);	// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;								// combined WB

			//impRunoffToSoil_adj is depth on pervious area, take product with perCover_frac to get fractional runoff depth from catchment
			PerArea_Flow_To_Infil_WB = (folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac) +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_frac +																// Added water from impervious run-on
				(folder->VarVecDict["perDepStor"][i-1] * TotalPerviousCover_frac) - 																	// Previous PerDep sTorage
				(folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_frac) -																		// Loss to Dep Evap
				(folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_frac) -																		// Current PerDep Stor
				(folder->VarVecDict["Water_forSoilRecharge_m"][i] * TotalPerviousCover_frac);																	// flow to infiltration routine
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i]
			<< "," << input->Rain[i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac
			<< "," << PerArea_WB
			<< "," << input->Snow[i]
			<< "," << input->Snow[i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["snowOpen"][i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_frac
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_frac
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_frac
			<< "," << folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_frac
			<< "," << PerArea_SWEB
			<< "," << PerArea_comb_WB
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_frac
			<< "," << folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_frac
			<< "," << folder->VarVecDict["waterToPerDep"][i] * TotalPerviousCover_frac	
			<< "," << folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_frac					//Time series, use instantaneous value, or compare between two time steps, should not be summed in analysis
			<< "," << folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_frac
			<< "," << folder->VarVecDict["Water_forSoilRecharge_m"][i] * TotalPerviousCover_frac		// already adjusted by the fraction PerArea 
			<< "," << PerArea_Flow_To_Infil_WB
			<< std::endl;
	}

}

void GIOutputWriter::writeWaterOnPerAreaFile_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnPerAreaFile_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreePerviousCover_m2 = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double BareSoilCover_m2 = folder->ParamDict["SoilCover_frac"] * folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = TreePerviousCover_m2 + ShortVegCover_m2 + BareSoilCover_m2;

	double PerArea_WB = 0.0;
	double PerArea_SWEB = 0.0;
	double PerArea_comb_WB = 0.0;
	double PerArea_Flow_To_Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_m3"
		<< "," << "Rain_BS_m3"
		<< "," << "ThroughFlow_TCPA_W_m3"
		<< "," << "ThroughFlow_SV_W_m3"
		<< "," << "SnowMelt_TCPA_m3"
		<< "," << "SnowMelt_SV_m3"
		<< "," << "SnowMelt_BS_m3"
		<< "," << "Water_PA_m3"
		<< "," << "PerArea_WB_m3"
		<< "," << "SWE_m3"
		<< "," << "SWE_BareSoil_m3"
		<< "," << "ThroughFlow_TCPA_SWE_m3"
		<< "," << "ThroughFlow_SV_SWE_m3"
		<< "," << "SWE_on_TCPA_m3"
		<< "," << "SWE_on_SV_m3"
		<< "," << "SWE_on_BS_m3"
		<< "," << "SnowSub_TCPA_m3"
		<< "," << "SnowSub_SV_m3"
		<< "," << "SnowSub_BS_m3"
		<< "," << "PerArea_SWEB_m3"
		<< "," << "PerArea_comb_WB_m3"
		<< "," << "Water_PA_Adj_m3"
		<< "," << "Run_on_from_IA_m3"
		<< "," << "Inflow_to_PerDep_m3"
		<< "," << "PerDepStor_PA_m3"
		<< "," << "PerDepEvap_PA_m3"
		<< "," << "PerFlow_To_Infil_m3"
		<< "," << "PA_Flow_To_Infil_WB_m3"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{
			PerArea_WB = (input->Rain[i]  * BareSoilCover_m2) +									// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_m2) +									// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2) +									// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) +									// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) +								// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) - 								// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2);								// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (0) +															// previously stored SWE BS
				(0) +																		// previously stored SWE TCPA
				(0) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_m2) +													// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_m2) + 										// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2) + 										// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_m2) -												// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) -											// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) -										// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) -										// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_m2) -											// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_m2) -										// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_m2);										// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											// combined WB

			PerArea_Flow_To_Infil_WB = folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2 +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_m2 +																	// Added water from impervious run-on
				(0) - 																				// Previous PerDep sTorage
				folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_m2 -																		// Loss to Dep Evap
				folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_m2 -																	// Current PerDep Stor
				folder->VarVecDict["Water_forSoilRecharge_m"][i] * TotalPerviousCover_m2;																	// flow to infiltration routine
		}
		else
		{

			PerArea_WB = (input->Rain[i] * BareSoilCover_m2) +									// added rain to BS
				(folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_m2) +									// added rain to TCPA
				(folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2) +									// added rain to SV
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) +									// Added snowmelt to BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) +								// Added snowmelt to TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) - 								// Added snowmelt to SV
				(folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2);								// Final water on IA - From model, includes snowmelt - have to unweight to match

			PerArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * BareSoilCover_m2) +															// previously stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i-1] * TreePerviousCover_m2) +																		// previously stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i-1] * ShortVegCover_m2) +																		// previously stored SWE SV
				(input->Snow[i] * BareSoilCover_m2) +													// added SWE to BS
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_m2) + 										// added SWE to TCPA
				(folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2) - 										// added SWE to SV
				(folder->VarVecDict["snowOpen"][i] * BareSoilCover_m2) -												// current stored SWE BS
				(folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_m2) -										// current stored SWE TCPA
				(folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2) -											// Loss snowmelt from BS
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2) -										// Loss snowmelt from TCPA
				(folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2) -										// Loss snowmelt from SV
				(folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_m2) -											// Loss of SnowSub BS
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_m2) -										// Loss of SnowSub TCPA
				(folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_m2);										// Loss of SnowSub TSV

			PerArea_comb_WB = PerArea_WB + PerArea_SWEB;											// combined WB

			PerArea_Flow_To_Infil_WB = folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2 +											// Incoming water only (rain + snowmelt) - already weighted
				folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_m2 +																	// Added water from impervious run-on
				folder->VarVecDict["perDepStor"][i-1] * TotalPerviousCover_m2 - 																	// Previous PerDep sTorage
				folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_m2 -																		// Loss to Dep Evap
				folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_m2 -																		// Current PerDep Stor
				folder->VarVecDict["Water_forSoilRecharge_m"][i] * TotalPerviousCover_m2;																	// flow to infiltration routine
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * folder->ParamDict["Area"]
			<< "," << input->Rain[i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["shortVegThroughW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["snowMeltUnderShortVeg"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2
			<< "," << PerArea_WB
			<< "," << input->Snow[i] * folder->ParamDict["Area"]
			<< "," << input->Snow[i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["shortVegThroughSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["snowOpen"][i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["snowUnderShortVeg"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * BareSoilCover_m2
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreePerviousCover_m2
			<< "," << folder->VarVecDict["snowEvapUnderShortVeg"][i] * ShortVegCover_m2
			<< "," << PerArea_SWEB
			<< "," << PerArea_comb_WB
			<< "," << folder->VarVecDict["waterOnPerArea"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["impRunoffToSoil_adj"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["waterToPerDep"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["perDepStor"][i] * TotalPerviousCover_m2		
			<< "," << folder->VarVecDict["perDepEvap"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["Water_forSoilRecharge_m"][i] * TotalPerviousCover_m2				
			<< "," << PerArea_Flow_To_Infil_WB
			<< std::endl;
	}

}

void GIOutputWriter::writeWaterOnImpAreaFile_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnImpAreaFile_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreeImperviousCover_frac = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double NoTreeImperviousCover_frac = folder->ParamDict["SurfaceWaterIA_frac"] + folder->ParamDict["ImperviousCover_frac"];
	double TotalImperviousCover_frac = TreeImperviousCover_frac + NoTreeImperviousCover_frac;

	double ImpArea_WB = 0.0;
	double ImpArea_SWEB = 0.0;
	double ImpArea_comb_WB = 0.0;
	double ImpArea_Runoff_WB = 0.0;
	double total_IA_W_SWE = 0.0;
	double in = 0.0;
	double out = 0.0;
	double loss = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_OIA_m"
		<< "," << "ThroughFlow_Tree_W_TCIA_m"
		<< "," << "SnowMelt_OIA_m"
		<< "," << "SnowMelt_TCIA_m"
		<< "," << "Water_IA_m"
		<< "," << "ImpArea_WB_m"
		<< "," << "SWE_OIA_m"
		<< "," << "ThroughFlow_Tree_SWE_TCIA_m"
		<< "," << "SWE_on_OIA_m"
		<< "," << "SWE_on_TCIA_m"
		<< "," << "SnowSub_OIA_m"
		<< "," << "SnowSub_TCIA_m"
		<< "," << "ImpArea_SWEB_m"
		<< "," << "ImpArea_comb_WB_m"
		<< "," << "ImpDepStor_IA_m"
		<< "," << "ImpDepEvap_IA_m"
		<< "," << "ImperFlow_Soil_m"
		<< "," << "ImperFlow_Outlet_nonRouted_m"
		<< "," << "ImpArea_Runoff_WB_m"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{

			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_frac) +					// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_frac) +		// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) +		// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -	// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac);		// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (0) +															// previously stored SWE OIA
				(0) +																		// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_frac) +								// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_frac) - 	// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_frac) -			// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_frac) -		// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) -		// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -	// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_frac) -		// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_frac);	// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac) +		// Incoming water only (rain + snowmelt) 
				(0) - 																											// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_frac) -				// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_frac) -				// Current ImpDep Stor
				folder->VarVecDict["impRunoffToSoil"][i] -																			// loss to run on to pervious area
				folder->VarVecDict["impRunoffDCIA"][i];																			// loss to DCIA flow to outlet
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Sub - predicted Water on IA - current stored = 0 
			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_frac) +						// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_frac) +			// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) +			// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -		// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac);			// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * NoTreeImperviousCover_frac) +	// previously stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i-1] * TreeImperviousCover_frac) +		// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_frac) +								// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_frac) - 	// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_frac) -			// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_frac) -		// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac) -		// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac) -	// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_frac) -		// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_frac);	// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac) +	// Incoming water only (rain + snowmelt) 
				(folder->VarVecDict["impDepStor"][i-1] * TotalImperviousCover_frac) - 	// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_frac) -		// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_frac) -		// Current ImpDep Stor
				folder->VarVecDict["impRunoffToSoil"][i] -								// loss to run on to pervious area
				folder->VarVecDict["impRunoffDCIA"][i];									// loss to DCIa flow to outlet
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_frac
			<< "," << ImpArea_WB
			<< "," << input->Snow[i]  * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_frac
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_frac
			<< "," << ImpArea_SWEB
			<< "," << ImpArea_comb_WB
			<< "," << folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_frac
			<< "," << folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_frac
			<< "," << folder->VarVecDict["impRunoffToSoil"][i]	//already adjusted by the fraction ImpArea non-DCIA
			<< "," << folder->VarVecDict["impRunoffDCIA"][i]	//already adjusted by the fraction of ImpAreaDCIA
			<< "," << ImpArea_Runoff_WB
			<< std::endl;
	}


}

void GIOutputWriter::writeWaterOnImpAreaFile_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterOnImpAreaFile_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	double TreeImperviousCover_m2 = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"]	* folder->ParamDict["Area"];
	double NoTreeImperviousCover_m2 = (folder->ParamDict["SurfaceWaterIA_frac"] + folder->ParamDict["ImperviousCover_frac"]) * folder->ParamDict["Area"];
	double TotalImperviousCover_m2 = TreeImperviousCover_m2 + NoTreeImperviousCover_m2;

	double ImpArea_WB = 0.0;
	double ImpArea_SWEB = 0.0;
	double ImpArea_comb_WB = 0.0;
	double ImpArea_Runoff_WB = 0.0;
	double total_IA_W_SWE = 0.0;
	double in = 0.0;
	double out = 0.0;
	double loss = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Rain_OIA_m3"
		<< "," << "ThroughFlow_TCIA_W_m3"
		<< "," << "SnowMelt_OIA_m3"
		<< "," << "SnowMelt_TCIA_m3"
		<< "," << "Water_IA_m3"
		<< "," << "ImpArea_WB_m3"
		<< "," << "SWE_OIA_m3"
		<< "," << "ThroughFlow_TCIA_SWE_m3"
		<< "," << "SWE_on_OIA_m3"
		<< "," << "SWE_on_TCIA_m3"
		<< "," << "SnowSub_OIA_m3"
		<< "," << "SnowSub_TCIA_m3"
		<< "," << "ImpArea_SWEB_m3"
		<< "," << "ImpArea_comb_WB_m3"
		<< "," << "ImpDepStor_IA_m3"
		<< "," << "ImpDepEvap_IA_m3"
		<< "," << "ImperFlow_Soil_m3"
		<< "," << "ImperFlow_Outlet_nonRouted_m3"
		<< "," << "ImpArea_Runoff_WB_m3"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		if (i == 0)
		{

			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_m2) +						// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_m2) +			// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) +		// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -		// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2);		// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (0) +															// previously stored SWE OIA
				(0) +																		// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_m2) +								// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_m2) - 		// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_m2) -			// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_m2) -			// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) -		// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -		// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_m2) -		// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_m2);		// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2) +		// Incoming water only (rain + snowmelt) 
				(0) - 																						// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_m2) -			// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_m2) -			// Current ImpDep Stor
				(folder->VarVecDict["impRunoffToSoil"][i] * folder->ParamDict["Area"]) -	// impRunoffToSoil specific to impervious area 
				(folder->VarVecDict["impRunoffDCIA"][i] * folder->ParamDict["Area"]);		// impRunoffDCIA specific to impervious area		}
		}
		else
		{
			// incoming precip + previously stored (0 first TS) - Sub - predicted Water on IA - current stored = 0 
			ImpArea_WB = (input->Rain[i] * NoTreeImperviousCover_m2) +						// added rain to OIA
				(folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_m2) +			// added rain to TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) +		// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -		// Added snowmelt to TCIA
				(folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2);		// Final water on IA - From model, includes snowmelt

			ImpArea_SWEB = (folder->VarVecDict["snowOpen"][i-1] * NoTreeImperviousCover_m2) +	// previously stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i-1] * TreeImperviousCover_m2) +			// previously stored SWE TCIA
				(input->Snow[i] * NoTreeImperviousCover_m2) +									// added SWE to OIA
				(folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_m2) - 			// added SWE to TCIA
				(folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_m2) -				// current stored SWE OIA
				(folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_m2) -				// current stored SWE TCIA
				(folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2) -			// Added snowmelt to OIA
				(folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2) -			// Loss snowmelt from TCIA
				(folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_m2) -			// Loss of SnowSub OIA
				(folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_m2);			// Loss of SnowSub TCIA

			ImpArea_comb_WB = ImpArea_WB + ImpArea_SWEB;								// combined WB

			ImpArea_Runoff_WB = (folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2) +		// Incoming water only (rain + snowmelt) 
				(folder->VarVecDict["impDepStor"][i-1] * TotalImperviousCover_m2) - 						// Previous ImpDep sTorage
				(folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_m2) -							// Loss to Dep Evap
				(folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_m2) -							// Current ImpDep Stor
				(folder->VarVecDict["impRunoffToSoil"][i] * folder->ParamDict["Area"]) -					// impRunoffToSoil specific to impervious area 
				(folder->VarVecDict["impRunoffDCIA"][i] * folder->ParamDict["Area"]);						// impRunoffDCIA specific to impervious area 
		}


		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << input->Rain[i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["treeThroughW"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowMeltOpen"][i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowMeltUnderTree"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["waterOnImpArea"][i] * TotalImperviousCover_m2
			<< "," << ImpArea_WB
			<< "," << input->Snow[i]  * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["treeThruFlowSWE"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowOpen"][i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowUnderTree"][i] * TreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowEvapOpen"][i] * NoTreeImperviousCover_m2
			<< "," << folder->VarVecDict["snowEvapUnderTree"][i] * TreeImperviousCover_m2
			<< "," << ImpArea_SWEB
			<< "," << ImpArea_comb_WB
			<< "," << folder->VarVecDict["impDepStor"][i] * TotalImperviousCover_m2
			<< "," << folder->VarVecDict["impDepEvap"][i] * TotalImperviousCover_m2
			<< "," << folder->VarVecDict["impRunoffToSoil"][i] * folder->ParamDict["Area"]		// impRunoffToSoil specific to impervious area 
			<< "," << folder->VarVecDict["impRunoffDCIA"][i] * folder->ParamDict["Area"]		// impRunoffDCIA specific to impervious area 
			<< "," << ImpArea_Runoff_WB
			<< std::endl;
	}


}

void GIOutputWriter::writeEvap_ET_Infil_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Evap_TC_W_m"
		<< "," << "Evap_TC_SWE_m"
		<< "," << "Evap_SV_W_m"
		<< "," << "Evap_SV_SWE_m"
		<< "," << "Infiltration_PA_m"
		<< "," << "rET_VegCover_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * folder->ParamDict["TotalTreeCover"]
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * folder->ParamDict["TotalTreeCover"]
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * folder->ParamDict["ShortVegetationCover_frac"]
			<< "," << folder->VarVecDict["Infiltration_m"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< "," << folder->VarVecDict["EvapoTranspiration_RootZone_m"][i] * folder->ParamDict["TotalPerviousCover_frac"]
			<< std::endl;
	}
	outfile.close();

}

void GIOutputWriter::writeEvap_ET_Infil_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Evap_ET_Infil_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double TotalTreeCover_m2 = folder->ParamDict["TotalTreeCover"] * folder->ParamDict["Area"];
	double ShortVegCover_m2 = folder->ParamDict["ShortVegetationCover_frac"] * folder->ParamDict["Area"];
	double TotalPerviousCover_m2 = folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["Area"];

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "Evap_TC_W_m3"
		<< "," << "Evap_TC_SWE_m3"
		<< "," << "Evap_SV_W_m3"
		<< "," << "Evap_SV_SWE_m3"
		<< "," << "Infiltration_PA_m3"
		<< "," << "rET_VegCover_m3"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["treeEvapW"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["treeEvapSWE"][i] * TotalTreeCover_m2
			<< "," << folder->VarVecDict["shortVegEvapW"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["shortVegEvapSWE"][i] * ShortVegCover_m2
			<< "," << folder->VarVecDict["Infiltration_m"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["EvapoTranspiration_RootZone_m"][i] * TotalPerviousCover_m2
			<< std::endl;

	}
	outfile.close();
}

void GIOutputWriter::writeWaterInRootZone_VWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_VWB_BulkArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PA_Flow_To_Infil_m"
		<< "," << "Infil_InfilExcess_m"
		<< "," << "Infil_SatExcess_m"
		<< "," << "Infil_Total_m"
		<< "," << "Runoff_InfilExcess_m"
		<< "," << "Runoff_SatExcess_m"
		<< "," << "Runoff_Perv_Total_m"
		<< "," << "RootZone_WB_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = folder->VarVecDict["Infiltration_m"][i] -
			folder->VarVecDict["Infiltration_viaSatEx_m"][i] -
			folder->VarVecDict["Infiltration_viaInfilEx_m"][i];

		RootZone_WB = folder->VarVecDict["Water_forInfiltration_m"][i] -
			(folder->VarVecDict["Infiltration_viaSatEx_m"][i] + folder->VarVecDict["Infiltration_viaInfilEx_m"][i]) -
			(folder->VarVecDict["Runoff_SatExcess_m"][i] + folder->VarVecDict["Runoff_PervInfilEx_m"][i]);

		perAreaRunoff_WB = (folder->VarVecDict["Runoff_SatExcess_m"][i]) +
			(folder->VarVecDict["Runoff_PervInfilEx_m"][i]); 

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["Water_forInfiltration_m"][i]
			<< "," << folder->VarVecDict["Infiltration_viaInfilEx_m"][i]
			<< "," << folder->VarVecDict["Infiltration_viaSatEx_m"][i]
			<< "," << folder->VarVecDict["Infiltration_m"][i]
			<< "," << folder->VarVecDict["Runoff_PervInfilEx_m"][i]
			<< "," << folder->VarVecDict["Runoff_SatExcess_m"][i]
			<< "," << folder->VarVecDict["Runoff_SatExcess_m"][i] + folder->VarVecDict["Runoff_PervInfilEx_m"][i]
			<< "," << RootZone_WB
			<< std::endl;

	}
	outfile.close();

}

void GIOutputWriter::writeWaterInRootZone_FWB_BulkArea(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_FWB_BulkArea.csv");
	if (!outfile.good())
		return;

	double TotalPerviousCover_m2 = folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["Area"];

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PA_Flow_To_Infil_m3"
		<< "," << "Infil_InfilExcess_m3"
		<< "," << "Infil_SatExcess_m3"
		<< "," << "Infil_Total_m3"
		<< "," << "Runoff_InfilExcess_m3"
		<< "," << "Runoff_SatExcess_m3"
		<< "," << "Runoff_Perv_Total_m3"
		<< "," << "RootZone_WB_m3"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		Infil_WB = (folder->VarVecDict["Infiltration_m"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["Infiltration_viaSatEx_m"][i] * TotalPerviousCover_m2) -
			(folder->VarVecDict["Infiltration_viaInfilEx_m"][i] * TotalPerviousCover_m2);

		RootZone_WB = (folder->VarVecDict["Water_forInfiltration_m"][i] * TotalPerviousCover_m2) -
			((folder->VarVecDict["Infiltration_viaSatEx_m"][i] + folder->VarVecDict["Infiltration_viaInfilEx_m"][i]) * TotalPerviousCover_m2) -
			((folder->VarVecDict["Runoff_SatExcess_m"][i] + folder->VarVecDict["Runoff_PervInfilEx_m"][i]) * TotalPerviousCover_m2);

		perAreaRunoff_WB = (folder->VarVecDict["Runoff_SatExcess_m"][i] * TotalPerviousCover_m2) +
			(folder->VarVecDict["Runoff_PervInfilEx_m"][i] * TotalPerviousCover_m2);

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["Water_forInfiltration_m"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["Infiltration_viaInfilEx_m"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["Infiltration_viaSatEx_m"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["Infiltration_m"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["Runoff_PervInfilEx_m"][i] * TotalPerviousCover_m2
			<< "," << folder->VarVecDict["Runoff_SatExcess_m"][i] * TotalPerviousCover_m2
			<< "," << (folder->VarVecDict["Runoff_PervInfilEx_m"][i] + folder->VarVecDict["Runoff_SatExcess_m"][i] ) * TotalPerviousCover_m2
			<< "," << RootZone_WB
			<< std::endl;
	}
	outfile.close();
}

void GIOutputWriter::writeWaterInRootZone_VWB_GIArea(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterInRootZone_VWB_GIArea.csv");
	if (!outfile.good())
		return;

	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	double RootZone_WB = 0.0;
	double perAreaRunoff_WB = 0.0;
	double Infil_WB = 0.0;

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "PA_Flow_To_Infil_m"
		<< "," << "Infil_InfilExcess_m"
		<< "," << "Infil_SatExcess_m"
		<< "," << "Infil_Total_m"
		<< "," << "Runoff_InfilExcess_m"
		<< "," << "Runoff_SatExcess_m"
		<< "," << "Runoff_Perv_Total_m"
		<< "," << "RootZone_WB_m"
		<< std::endl;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		RootZone_WB = folder->VarVecDict["Water_forInfiltration_m"][i] -
			(folder->VarVecDict["Infiltration_viaSatEx_m"][i] + folder->VarVecDict["Infiltration_viaInfilEx_m"][i]) -
			(folder->VarVecDict["Runoff_SatExcess_m"][i] + folder->VarVecDict["Runoff_PervInfilEx_m"][i]);

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["Water_forInfiltration_m"][i]
			<< "," << folder->VarVecDict["Infiltration_viaInfilEx_m"][i]
			<< "," << folder->VarVecDict["Infiltration_viaSatEx_m"][i]
			<< "," << folder->VarVecDict["Infiltration_m"][i]
			<< "," << folder->VarVecDict["Runoff_PervInfilEx_m"][i]
			<< "," << folder->VarVecDict["Runoff_SatExcess_m"][i]
			<< "," << folder->VarVecDict["Runoff_SatExcess_m"][i] + folder->VarVecDict["Runoff_PervInfilEx_m"][i]
			<< "," << RootZone_WB
			<< std::endl;

	}
	outfile.close();

}


void GIOutputWriter::writeWaterBalance_RainGarden(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RainGarden.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeWaterBalance_BioRetention(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_BioRetention.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeWaterBalance_InfilTrench(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_InfilTrench.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeWaterBalance_Swale(Inputs *input, DataFolder *folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_Swale.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

void GIOutputWriter::writeWaterBalance_PermeablePavement(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_PermeablePavement.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		<< "," << "GI_PavementPercolation_f4_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_PavementPercolation_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

//Li: add waterbalance output for green roof
void GIOutputWriter::writeWaterBalance_GreenRoof(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_GreenRoof.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

//Li: added waterbalance output for rooftop disconnection
void GIOutputWriter::writeWaterBalance_RoofDiscon(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RoofDiscon.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}


//Li:added waterbalance output for rain barrels
void GIOutputWriter::writeWaterBalance_RainBarrel(Inputs* input, DataFolder* folder)
{
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RainBarrel.csv");
	if (!outfile.good())
		return;


	outfile.precision(15);
	//outfile.setf(ios::fixed);
	outfile.setf(ios::showpoint);

	if (!outfile.good())
		return;

	//Li 10242020 get folder->VarDict["GI_SurfaceWaterBudget_m"] & folder->VarDict["GI_TotalWaterBudget_m"]
	//GIOutputWriter::waterbudget(input, folder);

	outfile << "yyyymmdd"
		<< "," << "Hr:Min:sec"
		<< "," << "GI_Inflow_q0_m"
		<< "," << "GI_SurfaceInfil_f1_m"
		<< "," << "GI_SoilPercolation_f2_m"
		<< "," << "GI_StorageExfil_f3_m"
		<< "," << "GI_SurfaceOutflow_q1_m"
		<< "," << "GI_StorageDrain_q3_m"
		<< "," << "GI_PavementDepth_d4_m"
		<< "," << "GI_SurfaceDepth_d4_m"
		<< "," << "GI_SoilMoisture_d2_m3_p_m3"
		<< "," << "GI_StorageDepth_d3_m"
		//Added by Li to show the total Evap
		<< "," << "GI_EvaporationAllLayers_m"
		<< "," << "GI_WaterTableDepth_m"
		<< "," << "upsat2_m"
		<< "," << "upsat3_m"
		<< "," << "upsat4_m"
		<< std::endl;


	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{

		outfile << input->CalendarDates[i]
			<< "," << input->TimeSeries[i]
			<< "," << folder->VarVecDict["GI_Inflow_m"][i]
			<< "," << folder->VarVecDict["GI_Infiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SoilPercolation_m"][i]
			<< "," << folder->VarVecDict["GI_StorageExfiltration_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceOutflow_m"][i]
			<< "," << folder->VarVecDict["GI_StorageDrainFlow_m"][i]
			<< "," << folder->VarVecDict["GI_PavementStorageDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SurfaceDepth_m"][i]
			<< "," << folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i]
			<< "," << folder->VarVecDict["GI_StorageDepth_m"][i]
			//Added by Li to show the total Evap
			<< "," << folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i]
			<< "," << folder->VarVecDict["GI_WaterTableDepth_m"][i]
			<< "," << folder->VarVecDict["upSat_2"][i]
			<< "," << folder->VarVecDict["upSat_3"][i]
			<< "," << folder->VarVecDict["upSat_4"][i]
			<< endl;
	}

	outfile.close();

}

//Li 10242020  check water budget
void GIOutputWriter::waterbudget(Inputs* input, DataFolder* folder) {
	double GI_TotalInflow_q0_m = 0.0;

	double GI_TotalStorageDrainFlow_q3_m = 0.0;
	double GI_TotalSurfaceOutflow_q1_m = 0.0;
	double GI_TotalExfilrationToNativeSoil_f2_or_f3_m = 0.0;
	double GI_TotalInfiltration_f1_m = 0.0;

	double GI_DeltaStorageDepth_d3_m = 0.0;
	double GI_DeltaSoilMoisture_d2_m3_p_m3 = 0.0;
	double GI_DeltaSurfaceDepth_d1_m = 0.0;
	double GI_PavementDepth_d4_m = 0.0;

	double GI_TotalEvaporationAllLayers_m = 0.0;

	for (int i = 0; i < input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		GI_TotalInflow_q0_m = GI_TotalInflow_q0_m + folder->VarVecDict["GI_Inflow_m"][i];

		GI_TotalSurfaceOutflow_q1_m = GI_TotalSurfaceOutflow_q1_m + folder->VarVecDict["GI_SurfaceOutflow_m"][i];
		GI_TotalInfiltration_f1_m = GI_TotalInfiltration_f1_m + folder->VarVecDict["GI_Infiltration_m"][i];

		GI_TotalStorageDrainFlow_q3_m = GI_TotalStorageDrainFlow_q3_m + folder->VarVecDict["GI_StorageDrainFlow_m"][i];

		//Define GI exfiltration to native soils based on presence of storage layer
		if (folder->ParamDict["storThickness_m"] == 0.0) 
		{
			//RainGarden has not storage layer, and percolation provides exfiltration to native soil
			GI_TotalExfilrationToNativeSoil_f2_or_f3_m = GI_TotalExfilrationToNativeSoil_f2_or_f3_m + folder->VarVecDict["GI_SoilPercolation_m"][i];
		}
		else {
			//Bioretention may have storage layer, and it provides exfiltration to native soil
			GI_TotalExfilrationToNativeSoil_f2_or_f3_m = GI_TotalExfilrationToNativeSoil_f2_or_f3_m + folder->VarVecDict["GI_StorageExfiltration_m"][i];
		}
		GI_DeltaStorageDepth_d3_m = (folder->VarVecDict["GI_StorageDepth_m"][i] - folder->VarVecDict["GI_StorageDepth_m"][0]) * folder->ParamDict["storVoidFrac"];
		GI_DeltaSoilMoisture_d2_m3_p_m3 = (folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][i] - folder->VarVecDict["GI_SoilMoisture_m3_p_m3"][0]) * folder->ParamDict["soilThickness_m"];
		GI_DeltaSurfaceDepth_d1_m = folder->VarVecDict["GI_SurfaceDepth_m"][i] - folder->VarVecDict["GI_SurfaceDepth_m"][0];
		GI_PavementDepth_d4_m = folder->VarVecDict["GI_PavementStorageDepth_m"][i] - folder->VarVecDict["GI_PavementStorageDepth_m"][0];

		GI_TotalEvaporationAllLayers_m = GI_TotalEvaporationAllLayers_m + folder->VarVecDict["GI_GI_EvaporationAllLayers_m"][i];
		//Check the water budget of the surface layer for every time step q0=?q1+f1+d1
		//folder->VarVecDict["SurfaceBudget"][i] = folder->VarVecDict["GI_Inflow_m"][i] - folder->VarVecDict["GI_SurfaceDepth_m"][i] - folder->VarVecDict["GI_SurfaceOutflow_m"][i] - folder->VarVecDict["GI_Infiltration_m"][i];
	}

	//Check the water budget of the surface layer
	folder->VarDict["GI_SurfaceWaterBudget_m"] = GI_TotalInflow_q0_m - GI_TotalSurfaceOutflow_q1_m - GI_TotalInfiltration_f1_m - GI_DeltaSurfaceDepth_d1_m;
	//Check the water budget of the whole device, including surface, soil, storage, evap
	folder->VarDict["GI_TotalWaterBudget_m"] = GI_TotalInflow_q0_m - GI_TotalStorageDrainFlow_q3_m - GI_TotalSurfaceOutflow_q1_m - GI_TotalExfilrationToNativeSoil_f2_or_f3_m
		- GI_DeltaStorageDepth_d3_m - GI_DeltaSoilMoisture_d2_m3_p_m3 - GI_DeltaSurfaceDepth_d1_m - GI_TotalEvaporationAllLayers_m - GI_PavementDepth_d4_m;

	//Save the results in VarDict, for passing the results to statisticalOurpurWriter.cpp. 
	folder->VarDict["GI_TotalInflow_q0_m"] = GI_TotalInflow_q0_m;
	folder->VarDict["GI_TotalStorageDrainFlow_q3_m"] = GI_TotalStorageDrainFlow_q3_m;
	folder->VarDict["GI_TotalSurfaceOutflow_q1_m"] = GI_TotalSurfaceOutflow_q1_m;
	folder->VarDict["GI_TotalExfilrationToNativeSoil_m"] = GI_TotalExfilrationToNativeSoil_f2_or_f3_m;
	folder->VarDict["GI_TotalInfiltration_f1_m"] = GI_TotalInfiltration_f1_m;
	folder->VarDict["GI_DeltaSurfaceDepth_d1_m"] = GI_DeltaSurfaceDepth_d1_m;

}