#include "StatisticalOutputWriter.h"
#include <iomanip>
#include <iostream>
#include "GIOutputWriter.h"



void StatisticalOutputWriter::writeLAISeries(Inputs *input)
{

	ofstream outfileLAI(input->SimulationStringParams["OutputDirectory"] + "VegetationLAISeries.dat");
	if (!outfileLAI.good())
	{
		LogMessage::log("Could not create: "+input->SimulationStringParams["OutputDirectory"] + "VegetationLAISeries.dat");
		return;
	}
	outfileLAI<<"yyyymmdd   "<<"Hr:Min:Sec "<<"TreeLAI "<<"ShortVegLAI  "<<endl;

	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	for(int i=0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		  outfileLAI<<input->CalendarDates[i]<<"   "<<input->TimeSeries[i]<<"    "<<input->TreeLAI[i]<<"     "<<input->ShortVegLAI[i]<<"    "<<endl;
	}
	outfileLAI.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "VegetationLAISeries.dat");

}

//Continuity maintained when adjusting to folder area from cover type area (e.g., pervious or impervious). 
//		Depth_adjusted_CoverType_m = Depth_unadjusted_CoverType_m * Area_CoverType_fraction = Depth_unadjusted_CoverType_m * Area_CoverType_m2 / Area_Folder_m2 
//		Adjustment allows depths reported for folder (e.g., bulk area, GI, etc.) to balance with total inflow 
//		For a folder with Area_Pervious_fraction = 0.8 (fully saturated) and Area_Impervious_fraction = 0.2, each generating Depth_unadjusted_CoverType_m = 0.05 of runoff. 
//		Depth_adjusted_Pervious_m = Depth_unadjusted_Pervious_m * Area_Pervious_fraction and Depth_adjusted_Impervious_m = Depth_unadjusted_Impervious_m * Area_Impervious_fraction

void StatisticalOutputWriter::writeCumulativeFile(Inputs *input, DataFolder* BulkAreafolder)
{
	std::string path;
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") { //RC,11/30/2018-in future rename to Output_BulkArea if isolated to BulkArea, then have Output.dat for whole prj area
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput.dat"; //For now, naming Output.dat so when no GI is run we get output for prj, which GUI & tests expect
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_TreePit.dat";
	}
	//added by Li
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_InfilTrench.dat";
	}

	std::ifstream infile(path);
	bool exists = infile.good();

	std::ofstream OutFile(path, std::ios::app);
	if (!OutFile.good())
	{
		LogMessage::log("Could not create: " + path);
		return;
	}
	
	double TotalImperviousCover_frac = BulkAreafolder->ParamDict["TotalImperviousCover_frac"]; 
	double PerPercent = BulkAreafolder->ParamDict["TotalPerviousCover_frac"];
	double DCIA_frac = BulkAreafolder->ParamDict["DCIA_frac"];
    double NoTreeImperviousCover_frac = BulkAreafolder->ParamDict["SurfaceWaterIA_frac"] + BulkAreafolder->ParamDict["ImperviousCover_frac"];
	double TreePerviousCover_frac = BulkAreafolder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	double TreeImperviousCover_frac = BulkAreafolder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
    double BareSoilCover_frac = BulkAreafolder->ParamDict["SoilCover_frac"];
	double ShortVegCover_frac = BulkAreafolder->ParamDict["ShortVegetationCover_frac"];
	double Total_Flow_Routed_mm = input->RepoDict["total_Runoff_R_m"]*1000;
	double TotalPerviousAreaRunoff_mm = input->RepoDict["total_Runoff_Pervious_R_m"]*1000;
	double Total_SubsurfaceFlow_Routed_mm = input->RepoDict["total_Runoff_Subsurface_R_m"]*1000;
	double Total_Impervious_Flow_Routed_mm = input->RepoDict["total_Runoff_Impervious_R_m"]*1000;
	
	if (!exists)
	{
		OutFile << "  " << std::setw(5) << "TotalImperviousArea_percent"
			<< "  " << std::setw(10) << "Pervious-Percentage "
			<< "  " << std::setw(10) << "TotalImpAreaDirectConnect_frac"
			<< "  " << std::setw(10) << "NoTreeImpAreaDirectConnect_frac "
			<< "  " << std::setw(10) << "TreePerviousCover_frac"
			<< "  " << std::setw(10) << "TreeImperviousCover_frac"
			<< "  " << std::setw(10) << "SoilCover_frac"
			<< "  " << std::setw(10) << "ShortVegCover_frac"
			<< "  " << std::setw(10) << "TotalFlow"
			<< "  " << std::setw(10) << "TotalPerviousRunoff_mm"
			<< "  " << std::setw(10) << "TotalBaseFlow_mm"
			<< "  " << std::setw(10) << "TotalDCIARunoff_mm" << std::endl << std::endl;
	}
	OutFile << "  " << std::setw(15) << TotalImperviousCover_frac
		<< "  " << std::setw(20) << (TreePerviousCover_frac+ TreeImperviousCover_frac)*100
		<< "  " << std::setw(15) << DCIA_frac
		<< "  " << std::setw(15) << NoTreeImperviousCover_frac
		<< "  " << std::setw(20) << TreePerviousCover_frac
		<< "  " << std::setw(15) << TreeImperviousCover_frac
		<< "  " << std::setw(15) << BareSoilCover_frac
		<< "  " << std::setw(15) << ShortVegCover_frac
		<< "  " << std::setw(15) << Total_Flow_Routed_mm
		<< "  " << std::setw(15) << TotalPerviousAreaRunoff_mm
		<< "  " << std::setw(15) << Total_SubsurfaceFlow_Routed_mm
		<< "  " << std::setw(15) << Total_Impervious_Flow_Routed_mm << std::endl;


	OutFile.close();
	LogMessage::log("Created: " + path);
}

void StatisticalOutputWriter::writeWaterBalanceFile(Inputs *input, DataFolder* BulkAreafolder,DataFolder* folder)
{
	//cte 20211014 Only one CumWaterBalance file should be written - it is for the Catchment area
	//	This may require summing the BulkArea and GI components, and their transfers, in FlowSumCalc.cpp
	std::string path;
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance.dat";
	}
	/*
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_TreePit.dat";
	}
	//Added by Li
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumWaterBalance_InfilTrench.dat";
	}
	*/

	std::ofstream outfileWB(path);
	if (!outfileWB)
	{
		LogMessage::log("Could not create: " + path);
		return;
	}

	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	int totalTimeSteps = input->SimulationNumericalParams["TotalTimeSteps"] - 1;

	//cte 20210408: should add _mm and _m to totals, which conisder variables present on or in BulkArea folders 
	double TotalImperviousCover_frac = BulkAreafolder->ParamDict["TotalImperviousCover_frac"];
	double TotalPerviousCover_frac = BulkAreafolder->ParamDict["TotalPerviousCover_frac"];
	double TotalImpDirectConnect_frac = BulkAreafolder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"];
	double TotalImpNotDirectConnect_frac = BulkAreafolder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"];
	double NoTreeImpAreaDirectConnect_frac = folder->ParamDict["ImperviousCover_frac"] * folder->ParamDict["DCIA_frac"];
	double TreePerviousCover_frac = BulkAreafolder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	double TreeImperviousCover_frac = BulkAreafolder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double ShortVegCover_frac = BulkAreafolder->ParamDict["ShortVegetationCover_frac"];
	double BareSoilCover_frac = BulkAreafolder->ParamDict["SoilCover_frac"];
	double ImperviousCover_frac = BulkAreafolder->ParamDict["ImperviousCover_frac"];
	
	//GI terms are set to zero when in BulkArea output
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") {
		folder->VarDict["GI_SurfaceWaterBudget_m"] == 0.0;
		folder->VarDict["GI_TotalWaterBudget_m"] == 0.0;
		folder->VarDict["GI_TotalInflow_q0_m"] = 0;
		folder->VarDict["GI_TotalSurfaceOutflow_q1_m"] = 0;
		folder->VarDict["GI_TotalInfiltration_f1_m"] = 0;
		folder->VarDict["GI_DeltaSurfaceDepth_d1_m"] = 0;
	}
	else {
		GIOutputWriter::waterbudget(input, folder);
	}

	//CumWaterBalance.dat for Catchment output
	//Note, multiplication of variables by fractional area below is to adjust depths from one analysis area to entire catchment area
	//	Rationale:	1. variables arrive as depths for an analysis area (e.g., pervious area), which are not conserved when transfered to the catchment area
	//				2. multiplying these depths by their analysis area gives a volume, which are conserved when transfered between different areas
	//				3. multiplying these depths by their fractional area is equivalent to multiplying by their analysis area and dividing by the catchment area, getting depth for catchment area
	outfileWB << "WaterBalanceVariable_units,Value" << endl;
	outfileWB << "Area_ImperviousAll_frac" << "," << TotalImperviousCover_frac << endl;
	outfileWB << "Area_PerviousAll_frac" << "," << TotalPerviousCover_frac << endl;
	outfileWB << "Area_ImperviousTree_frac" << "," << TreeImperviousCover_frac << endl;
	outfileWB << "Area_PerviousTree_frac" << "," << TreePerviousCover_frac << endl;
	outfileWB << "Area_ShortVegCover_frac" << "," << ShortVegCover_frac << endl;
	outfileWB << "Area_BareSoilCover_frac" << "," << BareSoilCover_frac << endl;
	outfileWB << "Area_DCIA_frac" << "," << BulkAreafolder->ParamDict["DCIA_frac"] << endl;
	outfileWB << "Area_InfiltExcessGoverned_frac" << "," << BulkAreafolder->ParamDict["InfiltExcessGovernedArea_frac"] << endl;
	outfileWB << "Area_MacroPore_frac" << "," << folder->ParamDict["PMacro_frac"] << endl;
	outfileWB << "TS_1st_SurficialGroundwater_frac" << "," << input->RepoVecDict["Groundwater_surficial_frac"][0] << endl;
	outfileWB << "TS_Last_SurficialGroundwater_frac" << "," << input->RepoVecDict["Groundwater_surficial_frac"][totalTimeSteps] << endl;
	outfileWB << "TS_1st_SurficialGroundwater_mm" << "," << input->RepoVecDict["Groundwater_surficial_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_Last_SurficialGroundwater_mm" << "," << input->RepoVecDict["Groundwater_surficial_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_Precip_RainSWE_mm" << "," << input->TotalPrecip_m * 1000 << endl;
	outfileWB << "Total_Rain_mm" << "," << input->TotalRain_m * 1000 << endl;
	outfileWB << "Total_SnowWE_mm" << "," << input->TotalSnow_m * 1000 << endl;
	outfileWB << "Total_TreeIntercept_RainSWE_mm" << "," << (input->RepoDict["total_InterceptionRain_viaTree_m"] + input->RepoDict["total_InterceptionSnowWE_viaTree_m"]) * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_ShortVegIntercept_RainSWE_mm" << "," << (input->RepoDict["total_InterceptionRain_viaSVeg_m"] + input->RepoDict["total_InterceptionSnowWE_viaSVeg_m"]) * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_TreeEvap_RainSWE_mm" << "," << input->RepoDict["total_Evaporation_viaTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_ShortVegEvap_RainSWE_mm" << "," << input->RepoDict["total_Evaporation_viaSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "TS_Last_Tree_LeafStorage_Rain_mm" << "," << folder->VarVecDict["treeStoredW"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "TS_Last_Tree_LeafStorage_SWE_mm" << "," << folder->VarVecDict["treeStoredSWE"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "TS_Last_ShortVeg_LeafStorage_Rain_mm" << "," << folder->VarVecDict["shortVegStoredW"][totalTimeSteps] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "TS_Last_ShortVeg_LeafStorage_SWE_mm" << "," << folder->VarVecDict["shortVegStoredSWE"][totalTimeSteps] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_TreeThruFlow_RainSWE_mm" << "," << input->RepoDict["totalTreeThruFlow_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_TreeThruFlow_SnowWE_mm" << "," << input->RepoDict["totalTreeThruFlowSnowWE_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_ShortVegThruFlow_RainSWE_mm" << "," << input->RepoDict["totalShortVegThruFlow_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_ShortVegThruFlow_SnowWE_mm" << "," << input->RepoDict["totalShortVegThruFlowSnowWE_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "TS_Last_SnowWE_Open_mm" << "," << input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000 * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac + ShortVegCover_frac)) << endl;
	outfileWB << "TS_Last_SnowWE_UnderTree_mm" << "," << input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "TS_Last_SnowWE_UnderSVeg_mm" << "," << input->RepoVecDict["totalSnowUnderShortVeg_TS_m"][totalTimeSteps] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_SnowWE_Interception_Tree_mm" << "," << input->RepoDict["total_InterceptionSnowWE_viaTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_SnowWE_Interception_SVeg_mm" << "," << input->RepoDict["total_InterceptionSnowWE_viaSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_SnowWE_Sub_Tree_mm" << "," << folder->VarDict["totalTreeEvapSWE"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_SnowWE_Sub_SVeg_mm" << "," << folder->VarDict["totalShortVegEvapSWE"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_SnowWE_Sub_UnderTree_mm" << "," << input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_SnowWE_Sub_UnderShortVeg_mm" << "," << input->RepoDict["total_SnowSublimation_UnderSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_SnowWE_Sub_OpenArea_mm" << "," << input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000 * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac + ShortVegCover_frac)) << endl;
	outfileWB << "Total_SnowWE_Melt_TreeCanopy_mm" << "," << input->RepoDict["total_SnowMelt_TreeCanopy_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_SnowWE_Melt_SVegCanopy_mm" << "," << input->RepoDict["total_SnowMelt_SVegCanopy_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_SnowWE_Melt_UnderTree_mm" << "," << input->RepoDict["total_SnowMelt_UnderTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	outfileWB << "Total_SnowWE_Melt_UnderShortVeg_mm" << "," << input->RepoDict["total_SnowMelt_UnderSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	outfileWB << "Total_SnowWE_Melt_OpenArea_mm" << "," << input->RepoDict["total_SnowMelt_OpenArea_m"] * 1000 * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac + ShortVegCover_frac)) << endl;
	outfileWB << "Total_WaterOnPerArea_mm" << "," << input->RepoDict["totalWaterOnPerviousArea_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_WaterOnImpArea_mm" << "," << input->RepoDict["totalWaterOnImperviousArea_m"] * 1000 * TotalImperviousCover_frac  << endl;
	outfileWB << "Total_ImperDepEvap_mm" << "," << input->RepoDict["totalImpDepEvap_m"] * 1000 * TotalImperviousCover_frac  << endl;
	outfileWB << "TS_Last_ImpDepStor_mm" << "," << input->RepoVecDict["totalImpDepStor_TS_m"][totalTimeSteps] * 1000 * TotalImperviousCover_frac << endl;
	//precursor of total_Runoff_Impervious_NR_m, folder->VarDict["Runoff_ImperviousToOutlet_m"], adjusted to catchment area depth in ImperviousDepressionStorageCalc.cpp with Runoff_ImperviousToPervious_m multiplication by ImpAreaNotDirectlyConnected2Outlet_frac
	outfileWB << "Total_ImperFlowToSoil_mm" << "," << input->RepoDict["totalImpFlowToSoil_m"] * 1000 << endl;
	//precursor of total_Runoff_Impervious_NR_m, folder->VarDict["Runoff_ImperviousToOutlet_m"], adjusted to catchment area depth in ImperviousDepressionStorageCalc.cpp with Runoff_ImperviousToOutlet_m multiplication by ImpAreaDirectlyConnected2Outlet_frac or TotalImperviousCover_frac
	outfileWB << "Total_ImpFlowToOutlet_mm" << "," << input->RepoDict["totalDCIAFlowToOutlet_m"] * 1000 << endl;
	outfileWB << "Total_WaterToPerDep_mm" << "," << input->RepoDict["totalWaterToSoil_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_PerDepEvap_mm" << "," << input->RepoDict["totalPerDepEvap_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_Last_PerDepStor_mm" << "," << input->RepoVecDict["totalPerDepStore_TS_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	//Maximum Soil Moisture Deficit (m) x 1000
	outfileWB << "MSD_mm" << "," << folder->ParamDict["MSD_m"] * 1000 << endl;
	//(m3_p_m3)DWSMI = deficit water soil moisture initial(volumetric moisture content for units)
	outfileWB << "TS_1st_RootZoneMoisture_unFilled_frac" << "," << folder->ParamDict["DWSMI"] << endl;
	//computed from SD0 and MSD, not adjusted to bulk area
	outfileWB << "TS_1st_RootZoneMoisture_unFilled_mm" << "," << ((folder->ParamDict["SD0_percent"] / 100) * folder->ParamDict["MSD_m"]) * 1000 << endl;
	outfileWB << "TS_1st_RootZoneStor_mm" << "," << input->RepoVecDict["Storage_Used_RootZone_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_Last_RootZoneStor_mm" << "," << input->RepoVecDict["Storage_Used_RootZone_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_1st_UnsatZoneStor_mm" << "," << input->RepoVecDict["Storage_Used_UnsatZone_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_Last_UnsatZoneStor_mm" << "," << input->RepoVecDict["Storage_Used_UnsatZone_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_1st_AveSMD_mm" << "," << input->RepoVecDict["aveSMD_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "TS_Last_AveSMD_mm" << "," << input->RepoVecDict["aveSMD_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_FlowToInfil_mm" << "," << input->RepoDict["total_Water_forSoilRecharge_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_PMacroFlow_mm" << "," << input->RepoDict["total_Drainage_macroPore_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_Infiltration_mm" << "," << input->RepoDict["total_Infiltration_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_InfilExInfil_mm" << "," << input->RepoDict["total_InfilExInfil_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_SatEx_Infil_mm" << "," << input->RepoDict["total_SatEx_Infil_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_Infil_Excess_RO_mm" << "," << input->RepoDict["totalInfilExQ_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_Sat_Excess_RO_mm" << "," << input->RepoDict["totalSatExQ_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_RootZoneET_mm" << "," << input->RepoDict["totalRootZoneEvapoTranspiration_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_RootZone_Flow_mm" << "," << input->RepoDict["total_Drainage_RootZone_m"] * 1000 * TotalPerviousCover_frac << endl;
	outfileWB << "Total_UnsatZone_Flow_mm" << "," << input->RepoDict["total_unsatZoneFlow_m"] * 1000 * TotalPerviousCover_frac << endl;
	//total_Runoff_Pervious_NR_m (m) is depth over catchment area, previously multiplied by TotalPerviousCover_frac in FlowSumCalc.cpp and then folder area in StoreTimeStepTotalsCalc.cpp
	outfileWB << "Total_Pervious_Flow_NonRouted_mm" << "," << input->RepoDict["total_Runoff_Pervious_NR_m"] * 1000 << endl;
	//total_Runoff_Impervious_NR_m (m) is depth over catchment area, previously multiplied by TotalImperviousCover_frac (DCIA) in ImperviousDepressionStorageCalc.cpp and then folder area in StoreTimeStepTotalsCalc.cpp 
	outfileWB << "Total_Impervious_Flow_NonRouted_mm" << "," << input->RepoDict["total_Runoff_Impervious_NR_m"] * 1000 << endl;
	//total_Runoff_Subsurface_NR_m (m) is depth over catchment area, previously multiplied by TotalPerviousCover_frac in FlowSumCalc.cpp and then folder area in StoreTimeStepTotalsCalc.cpp
	outfileWB << "Total_SubsurfaceFlow_NonRouted_mm" << "," << input->RepoDict["total_Runoff_Subsurface_NR_m"] * 1000  << endl;
	//total_Runoff_All_NR_m (m) is depth over catchment, summing folder depth input->RepoDict["total_Runoff_Pervious_NR_m"] and input->RepoDict["total_Runoff_Impervious_NR_m"] and then folder area in StoreTimeStepTotalsCalc.cpp
	outfileWB << "Total_Flow_NonRouted_mm" << "," << input->RepoDict["total_Runoff_All_NR_m"] * 1000 << endl;
	// adjusted to bulk area depth in FlowSumCalc.cpp with multiplication of PAQ with PerviousAreaOfFolder_frac
	outfileWB << "Total_Pervious_Flow_Routed_mm" << "," << input->RepoDict["total_Runoff_Pervious_R_m"] * 1000 << endl;
	// adjusted to bulk area depth in ImperviousDepressionStorageCalc.cpp with multiplication of PAQ with folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"]
	outfileWB << "Total_Impervious_Flow_Routed_mm" << "," << input->RepoDict["total_Runoff_Impervious_R_m"] * 1000 << endl;
	outfileWB << "Total_SubsurfaceFlow_Routed_mm" << "," << input->RepoDict["total_Runoff_Subsurface_R_m"] * 1000 << endl;
	outfileWB << "Total_Flow_Routed_mm" << "," << input->RepoDict["total_Runoff_R_m"] * 1000 << endl;
	outfileWB << "GI_Total_Inflow_mm" << "," << folder->VarDict["GI_TotalInflow_q0_m"] * 1000 << endl;
	outfileWB << "GI_Total_SurfaceOutflow_mm" << "," << folder->VarDict["GI_TotalSurfaceOutflow_q1_m"] * 1000 << endl;
	outfileWB << "GI_Total_SurfaceInfil_mm" << "," << folder->VarDict["GI_TotalInfiltration_f1_m"] * 1000 << endl;
	outfileWB << "GI_Delta_Surfacedepth_mm" << "," << folder->VarDict["GI_DeltaSurfaceDepth_d1_m"] * 1000 << endl;
	outfileWB << "GI_SurfaceBudget_mm" << "," << folder->VarDict["GI_SurfaceWaterBudget_m"] * 1000 << endl;
	outfileWB << "GI_TotalBudget_mm" << "," << folder->VarDict["GI_TotalWaterBudget_m"] * 1000 << endl;
	//Infiltration type for decay function with hydraulic conductivity and computation of topographic index
	if (input->SimulationStringParams["Infiltration"] == "ExponentialDecay") {
		outfileWB << "HydraulicConductivityDecay_function" << "," << 0 << endl;
	}
	if (input->SimulationStringParams["Infiltration"] == "PowerDecay") {
		outfileWB << "HydraulicConductivityDecay_function" << "," << 1 << endl;
	}
	//Timestep parameter for adjusting non-hourly rate constants given in HydroPlusConfig.xml
	outfileWB << "TimeStep_sec" << "," << folder->ParamDict["TimeStep_sec"] << endl;
	//Scaling parameter for adjusting soil moisture across topographic index
	outfileWB << "m_ScalingParam" << "," << folder->ParamDict["m"] << endl;
	//Power decay parameter for adjusting conductivity across topographic index
	outfileWB << "n_PowerDecayParam" << "," << folder->ParamDict["n"] << endl;
	//Transmissivity (m^2/dt) at catchment saturation
	outfileWB << "T0_m2_p_dt" << "," << folder->ParamDict["T0_m2ph"] * folder->ParamDict["TimeStep_sec"] / 3600 << endl;
	//Subsurface flow (m/dt) at start of simulation
	outfileWB << "Q0_m_p_dt" << "," << folder->ParamDict["Q0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600 << endl;
	//Average topographic index value
	outfileWB << "TopographicIndex_Avg" << "," << input->AverageIndex << endl;
	//Number of topographic index bins
	outfileWB << "NAC" << "," << input->SimulationNumericalParams["TopographicIndexBins"] << endl;

	//Loop through all topographic index values, write TI value
	for (int ia = 0; ia < input->IndexSize; ia++) {
		outfileWB << "TopographicIndex_" << ia << "," << input->Index[ia] << endl;
	}

	//Loop through all topographic index values; ordered in histogram so TI is wettest for ia = 0, but this is upper limit and has no fractional area  
	for (int ia = 0; ia < input->IndexSize; ia++) {
		outfileWB << "TopographicIndexArea_f_" << ia << "," << input->IndexArea[ia] << endl;
	}

	outfileWB.close();
	LogMessage::log("Created: " + path);

	//cte 20210806 Water budget is written for catchment area, using RepoDict and RepoVecDict variables.
	//	1. Need to check if budget erroneously uses folder variables (e.g., VarDict or VarVecDict and non RepoDict or RepoVecDict variables), which
	//		would be a problem when running GI scenarios given they have multiple folders within a catchment (e.g., BulkArea and GI area)
	// 	   This needs to be rewritten as RepoVecDict: folder->VarVecDict["treeStoredW"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac)
	//	2. Need to determine if this catchment area approach can work for multiple folders using the area-weighted summation in StoreTimeStepTotalsCalc.cpp (e.g.,  * fArea & +=) and perhaps FlowSumCalc.cpp
	//		or if we need a new routine to isolate BulkArea and GI water balances, similar to how we define terms for GI, such as GI_Total_Inflow_mm, above using VarDict variables 
	//	3. Need to incorporate the water transfers for simulations with multiple folders, between BulkArea and GI, which are surface runoff from BulkArea to GI, and q1 and q3 flow from GI to BulkArea outlet
	//		but may include upwelling flow from BulkArea groundwater table to GI in LocalSoilMoistureCalc.cpp
	//	4. Need to replace all uses of fractional areas that are related to folder with fractional areas related to catchment for the above water balance terms
	//		This and other fractional terms above need to be rewritten in code: TreePerviousCover_frac + TreeImperviousCover_frac
	//		Above the code is defining these based on BulkArea, as: TotalImperviousCover_frac = BulkAreafolder->ParamDict["TotalImperviousCover_frac"]
	//		This works when simulating catchment as 1 folder, but when simulating GI needs fractional area weighting.
	//		Perhaps use a condition: (input->RepoDict["GI_FolderOperational"] = 1 && folder->ParamStringDict["Type"] == "BulkArea") to weight with GI
	//		Perhaps use this type of loop from StoreTimeStepTotalsCalc.cpp
			/* 	//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
			for (int i = 0; i < DataDrawers.size(); ++i)
			{
				//Loop through the DataFolders in each DataDrawer, where the upper bound is based on the specific DataDrawer[i] attribute .size()
				for (int j = 0; j < DataDrawers[i].size(); ++j)
				{
					//For DataFolders that have data, BulkArea or GI
					if (DataDrawers[i][j]->ParamStringDict["Type"]  != "NODATA")
					{
						DataFolder *folder = DataDrawers[i][j];

						//Note, multiplication of variables by fractional area below is to adjust depths from one analysis area to entire catchment area
						//	Rationale:	1. variables arrive as depths for an analysis area (e.g., Bulk Area or GI Area), which are not conserved when transfered to the catchment area
						//				2. multiplying these depths by their analysis area gives a volume, which are conserved when transfered between different areas
						//				3. multiplying these depths by their fractional area is equivalent to multiplying by their analysis area and dividing by the catchment area, getting depth for catchment area
						//				4. fArea multiplication below preserves depth for cover type area, which is addressed in StatisticalOutputWriter.cpp
						//area is DataFolder area (m2)
						area = folder->ParamDict["Area"];
						//fArea is fraction of DataFolder area (m2) to catchment area (m2)
						fArea = area / watershedArea; */

	//Water budget calculation using terms in CumWaterBalance.dat; all variable names and values are assigned from the file.
	double CA_TC_frac = TreeImperviousCover_frac + TreePerviousCover_frac;
	double IA_NoTC_frac = TotalImperviousCover_frac - TreeImperviousCover_frac;
	double CA_Open_frac = 1 - CA_TC_frac - ShortVegCover_frac;
	double PA_NoTC_frac = TotalPerviousCover_frac - TreePerviousCover_frac;
	//======================================================
	//Water Balance Equations:
	// 
	//Note, multiplication of variables by fractional area below is to adjust depths from one analysis area to entire catchment area
	//	Rationale:	1. variables arrive as depths for an analysis area (e.g., pervious area), which are not conserved when transfered to the catchment area
	//				2. multiplying these depths by their analysis area gives a volume, which are conserved when transfered between different areas
	//				3. multiplying these depths by their fractional area is equivalent to multiplying by their analysis area and dividing by the catchment area, getting depth for catchment area
	//======================================================
	//Eq. 1 Tree Canopy water balance (mm), relative to tree canopy area
	double TC_Total_Precip_RainSWE_mm, TC_Evap_mm, TC_StorageFinal_Canopy_mm, TC_ThroughFall_mm, TC_WaterBalance_mm;

	TC_Total_Precip_RainSWE_mm = input->TotalPrecip_m * 1000;
	TC_Evap_mm = input->RepoDict["total_Evaporation_viaTree_m"] * 1000;
	TC_StorageFinal_Canopy_mm = folder->VarVecDict["treeStoredW"][totalTimeSteps] * 1000 + folder->VarVecDict["treeStoredSWE"][totalTimeSteps] * 1000;
	TC_ThroughFall_mm = input->RepoDict["totalTreeThruFlow_m"] * 1000;
	//Balance is simple input = change in storage + output(s)
	TC_WaterBalance_mm = TC_Total_Precip_RainSWE_mm - TC_StorageFinal_Canopy_mm - TC_Evap_mm - TC_ThroughFall_mm;

	//======================================================
	//Eq. 2 Short Veg Canopy water balance (mm), relative to short vegetation area
	double SV_Total_Precip_RainSWE_mm, SV_Evap_mm, SV_StorageFinal_Canopy_mm, SV_ThroughFall_mm, SV_WaterBalance_mm;

	SV_Total_Precip_RainSWE_mm = input->TotalPrecip_m * 1000;
	SV_Evap_mm = input->RepoDict["total_Evaporation_viaSVeg_m"] * 1000;
	SV_StorageFinal_Canopy_mm = (folder->VarVecDict["shortVegStoredW"][totalTimeSteps] * 1000 + folder->VarVecDict["shortVegStoredSWE"][totalTimeSteps] * 1000);
	SV_ThroughFall_mm = input->RepoDict["totalShortVegThruFlow_m"] * 1000;
	//Balance is simple input = change in storage + output(s)
	SV_WaterBalance_mm = SV_Total_Precip_RainSWE_mm - SV_StorageFinal_Canopy_mm - SV_Evap_mm - SV_ThroughFall_mm;

	//======================================================
	//Eq. 3 Canopy Water balance (mm), relative to catchment area
	double TC_SV_Total_Precip_RainSWE_mm, CA_TC_SV_Evap_mm, CA_TC_SV_StorageFinal_Canopy_mm, CA_TC_SV_ThroughFall_mm, CA_TC_SV_WaterBalance_mm;

	TC_SV_Total_Precip_RainSWE_mm = input->TotalPrecip_m * 1000 * (CA_TC_frac + ShortVegCover_frac);
	CA_TC_SV_Evap_mm = input->RepoDict["total_Evaporation_viaTree_m"] * 1000 * CA_TC_frac + input->RepoDict["total_Evaporation_viaSVeg_m"] * 1000 * ShortVegCover_frac;
	CA_TC_SV_StorageFinal_Canopy_mm = (folder->VarVecDict["treeStoredW"][totalTimeSteps] * 1000 + folder->VarVecDict["treeStoredSWE"][totalTimeSteps] * 1000) * CA_TC_frac
		+ (folder->VarVecDict["shortVegStoredW"][totalTimeSteps] * 1000 + folder->VarVecDict["shortVegStoredSWE"][totalTimeSteps] * 1000) * ShortVegCover_frac;
	CA_TC_SV_ThroughFall_mm = input->RepoDict["totalTreeThruFlow_m"] * 1000 * CA_TC_frac + input->RepoDict["totalShortVegThruFlow_m"] * 1000 * ShortVegCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_TC_SV_WaterBalance_mm = TC_SV_Total_Precip_RainSWE_mm - CA_TC_SV_StorageFinal_Canopy_mm - CA_TC_SV_Evap_mm - CA_TC_SV_ThroughFall_mm;

	//======================================================
	//Eq. 4 Impervious Area Water Atmospheric Inputs balance (mm), relative to catchment area
	double CA_IA_TC_Throughfall_mm, CA_IA_NoTC_Precip_mm, CA_IA_TC_Sublimation_SnowWE_mm, CA_IA_TC_StorageFinal_Ground_SnowWE_mm, CA_IA_NoTC_Sublimation_SnowWE_mm, CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm, CA_IA_WaterOn_mm, CA_IA_Input_WaterBalance_mm;

	CA_IA_NoTC_Precip_mm = IA_NoTC_frac * input->TotalPrecip_m * 1000;
	CA_IA_WaterOn_mm = input->RepoDict["totalWaterOnImperviousArea_m"] * 1000 * TotalImperviousCover_frac;
	CA_IA_TC_Throughfall_mm = TreeImperviousCover_frac * input->RepoDict["totalTreeThruFlow_m"] * 1000;
	CA_IA_TC_Sublimation_SnowWE_mm = TreeImperviousCover_frac * input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000;
	CA_IA_TC_StorageFinal_Ground_SnowWE_mm = TreeImperviousCover_frac * input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000;
	CA_IA_NoTC_Sublimation_SnowWE_mm = IA_NoTC_frac * input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000;
	CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm = IA_NoTC_frac * input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000;
	//Balance is simple input = change in storage + output(s) [Snow melt is included in CA_IA_WaterOn_mm]
	CA_IA_Input_WaterBalance_mm = CA_IA_TC_Throughfall_mm + CA_IA_NoTC_Precip_mm - CA_IA_TC_Sublimation_SnowWE_mm - CA_IA_TC_StorageFinal_Ground_SnowWE_mm - CA_IA_NoTC_Sublimation_SnowWE_mm - CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm - CA_IA_WaterOn_mm;

	//======================================================
	//Eq. 5 Impervious Area Water Storage + Atmospheric Outputs balance (mm), relative to catchment area
	double CA_IA_StorageFinal_Depression_mm, CA_IA_Evaporation_Depression_mm, CA_IA_Runoff_To_PA_mm, CA_IA_Runoff_To_DCIA_mm, CA_IA_AtmosphericOutputs_WaterBalance_mm;

	CA_IA_WaterOn_mm = input->RepoDict["totalWaterOnImperviousArea_m"] * 1000 * TotalImperviousCover_frac;
	CA_IA_StorageFinal_Depression_mm = input->RepoVecDict["totalImpDepStor_TS_m"][totalTimeSteps] * 1000 * TotalImperviousCover_frac;
	CA_IA_Evaporation_Depression_mm = input->RepoDict["totalImpDepEvap_m"] * 1000 * TotalImperviousCover_frac;
	if (TotalImperviousCover_frac > 0) {
		CA_IA_Runoff_To_PA_mm = input->RepoDict["totalImpFlowToSoil_m"] * 1000;
		CA_IA_Runoff_To_DCIA_mm = input->RepoDict["totalDCIAFlowToOutlet_m"] * 1000;
	}
	else {
		CA_IA_Runoff_To_PA_mm = 0;
		CA_IA_Runoff_To_DCIA_mm = 0;
	}
	//Balance is simple input = change in storage + output(s) [Snow melt is included in WaterOn]
	CA_IA_AtmosphericOutputs_WaterBalance_mm = CA_IA_WaterOn_mm - CA_IA_StorageFinal_Depression_mm - CA_IA_Evaporation_Depression_mm - CA_IA_Runoff_To_PA_mm - CA_IA_Runoff_To_DCIA_mm;

	//======================================================
	//Eq. 6 Impervious Area non-routed Runoff balance (mm), relative to catchment area
	double CA_IA_RunOff_WaterBalance_mm;
	double CA_IA_Runoff_NR_mm = input->RepoDict["total_Runoff_Impervious_NR_m"] * 1000;
	if (TotalImperviousCover_frac > 0) {
		CA_IA_Runoff_To_DCIA_mm = input->RepoDict["totalDCIAFlowToOutlet_m"] * 1000;
	}
	else {
		CA_IA_Runoff_To_DCIA_mm = 0;
	}
	//Balance is simple input = change in storage + output(s)
	CA_IA_RunOff_WaterBalance_mm = CA_IA_Runoff_To_DCIA_mm - CA_IA_Runoff_NR_mm;

	//======================================================
	//Eq. 7 Pervious Area Water Atmospheric Inputs balance (mm), relative to catchment area
	double CA_PA_WaterOn_mm, CA_PA_NoTC_NoSV_Precip_mm, CA_SV_ThroughFall_mm, CA_SV_Sublimation_SnowWE_mm, CA_SV_StorageFinal_Ground_SnowWE_mm, CA_PA_TC_ThroughFall_mm, CA_PA_TC_Sublimation_SnowWE_mm,
		CA_PA_TC_StorageFinal_Ground_SnowWE_mm, CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm, CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm, CA_PA_AtmosphericInputs_WaterBalance_mm;

	CA_PA_WaterOn_mm = input->RepoDict["totalWaterOnPerviousArea_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_NoTC_NoSV_Precip_mm = (PA_NoTC_frac - ShortVegCover_frac) * input->TotalPrecip_m * 1000;
	CA_SV_ThroughFall_mm = input->RepoDict["totalShortVegThruFlow_m"] * 1000 * ShortVegCover_frac;
	CA_SV_Sublimation_SnowWE_mm = input->RepoDict["total_SnowSublimation_UnderSVeg_m"] * 1000 * ShortVegCover_frac;
	CA_SV_StorageFinal_Ground_SnowWE_mm = input->RepoVecDict["totalSnowUnderShortVeg_TS_m"][totalTimeSteps] * 1000 * ShortVegCover_frac;
	//reduce by pervious area with trees
	CA_PA_TC_ThroughFall_mm = TreePerviousCover_frac * input->RepoDict["totalTreeThruFlow_m"] * 1000;
	CA_PA_TC_Sublimation_SnowWE_mm = TreePerviousCover_frac * input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000;
	CA_PA_TC_StorageFinal_Ground_SnowWE_mm = TreePerviousCover_frac * input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000;
	//reduce by pervious area without vegetation
	CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm = (PA_NoTC_frac - ShortVegCover_frac) * input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000;
	CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm = (PA_NoTC_frac - ShortVegCover_frac) * input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000;
	// Balance is simple input = change in storage + output(s)
	CA_PA_AtmosphericInputs_WaterBalance_mm = (CA_PA_TC_ThroughFall_mm + CA_SV_ThroughFall_mm + CA_PA_NoTC_NoSV_Precip_mm) - CA_PA_TC_Sublimation_SnowWE_mm - CA_PA_TC_StorageFinal_Ground_SnowWE_mm - CA_SV_Sublimation_SnowWE_mm - CA_SV_StorageFinal_Ground_SnowWE_mm - CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm - CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm - CA_PA_WaterOn_mm;

	//======================================================
	//Eq. 8 Pervious Area Water Atmospheric + Runon Inputs balance (mm), relative to catchment area
	double CA_PA_Runon_From_IA_mm, CA_PA_Flux_PrecipAndRunoffToSoil_mm, CA_PA_AtmosphericInputs_Runon_WaterBalance_mm;

	CA_PA_WaterOn_mm = input->RepoDict["totalWaterOnPerviousArea_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Flux_PrecipAndRunoffToSoil_mm = input->RepoDict["totalWaterToSoil_m"] * 1000 * TotalPerviousCover_frac;
	if (TotalImperviousCover_frac > 0) {
		CA_PA_Runon_From_IA_mm = input->RepoDict["totalImpFlowToSoil_m"] * 1000;
	}
	else {
		CA_PA_Runon_From_IA_mm = 0;
	}
	//Balance is simple input = change in storage + output(s)
	CA_PA_AtmosphericInputs_Runon_WaterBalance_mm = CA_PA_Flux_PrecipAndRunoffToSoil_mm - CA_PA_WaterOn_mm - CA_PA_Runon_From_IA_mm;


	//======================================================
	//Eq. 9 Pervious Area Water Storage + Atmospheric Outputs + Infiltration balance (mm), relative to catchment area
	double CA_PA_Water_To_Infil_mm, CA_PA_Evaporation_Depression_mm, CA_PA_StorageFinal_Depression_mm, CA_PA_WaterToInfilt_WaterBalance_mm;
	
	CA_PA_Water_To_Infil_mm = input->RepoDict["total_Water_forSoilRecharge_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Evaporation_Depression_mm = input->RepoDict["totalPerDepEvap_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_StorageFinal_Depression_mm = input->RepoVecDict["totalPerDepStore_TS_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_PA_WaterToInfilt_WaterBalance_mm = CA_PA_Flux_PrecipAndRunoffToSoil_mm - CA_PA_StorageFinal_Depression_mm - CA_PA_Evaporation_Depression_mm - CA_PA_Water_To_Infil_mm;
	
	//======================================================
	//Eq. 10 Pervious Area Water to Infiltration + Runoff + Macroporeflow balance (mm), relative to catchment area
	double CA_PA_Runoff_InfilExcess_mm, CA_PA_Runoff_SatExcess_mm, CA_PA_Infiltration_InfilExcess_mm, CA_PA_Infiltration_SatExcess_mm, CA_PA_Drainage_Macropore_mm, CA_PA_Infiltration_WaterBalance_mm;
	
	CA_PA_Runoff_InfilExcess_mm = input->RepoDict["totalInfilExQ_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Runoff_SatExcess_mm = input->RepoDict["totalSatExQ_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Infiltration_InfilExcess_mm = input->RepoDict["total_InfilExInfil_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Infiltration_SatExcess_mm = input->RepoDict["total_SatEx_Infil_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Drainage_Macropore_mm = input->RepoDict["total_Drainage_macroPore_m"] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_PA_Infiltration_WaterBalance_mm = CA_PA_Water_To_Infil_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm - CA_PA_Drainage_Macropore_mm;

	//======================================================
	//Eq. 11 Pervious Area Infiltration routines balance (mm), relative to catchment area
	double CA_PA_Infiltration_mm, CA_PA_InfiltComponents_WaterBalance_mm;
	CA_PA_Infiltration_InfilExcess_mm = input->RepoDict["total_InfilExInfil_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Infiltration_SatExcess_mm = input->RepoDict["total_SatEx_Infil_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Infiltration_mm = input->RepoDict["total_Infiltration_m"] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_PA_InfiltComponents_WaterBalance_mm = CA_PA_Infiltration_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm;

	//======================================================
	//Eq. 12 Pervious Area non-routed Runoff routines (mm), relative to catchment area
	double CA_PA_Runoff_NR_mm, CA_PA_WaterToInfilt_WaterBalance_NR_mm;
	
	CA_PA_Runoff_NR_mm = input->RepoDict["total_Runoff_Pervious_NR_m"] * 1000;
	CA_PA_Runoff_InfilExcess_mm = input->RepoDict["totalInfilExQ_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Runoff_SatExcess_mm = input->RepoDict["totalSatExQ_m"] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_PA_WaterToInfilt_WaterBalance_NR_mm = CA_PA_Runoff_NR_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm;

	//======================================================
	//Eq. 13 Pervious Area Root Zone balance (mm), relative to catchment area
	double CA_PA_EvapTranspiration_RootZone_mm, CA_PA_StorageChange_RootZone_mm, CA_PA_Drainage_RootZone_mm, CA_PA_RootZone_WaterBalance_mm;
	
	CA_PA_Infiltration_mm = input->RepoDict["total_Infiltration_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Drainage_Macropore_mm = input->RepoDict["total_Drainage_macroPore_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_EvapTranspiration_RootZone_mm = input->RepoDict["totalRootZoneEvapoTranspiration_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_StorageChange_RootZone_mm = input->RepoVecDict["Storage_Used_RootZone_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac - input->RepoVecDict["Storage_Used_RootZone_m"][0] * 1000 * TotalPerviousCover_frac;
	CA_PA_Drainage_RootZone_mm = input->RepoDict["total_Drainage_RootZone_m"] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_PA_RootZone_WaterBalance_mm = CA_PA_Infiltration_mm + CA_PA_Drainage_Macropore_mm - CA_PA_StorageChange_RootZone_mm - CA_PA_EvapTranspiration_RootZone_mm - CA_PA_Drainage_RootZone_mm;

	//======================================================
	//Eq. 14 Pervious Area Unsaturated Zone balance (mm), relative to catchment area
	double CA_PA_StorageChange_UnsatZone_mm, CA_PA_Drainage_UnsatZone_mm, CA_PA_UnsatZone_WaterBalance_mm;
	
	CA_PA_StorageChange_UnsatZone_mm = input->RepoVecDict["Storage_Used_UnsatZone_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac - input->RepoVecDict["Storage_Used_UnsatZone_m"][0] * 1000 * TotalPerviousCover_frac;
	CA_PA_Drainage_UnsatZone_mm = input->RepoDict["total_unsatZoneFlow_m"] * 1000 * TotalPerviousCover_frac;
	CA_PA_Drainage_RootZone_mm = input->RepoDict["total_Drainage_RootZone_m"] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)   
	CA_PA_UnsatZone_WaterBalance_mm = CA_PA_Drainage_RootZone_mm - CA_PA_Drainage_UnsatZone_mm - CA_PA_StorageChange_UnsatZone_mm;
	
	//======================================================
	//Eq. 15 Pervious Area Saturated Zone balance (mm), relative to catchment area
	//Theory is in HydroPlus.sln SemiAveSMDCalc.cpp, aveSMD = aveSMD - Drainage_UnsatZone_m + Runoff_Subsurface_m
	double CA_PA_Runoff_Subsurface_mm, CA_PA_StorageDeficit_AvgSoil_1st_mm, CA_PA_StorageDeficit_AvgSoil_Last_mm, CA_PA_SubsurfaceFlow_WaterBalance_mm;

	CA_PA_Runoff_Subsurface_mm = input->RepoDict["total_Runoff_Subsurface_NR_m"] * 1000;
	CA_PA_StorageDeficit_AvgSoil_1st_mm = input->RepoVecDict["aveSMD_m"][0] * 1000 * TotalPerviousCover_frac;
	CA_PA_StorageDeficit_AvgSoil_Last_mm = input->RepoVecDict["aveSMD_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac;
	//Balance is simple input = change in storage + output(s)
	CA_PA_SubsurfaceFlow_WaterBalance_mm = CA_PA_StorageDeficit_AvgSoil_1st_mm - CA_PA_Drainage_UnsatZone_mm + CA_PA_Runoff_Subsurface_mm - CA_PA_StorageDeficit_AvgSoil_Last_mm;
	
	//======================================================
	//Eq. 16 Total Snow Water Equivalent balance (mm), relative to catchment area
	double CA_Precip_SnowWE_mm, CA_Total_Sublimation_Ground_SnowWE_mm, CA_Total_Sublimation_Canopy_SnowWE_mm, CA_Total_Melt_Ground_SnowWE_mm, CA_Total_Melt_Canopy_SnowWE_mm, CA_Total_Storage_Ground_SnowWE_mm, CA_TS_Last_Canopy_SnowWE_mm, CA_SnowWE_WaterBalance_mm;

	CA_Precip_SnowWE_mm = input->TotalSnow_m * 1000;
	CA_Total_Sublimation_Ground_SnowWE_mm = input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000 * CA_TC_frac + input->RepoDict["total_SnowSublimation_UnderSVeg_m"] * 1000 * ShortVegCover_frac
		+ input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000 * (1 - (CA_TC_frac + ShortVegCover_frac));
	CA_Total_Sublimation_Canopy_SnowWE_mm = folder->VarDict["totalTreeEvapSWE"] * 1000 * CA_TC_frac + folder->VarDict["totalShortVegEvapSWE"] * 1000 * ShortVegCover_frac;
	CA_Total_Melt_Ground_SnowWE_mm = input->RepoDict["total_SnowMelt_UnderTree_m"] * 1000 * CA_TC_frac + input->RepoDict["total_SnowMelt_UnderSVeg_m"] * 1000 * ShortVegCover_frac
		+ input->RepoDict["total_SnowMelt_OpenArea_m"] * 1000 * (1 - (CA_TC_frac + ShortVegCover_frac));
	CA_Total_Melt_Canopy_SnowWE_mm = input->RepoDict["total_SnowMelt_TreeCanopy_m"] * 1000 * CA_TC_frac + input->RepoDict["total_SnowMelt_SVegCanopy_m"] * 1000 * ShortVegCover_frac;
	CA_Total_Storage_Ground_SnowWE_mm = input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000 * CA_TC_frac + input->RepoVecDict["totalSnowUnderShortVeg_TS_m"][totalTimeSteps] * 1000 * ShortVegCover_frac
		+ input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000 * (1 - (CA_TC_frac + ShortVegCover_frac));
	CA_TS_Last_Canopy_SnowWE_mm = folder->VarVecDict["treeStoredSWE"][totalTimeSteps] * 1000 * CA_TC_frac + folder->VarVecDict["shortVegStoredSWE"][totalTimeSteps] * 1000 * ShortVegCover_frac;
	//Balance is simple input = change in storage + output(s)   
	CA_SnowWE_WaterBalance_mm = CA_Precip_SnowWE_mm - CA_Total_Sublimation_Ground_SnowWE_mm - CA_Total_Sublimation_Canopy_SnowWE_mm - CA_Total_Melt_Ground_SnowWE_mm - CA_Total_Melt_Canopy_SnowWE_mm - CA_Total_Storage_Ground_SnowWE_mm - CA_TS_Last_Canopy_SnowWE_mm;

	//======================================================
	//Eq. 17 Total Surface Water balance NonRouted (mm), relative to catchment area
	double CA_Total_Precip_RainSWE_mm, CA_Total_EvapoSublimation_Canopy_mm, CA_Total_EvapoSublimation_Ground_mm, CA_Total_StorageFinal_Canopy_mm, CA_Total_StorageFinal_Ground_mm, CA_Total_Drainage_ToSoil_mm, CA_Total_Runoff_Surface_NR_mm, CA_Total_WaterBalance_NR_mm;

	CA_Total_Precip_RainSWE_mm = input->TotalPrecip_m * 1000;
	CA_Total_EvapoSublimation_Canopy_mm = input->RepoDict["total_Evaporation_viaTree_m"] * 1000 * CA_TC_frac + input->RepoDict["total_Evaporation_viaSVeg_m"] * 1000 * ShortVegCover_frac;
	CA_Total_EvapoSublimation_Ground_mm = input->RepoDict["totalImpDepEvap_m"] * 1000 * TotalImperviousCover_frac + input->RepoDict["totalPerDepEvap_m"] * 1000 * TotalPerviousCover_frac 
		+ input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000 * CA_TC_frac + input->RepoDict["total_SnowSublimation_UnderSVeg_m"] * 1000 * ShortVegCover_frac + input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000 * (1 - (CA_TC_frac + ShortVegCover_frac));
	CA_Total_StorageFinal_Canopy_mm = (folder->VarVecDict["treeStoredW"][totalTimeSteps] * 1000 * CA_TC_frac + folder->VarVecDict["treeStoredSWE"][totalTimeSteps] * 1000 * CA_TC_frac) + (folder->VarVecDict["shortVegStoredW"][totalTimeSteps] * 1000 * ShortVegCover_frac 
		+ folder->VarVecDict["shortVegStoredSWE"][totalTimeSteps] * 1000 * ShortVegCover_frac);
	CA_Total_StorageFinal_Ground_mm = input->RepoVecDict["totalImpDepStor_TS_m"][totalTimeSteps] * 1000 * TotalImperviousCover_frac + input->RepoVecDict["totalPerDepStore_TS_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac 
		+ input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000 * (1 - (CA_TC_frac + ShortVegCover_frac)) + input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000 * CA_TC_frac + input->RepoVecDict["totalSnowUnderShortVeg_TS_m"][totalTimeSteps] * 1000 * ShortVegCover_frac;
	CA_Total_Drainage_ToSoil_mm = (input->RepoDict["total_Drainage_macroPore_m"] + input->RepoDict["total_Infiltration_m"]) * 1000 * TotalPerviousCover_frac;
	CA_Total_Runoff_Surface_NR_mm = input->RepoDict["total_Runoff_Pervious_NR_m"] * 1000 + input->RepoDict["total_Runoff_Impervious_NR_m"] * 1000;
	//Balance is simple input = change in storage + output(s)
	CA_Total_WaterBalance_NR_mm = CA_Total_Precip_RainSWE_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_Drainage_ToSoil_mm - CA_Total_Runoff_Surface_NR_mm;

	//======================================================
	//Eq. 18 Total Surface and Subsurface Water balance NonRouted (mm), relative to catchment area
	double CA_Total_EvapoTranspiration_Soil_mm, CA_Total_StorageChange_Subsurface_mm, CA_Total_Runoff_NR_mm, CA_Total_WaterBalance_R_mm;
	
	CA_Total_EvapoTranspiration_Soil_mm = input->RepoDict["totalRootZoneEvapoTranspiration_m"] * 1000 * TotalPerviousCover_frac;
	CA_Total_EvapoSublimation_Ground_mm = input->RepoDict["totalImpDepEvap_m"] * 1000 * TotalImperviousCover_frac + input->RepoDict["totalPerDepEvap_m"] * 1000 * TotalPerviousCover_frac + input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000 * CA_TC_frac 
		+ input->RepoDict["total_SnowSublimation_UnderSVeg_m"] * 1000 * ShortVegCover_frac + input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000 * (1 - (CA_TC_frac + ShortVegCover_frac));
	CA_Total_StorageChange_Subsurface_mm = (input->RepoVecDict["Storage_Used_RootZone_m"][totalTimeSteps] * 1000  - input->RepoVecDict["Storage_Used_RootZone_m"][0] * 1000 ) * TotalPerviousCover_frac
		+ (input->RepoVecDict["Storage_Used_UnsatZone_m"][totalTimeSteps] * 1000 - input->RepoVecDict["Storage_Used_UnsatZone_m"][0] * 1000 )* TotalPerviousCover_frac + (input->RepoVecDict["aveSMD_m"][0] * 1000 - input->RepoVecDict["aveSMD_m"][totalTimeSteps] * 1000 )* TotalPerviousCover_frac;
	CA_Total_Runoff_NR_mm = input->RepoDict["total_Runoff_All_NR_m"] * 1000;
	//Balance is simple input = change in storage + output(s)
	CA_Total_WaterBalance_R_mm = CA_Total_Precip_RainSWE_mm - CA_Total_EvapoTranspiration_Soil_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_StorageChange_Subsurface_mm - CA_Total_Runoff_NR_mm;

	//======================================================
	//Eq. 19 Runoff NonRoutedRouted vs  Balance (mm), relative to catchment area
	double Total_Pervious_Flow_NonRouted_mm, Total_Impervious_Flow_NonRouted_mm, Total_SubsurfaceFlow_NonRouted_mm, Total_Flow_NonRouted_mm, Total_Pervious_Flow_Routed_mm, Total_Impervious_Flow_Routed_mm, Total_SubsurfaceFlow_Routed_mm, Total_Flow_Routed_mm, CA_RunoffRouting_Balance_mm;
	
	Total_Pervious_Flow_NonRouted_mm = input->RepoDict["total_Runoff_Pervious_NR_m"] * 1000;
	Total_Impervious_Flow_NonRouted_mm= input->RepoDict["total_Runoff_Impervious_NR_m"] * 1000;
	Total_SubsurfaceFlow_NonRouted_mm = input->RepoDict["total_Runoff_Subsurface_NR_m"] * 1000;
	Total_Flow_NonRouted_mm= input->RepoDict["total_Runoff_All_NR_m"] * 1000;
	Total_Pervious_Flow_Routed_mm = input->RepoDict["total_Runoff_Pervious_R_m"] * 1000;
	Total_Impervious_Flow_Routed_mm= input->RepoDict["total_Runoff_Impervious_R_m"] * 1000;
	Total_SubsurfaceFlow_Routed_mm = input->RepoDict["total_Runoff_Subsurface_R_m"] * 1000;
	Total_Flow_Routed_mm= input->RepoDict["total_Runoff_R_m"] * 1000;
	//Balance is simple input = (non-routed - routed) 
	CA_RunoffRouting_Balance_mm = (Total_Pervious_Flow_NonRouted_mm - Total_Pervious_Flow_Routed_mm) + (Total_Impervious_Flow_NonRouted_mm - Total_Impervious_Flow_Routed_mm) + (Total_SubsurfaceFlow_NonRouted_mm - Total_SubsurfaceFlow_Routed_mm) + (Total_Flow_NonRouted_mm - Total_Flow_Routed_mm);

	//====================================================================================================
	//====================================================================================================
	//cte 20211014 Only one WaterBudget based on the CumWaterBalance file should be written - it is for the Catchment area
	//	This may require summing the BulkArea and GI components, and their transfers, in FlowSumCalc.cpp

	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_CatchmentArea.dat";
	}
	/*
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_TreePit.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBudget_InfilTrench.dat";
	}
	*/
	ofstream WaterBudget(path);
	if (!WaterBudget.good())
		return;

	WaterBudget.precision(2);
	WaterBudget.setf(ios::fixed);
	WaterBudget.setf(ios::showpoint);

	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Model Variable (units are _fraction or _mm):                " << "     Value:" << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Area_ImperviousAll_frac" << setw((70 - size("Area_ImperviousAll_frac"))) << TotalImperviousCover_frac << endl;
	WaterBudget << "Area_PerviousAll_frac" << setw((70 - size("Area_PerviousAll_frac"))) <<  TotalPerviousCover_frac << endl;
	WaterBudget << "Area_ImperviousTree_frac" << setw((70 - size("Area_ImperviousTree_frac")))  << TreeImperviousCover_frac << endl;
	WaterBudget << "Area_PerviousTree_frac" << setw((70 - size("Area_PerviousTree_frac")))  << TreePerviousCover_frac << endl;
	WaterBudget << "Area_ShortVegCover_frac" << setw((70 - size("Area_ShortVegCover_frac")))  << ShortVegCover_frac << endl;
	WaterBudget << "Area_BareSoilCover_frac" << setw((70 - size("Area_BareSoilCover_frac")))  << BareSoilCover_frac << endl;
	WaterBudget << "Area_DCIA_frac" << setw((70 - size("Area_DCIA_frac")))  << BulkAreafolder->ParamDict["DCIA_frac"] << endl;
	WaterBudget << "Area_InfiltExcessGoverned_frac" << setw((70 - size("Area_InfiltExcessGoverned_frac")))  << BulkAreafolder->ParamDict["InfiltExcessGovernedArea_frac"] << endl;
	WaterBudget << "Area_MacroPore_frac" << setw((70 - size("Area_MacroPore_frac"))) << folder->ParamDict["PMacro_frac"] << endl;
	WaterBudget << "TS_1st_SurficialGroundwater_frac" << setw((70 - size("TS_1st_SurficialGroundwater_frac")))  << input->RepoVecDict["Groundwater_surficial_frac"][0] << endl;
	WaterBudget << "TS_Last_SurficialGroundwater_frac" << setw((70 - size("TS_Last_SurficialGroundwater_frac")))  << input->RepoVecDict["Groundwater_surficial_frac"][totalTimeSteps] << endl;
	WaterBudget << "TS_1st_SurficialGroundwater_mm" << setw((70 - size("TS_1st_SurficialGroundwater_mm")))  << input->RepoVecDict["Groundwater_surficial_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_Last_SurficialGroundwater_mm" << setw((70 - size("TS_Last_SurficialGroundwater_mm")))  << input->RepoVecDict["Groundwater_surficial_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_Precip_RainSWE_mm" << setw((70 - size("Total_Precip_RainSWE_mm")))  << input->TotalPrecip_m * 1000 << endl;
	WaterBudget << "Total_Rain_mm" << setw((70 - size("Total_Rain_mm")))  << input->TotalRain_m * 1000 << endl;
	WaterBudget << "Total_SnowWE_mm" << setw((70 - size("Total_SnowWE_mm")))  << input->TotalSnow_m * 1000 << endl;
	WaterBudget << "Total_TreeIntercept_RainSWE_mm" << setw((70 - size("Total_TreeIntercept_RainSWE_mm")))  << (input->RepoDict["total_InterceptionRain_viaTree_m"] + input->RepoDict["total_InterceptionSnowWE_viaTree_m"]) * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_ShortVegIntercept_RainSWE_mm" << setw((70 - size("Total_ShortVegIntercept_RainSWE_mm"))) << (input->RepoDict["total_InterceptionRain_viaSVeg_m"] + input->RepoDict["total_InterceptionSnowWE_viaSVeg_m"]) * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_TreeEvap_RainSWE_mm" << setw((70 - size("Total_TreeEvap_RainSWE_mm")))  << input->RepoDict["total_Evaporation_viaTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_ShortVegEvap_RainSWE_mm" << setw((70 - size("Total_ShortVegEvap_RainSWE_mm")))  << input->RepoDict["total_Evaporation_viaSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "TS_Last_Tree_LeafStorage_Rain_mm" << setw((70 - size("TS_Last_Tree_LeafStorage_Rain_mm"))) << folder->VarVecDict["treeStoredW"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "TS_Last_Tree_LeafStorage_SWE_mm" << setw((70 - size("TS_Last_Tree_LeafStorage_SWE_mm")))  << folder->VarVecDict["treeStoredSWE"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "TS_Last_ShortVeg_LeafStorage_Rain_mm" << setw((70 - size("TS_Last_ShortVeg_LeafStorage_Rain_mm")))  << folder->VarVecDict["shortVegStoredW"][totalTimeSteps] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "TS_Last_ShortVeg_LeafStorage_SWE_mm" << setw((70 - size("TS_Last_ShortVeg_LeafStorage_SWE_mm"))) << folder->VarVecDict["shortVegStoredSWE"][totalTimeSteps] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_TreeThruFlow_RainSWE_mm" << setw((70 - size("Total_TreeThruFlow_RainSWE_mm")))  << input->RepoDict["totalTreeThruFlow_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_TreeThruFlow_SnowWE_mm" << setw((70 - size("Total_TreeThruFlow_SnowWE_mm")))  << input->RepoDict["totalTreeThruFlowSnowWE_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_ShortVegThruFlow_RainSWE_mm" << setw((70 - size("Total_ShortVegThruFlow_RainSWE_mm")))  << input->RepoDict["totalShortVegThruFlow_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_ShortVegThruFlow_SnowWE_mm" << setw((70 - size("Total_ShortVegThruFlow_SnowWE_mm")))  << input->RepoDict["totalShortVegThruFlowSnowWE_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "TS_Last_SnowWE_Open_mm" << setw((70 - size("TS_Last_SnowWE_Open_mm")))  << input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000 * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac + ShortVegCover_frac)) << endl;
	WaterBudget << "TS_Last_SnowWE_UnderTree_mm" << setw((70 - size("TS_Last_SnowWE_UnderTree_mm")))  << input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "TS_Last_SnowWE_UnderSVeg_mm" << setw((70 - size("TS_Last_SnowWE_UnderSVeg_mm")))  << input->RepoVecDict["totalSnowUnderShortVeg_TS_m"][totalTimeSteps] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_SnowWE_Interception_Tree_mm" << setw((70 - size("Total_SnowWE_Interception_Tree_mm")))  << input->RepoDict["total_InterceptionSnowWE_viaTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_SnowWE_Interception_SVeg_mm" << setw((70 - size("Total_SnowWE_Interception_SVeg_mm")))  << input->RepoDict["total_InterceptionSnowWE_viaSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_SnowWE_Sub_Tree_mm" << setw((70 - size("Total_SnowWE_Sub_Tree_mm")))  << folder->VarDict["totalTreeEvapSWE"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_SnowWE_Sub_SVeg_mm" << setw((70 - size("Total_SnowWE_Sub_SVeg_mm")))  << folder->VarDict["totalShortVegEvapSWE"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_SnowWE_Sub_UnderTree_mm" << setw((70 - size("Total_SnowWE_Sub_UnderTree_mm")))  << input->RepoDict["total_SnowSublimation_UnderTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_SnowWE_Sub_UnderShortVeg_mm" << setw((70 - size("Total_SnowWE_Sub_UnderShortVeg_mm")))  << input->RepoDict["total_SnowSublimation_UnderSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_SnowWE_Sub_OpenArea_mm" << setw((70 - size("Total_SnowWE_Sub_OpenArea_mm")))  << input->RepoDict["total_SnowSublimation_OpenArea_m"] * 1000 * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac + ShortVegCover_frac)) << endl;
	WaterBudget << "Total_SnowWE_Melt_TreeCanopy_mm" << setw((70 - size("Total_SnowWE_Melt_TreeCanopy_mm")))  << input->RepoDict["total_SnowMelt_TreeCanopy_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_SnowWE_Melt_SVegCanopy_mm" << setw((70 - size("Total_SnowWE_Melt_SVegCanopy_mm")))  << input->RepoDict["total_SnowMelt_SVegCanopy_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_SnowWE_Melt_UnderTree_mm" << setw((70 - size("Total_SnowWE_Melt_UnderTree_mm")))  << input->RepoDict["total_SnowMelt_UnderTree_m"] * 1000 * (TreePerviousCover_frac + TreeImperviousCover_frac) << endl;
	WaterBudget << "Total_SnowWE_Melt_UnderShortVeg_mm" << setw((70 - size("Total_SnowWE_Melt_UnderShortVeg_mm"))) << input->RepoDict["total_SnowMelt_UnderSVeg_m"] * 1000 * ShortVegCover_frac << endl;
	WaterBudget << "Total_SnowWE_Melt_OpenArea_mm" << setw((70 - size("Total_SnowWE_Melt_OpenArea_mm")))  << input->RepoDict["total_SnowMelt_OpenArea_m"] * 1000 * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac + ShortVegCover_frac)) << endl;
	WaterBudget << "Total_WaterOnPerArea_mm" << setw((70 - size("Total_WaterOnPerArea_mm")))  << input->RepoDict["totalWaterOnPerviousArea_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_WaterOnImpArea_mm" << setw((70 - size("Total_WaterOnImpArea_mm")))  << input->RepoDict["totalWaterOnImperviousArea_m"] * 1000 * TotalImperviousCover_frac << endl;
	WaterBudget << "Total_ImperDepEvap_mm" << setw((70 - size("Total_ImperDepEvap_mm"))) << input->RepoDict["totalImpDepEvap_m"] * 1000 * TotalImperviousCover_frac << endl;
	WaterBudget << "TS_Last_ImpDepStor_mm" << setw((70 - size("TS_Last_ImpDepStor_mm"))) << input->RepoVecDict["totalImpDepStor_TS_m"][totalTimeSteps] * 1000 * TotalImperviousCover_frac << endl;
	//precursor of total_Runoff_Impervious_NR_m, folder->VarDict["Runoff_ImperviousToOutlet_m"], adjusted to catchment area depth in ImperviousDepressionStorageCalc.cpp with Runoff_ImperviousToPervious_m multiplication by ImpAreaNotDirectlyConnected2Outlet_frac
	WaterBudget << "Total_ImperFlowToSoil_mm" << setw((70 - size("Total_ImperFlowToSoil_mm")))  << input->RepoDict["totalImpFlowToSoil_m"] * 1000 << endl;
	//precursor of total_Runoff_Impervious_NR_m, folder->VarDict["Runoff_ImperviousToOutlet_m"], adjusted to catchment area depth in ImperviousDepressionStorageCalc.cpp with Runoff_ImperviousToOutlet_m multiplication by ImpAreaDirectlyConnected2Outlet_frac or TotalImperviousCover_frac
	WaterBudget << "Total_ImpFlowToOutlet_mm" << setw((70 - size("Total_ImpFlowToOutlet_mm"))) << input->RepoDict["totalDCIAFlowToOutlet_m"] * 1000 << endl;
	WaterBudget << "Total_WaterToPerDep_mm" << setw((70 - size("Total_WaterToPerDep_mm")))  << input->RepoDict["totalWaterToSoil_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_PerDepEvap_mm" << setw((70 - size("Total_PerDepEvap_mm")))  << input->RepoDict["totalPerDepEvap_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_Last_PerDepStor_mm" << setw((70 - size("TS_Last_PerDepStor_mm")))  << input->RepoVecDict["totalPerDepStore_TS_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	//Maximum Soil Moisture Deficit (m) x 1000
	WaterBudget << "MSD_mm" << setw((70 - size("MSD_mm"))) << folder->ParamDict["MSD_m"] * 1000 << endl;
	//(m3_p_m3)DWSMI = deficit water soil moisture initial(volumetric moisture content for units)
	WaterBudget << "TS_1st_RootZoneMoisture_unFilled_frac" << setw((70 - size("TS_1st_RootZoneMoisture_unFilled_frac")))  << folder->ParamDict["DWSMI"] << endl;
	//computed from SD0 and MSD, not adjusted to bulk area
	WaterBudget << "TS_1st_RootZoneMoisture_unFilled_mm" << setw((70 - size("TS_1st_RootZoneMoisture_unFilled_mm")))  << ((folder->ParamDict["SD0_percent"] / 100) * folder->ParamDict["MSD_m"]) * 1000 << endl;
	WaterBudget << "TS_1st_RootZoneStor_mm" << setw((70 - size("TS_1st_RootZoneStor_mm")))  << input->RepoVecDict["Storage_Used_RootZone_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_Last_RootZoneStor_mm" << setw((70 - size("TS_Last_RootZoneStor_mm"))) << input->RepoVecDict["Storage_Used_RootZone_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_1st_UnsatZoneStor_mm" << setw((70 - size("TS_1st_UnsatZoneStor_mm"))) << input->RepoVecDict["Storage_Used_UnsatZone_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_Last_UnsatZoneStor_mm" << setw((70 - size("TS_Last_UnsatZoneStor_mm"))) << input->RepoVecDict["Storage_Used_UnsatZone_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_1st_AveSMD_mm" << setw((70 - size("TS_1st_AveSMD_mm")))  << input->RepoVecDict["aveSMD_m"][0] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "TS_Last_AveSMD_mm" << setw((70 - size("TS_Last_AveSMD_mm"))) << input->RepoVecDict["aveSMD_m"][totalTimeSteps] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_FlowToInfil_mm" << setw((70 - size("Total_FlowToInfil_mm"))) << input->RepoDict["total_Water_forSoilRecharge_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_PMacroFlow_mm" << setw((70 - size("Total_PMacroFlow_mm")))  << input->RepoDict["total_Drainage_macroPore_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_Infiltration_mm" << setw((70 - size("Total_Infiltration_mm"))) << input->RepoDict["total_Infiltration_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_InfilExInfil_mm" << setw((70 - size("Total_InfilExInfil_mm"))) << input->RepoDict["total_InfilExInfil_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_SatEx_Infil_mm" << setw((70 - size("Total_SatEx_Infil_mm"))) << input->RepoDict["total_SatEx_Infil_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_Infil_Excess_RO_mm" << setw((70 - size("Total_Infil_Excess_RO_mm"))) << input->RepoDict["totalInfilExQ_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_Sat_Excess_RO_mm" << setw((70 - size("Total_Sat_Excess_RO_mm")))  << input->RepoDict["totalSatExQ_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_RootZoneET_mm" << setw((70 - size("Total_RootZoneET_mm"))) << input->RepoDict["totalRootZoneEvapoTranspiration_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_RootZone_Flow_mm" << setw((70 - size("Total_RootZone_Flow_mm"))) << input->RepoDict["total_Drainage_RootZone_m"] * 1000 * TotalPerviousCover_frac << endl;
	WaterBudget << "Total_UnsatZone_Flow_mm" << setw((70 - size("Total_UnsatZone_Flow_mm"))) << input->RepoDict["total_unsatZoneFlow_m"] * 1000 * TotalPerviousCover_frac << endl;
	//precursor of total_Runoff_Pervious_NR_m, (SatExQ + infilExQ), adjusted to folder area depth in FlowSumCalc.cpp with multiplication of PAQ with PerviousAreaOfFolder_frac
	WaterBudget << "Total_Pervious_Flow_NonRouted_mm" << setw((70 - size("Total_Pervious_Flow_NonRouted_mm"))) << input->RepoDict["total_Runoff_Pervious_NR_m"] * 1000 << endl;
	//precursor of total_Runoff_Impervious_NR_m, folder->VarDict["Runoff_ImperviousToOutlet_m"], adjusted to folder area depth in in ImperviousDepressionStorageCalc.cpp with multiplication of PAQ with folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] 
	WaterBudget << "Total_Impervious_Flow_NonRouted_mm" << setw((70 - size("Total_Impervious_Flow_NonRouted_mm"))) << input->RepoDict["total_Runoff_Impervious_NR_m"] * 1000 << endl;
	// should be adjusted to from pervious area, once we update topographic index
	WaterBudget << "Total_SubsurfaceFlow_NonRouted_mm" << setw((70 - size("Total_SubsurfaceFlow_NonRouted_mm"))) << input->RepoDict["total_Runoff_Subsurface_NR_m"] * 1000 << endl;
	// adjusted to folder area depth by adjusting input->RepoDict["total_Runoff_Pervious_NR_m"] and input->RepoDict["total_Runoff_Impervious_NR_m"] 
	WaterBudget << "Total_Flow_NonRouted_mm" << setw((70 - size("Total_Flow_NonRouted_mm"))) << input->RepoDict["total_Runoff_All_NR_m"] * 1000 << endl;
	// adjusted to bulk area depth in FlowSumCalc.cpp with multiplication of PAQ with PerviousAreaOfFolder_frac
	WaterBudget << "Total_Pervious_Flow_Routed_mm" << setw((70 - size("Total_Pervious_Flow_Routed_mm"))) << input->RepoDict["total_Runoff_Pervious_R_m"] * 1000 << endl;
	// adjusted to bulk area depth in ImperviousDepressionStorageCalc.cpp with multiplication of PAQ with folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"]
	WaterBudget << "Total_Impervious_Flow_Routed_mm" << setw((70 - size("Total_Impervious_Flow_Routed_mm"))) << input->RepoDict["total_Runoff_Impervious_R_m"] * 1000 << endl;
	WaterBudget << "Total_SubsurfaceFlow_Routed_mm" << setw((70 - size("Total_SubsurfaceFlow_Routed_mm"))) << input->RepoDict["total_Runoff_Subsurface_R_m"] * 1000 << endl;
	WaterBudget << "Total_Flow_Routed_mm" << setw((70 - size("Total_Flow_Routed_mm"))) << input->RepoDict["total_Runoff_R_m"] * 1000 << endl;
	WaterBudget << "GI_Total_Inflow_mm" << setw((70 - size("GI_Total_Inflow_mm"))) << folder->VarDict["GI_TotalInflow_q0_m"] * 1000 << endl;
	WaterBudget << "GI_Total_SurfaceOutflow_mm" << setw((70 - size("GI_Total_SurfaceOutflow_mm"))) << folder->VarDict["GI_TotalSurfaceOutflow_q1_m"] * 1000 << endl;
	WaterBudget << "GI_Total_SurfaceInfil_mm" << setw((70 - size("GI_Total_SurfaceInfil_mm"))) << folder->VarDict["GI_TotalInfiltration_f1_m"] * 1000 << endl;
	WaterBudget << "GI_Delta_Surfacedepth_mm" << setw((70 - size("GI_Delta_Surfacedepth_mm"))) << folder->VarDict["GI_DeltaSurfaceDepth_d1_m"] * 1000 << endl;
	WaterBudget << "GI_SurfaceBudget_mm" << setw((70 - size("GI_SurfaceBudget_mm"))) << folder->VarDict["GI_SurfaceWaterBudget_m"] * 1000 << endl;
	WaterBudget << "GI_TotalBudget_mm" << setw((70 - size("GI_TotalBudget_mm"))) << folder->VarDict["GI_TotalWaterBudget_m"] * 1000 << endl;

	//Write out specific water balances for each component of vertical system
	WaterBudget <<  endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Water Balance Equations (Eq.): " << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << " Eq. 1 Tree Canopy water balance (mm), relative to tree canopy area" << endl;
	WaterBudget << " TC_WaterBalance_mm = TC_Total_Precip_RainSWE_mm - TC_StorageFinal_Canopy_mm - TC_Evap_mm - TC_ThroughFall_mm " <<  endl;
	WaterBudget << " = " << TC_Total_Precip_RainSWE_mm <<" - " <<TC_StorageFinal_Canopy_mm << " - " << TC_Evap_mm << "- " << TC_ThroughFall_mm << endl;
	WaterBudget << " = " << TC_WaterBalance_mm << endl;
	WaterBudget << endl;
	
	WaterBudget << " Eq. 2 Short Veg Canopy water balance (mm), relative to short vegetation area" << endl;
	WaterBudget << " SV_WaterBalance_mm = SV_Total_Precip_RainSWE_mm - SV_StorageFinal_Canopy_mm - SV_Evap_mm - SV_ThroughFall_mm  " <<  endl;
	WaterBudget << " = " << SV_Total_Precip_RainSWE_mm << " - " << SV_StorageFinal_Canopy_mm << " - " << SV_Evap_mm  << " -"  << SV_ThroughFall_mm  << endl;
	WaterBudget << " = " << SV_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 3 Canopy Water balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_TC_SV_WaterBalance_mm = TC_SV_Total_Precip_RainSWE_mm - CA_TC_SV_StorageFinal_Canopy_mm - CA_TC_SV_Evap_mm - CA_TC_SV_ThroughFall_mm  " << endl;
	WaterBudget << " = " << TC_SV_Total_Precip_RainSWE_mm << " - " << CA_TC_SV_StorageFinal_Canopy_mm << " - " << CA_TC_SV_Evap_mm <<" - " << CA_TC_SV_ThroughFall_mm << endl;
	WaterBudget << " = " << CA_TC_SV_WaterBalance_mm << endl;
	WaterBudget << endl; 
	
	WaterBudget << " Eq. 4 Impervious Area Water Atmospheric Inputs balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_IA_Input_WaterBalance_mm = CA_IA_TC_Throughfall_mm + CA_IA_NoTC_Precip_mm - CA_IA_TC_Sublimation_SnowWE_mm - CA_IA_TC_StorageFinal_Ground_SnowWE_mm - CA_IA_NoTC_Sublimation_SnowWE_mm - CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm - CA_IA_WaterOn_mm " << endl;
	WaterBudget << " = " << CA_IA_TC_Throughfall_mm << " + " << CA_IA_NoTC_Precip_mm << " - " << CA_IA_TC_Sublimation_SnowWE_mm << " - " <<  CA_IA_TC_StorageFinal_Ground_SnowWE_mm << " - " << CA_IA_NoTC_Sublimation_SnowWE_mm << " - " << CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm << " - " << CA_IA_WaterOn_mm << endl; 
	WaterBudget << " = " << CA_IA_Input_WaterBalance_mm << endl;
	WaterBudget << endl; 
	
	WaterBudget << " Eq. 5 Impervious Area Water Storage + Atmospheric Outputs balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_IA_AtmosphericOutputs_WaterBalance_mm = CA_IA_WaterOn_mm - CA_IA_StorageFinal_Depression_mm - CA_IA_Evaporation_Depression_mm - CA_IA_Runoff_To_PA_mm - CA_IA_Runoff_To_DCIA_mm " << endl;
	WaterBudget << " = " << CA_IA_WaterOn_mm << " - " << CA_IA_StorageFinal_Depression_mm << " - " << CA_IA_Evaporation_Depression_mm << " - " << CA_IA_Runoff_To_PA_mm << " - " << CA_IA_Runoff_To_DCIA_mm << endl; 
	WaterBudget << " = " << CA_IA_AtmosphericOutputs_WaterBalance_mm << endl;
	WaterBudget << endl; 
	
	WaterBudget << " Eq. 6 Impervious Area non-routed Runoff balance (mm), relative to catchment area"<< endl;
	WaterBudget << " CA_IA_RunOff_WaterBalance_mm = CA_IA_Runoff_To_DCIA_mm - CA_IA_Runoff_NR_mm " << endl;
	WaterBudget << " = " << CA_IA_Runoff_To_DCIA_mm << " - " << CA_IA_Runoff_NR_mm  << endl;
	WaterBudget << " = " << CA_IA_RunOff_WaterBalance_mm << endl;
	WaterBudget << endl; 
	
	WaterBudget << " Eq. 7 Pervious Area Water Atmospheric Inputs balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_AtmosphericInputs_WaterBalance_mm = (CA_PA_TC_ThroughFall_mm + CA_SV_ThroughFall_mm + CA_PA_NoTC_NoSV_Precip_mm) - CA_PA_TC_Sublimation_SnowWE_mm - CA_PA_TC_StorageFinal_Ground_SnowWE_mm - CA_SV_Sublimation_SnowWE_mm - CA_SV_StorageFinal_Ground_SnowWE_mm - CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm -  CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm - CA_PA_WaterOn_mm " << endl;
	WaterBudget << " = "  << "(" << CA_PA_TC_ThroughFall_mm << " + " << CA_SV_ThroughFall_mm << " + " << CA_PA_NoTC_NoSV_Precip_mm << ")" << " - " << CA_PA_TC_Sublimation_SnowWE_mm << " - " << CA_PA_TC_StorageFinal_Ground_SnowWE_mm << " - " << CA_SV_Sublimation_SnowWE_mm << " - " << CA_SV_StorageFinal_Ground_SnowWE_mm << " - " << CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm << " - " << CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm << " - " << CA_PA_WaterOn_mm  << endl;
	WaterBudget << " = " << CA_PA_AtmosphericInputs_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 8 Pervious Area Water Atmospheric + Runon Inputs balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_AtmosphericInputs_Runon_WaterBalance_mm = CA_PA_Flux_PrecipAndRunoffToSoil_mm - CA_PA_WaterOn_mm - CA_PA_Runon_From_IA_mm" << endl;
	WaterBudget << " = " << CA_PA_Flux_PrecipAndRunoffToSoil_mm << " - " << CA_PA_WaterOn_mm << " - " << CA_PA_Runon_From_IA_mm << endl;
	WaterBudget << " = " << CA_PA_AtmosphericInputs_Runon_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 9 Pervious Area Water Storage + Atmospheric Outputs + Infiltration balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_WaterToInfilt_WaterBalance_mm = CA_PA_Flux_PrecipAndRunoffToSoil_mm - CA_PA_StorageFinal_Depression_mm - CA_PA_Evaporation_Depression_mm - CA_PA_Water_To_Infil_mm " << endl;
	WaterBudget << " = " << CA_PA_Flux_PrecipAndRunoffToSoil_mm << " - " << CA_PA_StorageFinal_Depression_mm << " - " << CA_PA_Evaporation_Depression_mm << " - " << CA_PA_Water_To_Infil_mm  << endl;
	WaterBudget << " = " << CA_PA_WaterToInfilt_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 10 Pervious Area Water to Infiltration + Runoff + Macroporeflow balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_Infiltration_WaterBalance_mm = CA_PA_Flux_PrecipAndRunoffToSoil_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm - CA_PA_Drainage_Macropore_mm " << endl;
	WaterBudget << " = " << CA_PA_Flux_PrecipAndRunoffToSoil_mm << " - " << CA_PA_Runoff_InfilExcess_mm << " - " << CA_PA_Runoff_SatExcess_mm << " - " << CA_PA_Infiltration_InfilExcess_mm << " - " << CA_PA_Infiltration_SatExcess_mm << " - " << CA_PA_Drainage_Macropore_mm  << endl;
	WaterBudget << " = " << CA_PA_Infiltration_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 11 Pervious Area Infiltration routines balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_InfiltComponents_WaterBalance_mm = CA_PA_Infiltration_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm" << endl;
	WaterBudget << " = " << CA_PA_Infiltration_mm << " - " << CA_PA_Infiltration_InfilExcess_mm << " - " << CA_PA_Infiltration_SatExcess_mm << endl;
	WaterBudget << " = " << CA_PA_InfiltComponents_WaterBalance_mm << endl;
	WaterBudget << endl;
	
	WaterBudget << " Eq. 12 Pervious Area non-routed Runoff routines (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_WaterToInfilt_WaterBalance_NR_mm = CA_PA_Runoff_NR_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm" << endl;
	WaterBudget << " = " << CA_PA_Runoff_NR_mm << " - " << CA_PA_Runoff_InfilExcess_mm << " - " << CA_PA_Runoff_SatExcess_mm << endl;
	WaterBudget << " = " << CA_PA_WaterToInfilt_WaterBalance_NR_mm << endl;
	WaterBudget << endl;
	
	WaterBudget << " Eq. 13 Pervious Area Root Zone balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_RootZone_WaterBalance_mm = CA_PA_Infiltration_mm + CA_PA_Drainage_Macropore_mm - CA_PA_StorageChange_RootZone_mm - CA_PA_EvapTranspiration_RootZone_mm - CA_PA_Drainage_RootZone_mm" << endl;
	WaterBudget << " = " << CA_PA_Infiltration_mm << " + " << CA_PA_Drainage_Macropore_mm << " - " << CA_PA_StorageChange_RootZone_mm << " - " << CA_PA_EvapTranspiration_RootZone_mm << " - " << CA_PA_Drainage_RootZone_mm << endl; 
	WaterBudget << " = " << CA_PA_RootZone_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 14 Pervious Area Unsaturated Zone balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_UnsatZone_WaterBalance_mm = CA_PA_Drainage_RootZone_mm - CA_PA_Drainage_UnsatZone_mm - CA_PA_StorageChange_UnsatZone_mm " << endl;
	WaterBudget << " = " << CA_PA_Drainage_RootZone_mm << " - " << CA_PA_Drainage_UnsatZone_mm << " - " << CA_PA_StorageChange_UnsatZone_mm  << endl;
	WaterBudget << " = " << CA_PA_UnsatZone_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 15 Pervious Area Saturated Zone balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_PA_SubsurfaceFlow_WaterBalance_mm = CA_PA_StorageDeficit_AvgSoil_1st_mm - CA_PA_Drainage_UnsatZone_mm + CA_PA_Runoff_Subsurface_mm - CA_PA_StorageDeficit_AvgSoil_Last_mm" << endl;
	WaterBudget << " = " << CA_PA_StorageDeficit_AvgSoil_1st_mm << " - " << CA_PA_Drainage_UnsatZone_mm << " + " << CA_PA_Runoff_Subsurface_mm << " - " << CA_PA_StorageDeficit_AvgSoil_Last_mm << endl;
	WaterBudget << " = " << CA_PA_SubsurfaceFlow_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 16 Total Snow Water Equivalent balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_SnowWE_WaterBalance_mm = CA_Precip_SnowWE_mm - CA_Total_Sublimation_Ground_SnowWE_mm - CA_Total_Sublimation_Canopy_SnowWE_mm - CA_Total_Melt_Ground_SnowWE_mm - CA_Total_Melt_Canopy_SnowWE_mm - CA_Total_Storage_Ground_SnowWE_mm - CA_TS_Last_Canopy_SnowWE_mm" << endl;
	WaterBudget << " = " << CA_Precip_SnowWE_mm << " - " << CA_Total_Sublimation_Ground_SnowWE_mm << " - " << CA_Total_Sublimation_Canopy_SnowWE_mm << " - " << CA_Total_Melt_Ground_SnowWE_mm << " - " << CA_Total_Melt_Canopy_SnowWE_mm << " - " << CA_Total_Storage_Ground_SnowWE_mm << " - " << CA_TS_Last_Canopy_SnowWE_mm << endl;
	WaterBudget << " = " << CA_SnowWE_WaterBalance_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 17 Total Surface Water balance NonRouted (mm), relative to catchment area" << endl;
	WaterBudget << " CA_Total_WaterBalance_NR_mm = CA_Total_Precip_RainSWE_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_Drainage_ToSoil_mm - CA_Total_Runoff_Surface_NR_mm" << endl;
	WaterBudget << " = " << CA_Total_Precip_RainSWE_mm << " - " << CA_Total_EvapoSublimation_Canopy_mm << " - " << CA_Total_EvapoSublimation_Ground_mm << " - " << CA_Total_StorageFinal_Canopy_mm << " - " << CA_Total_StorageFinal_Ground_mm << " - " << CA_Total_Drainage_ToSoil_mm << " - " << CA_Total_Runoff_Surface_NR_mm << endl;
	WaterBudget << " = " << CA_Total_WaterBalance_NR_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 18 Total Surface and Subsurface Water balance NonRouted (mm), relative to catchment area" << endl;
	WaterBudget << " CA_Total_WaterBalance_R_mm = CA_Total_Precip_RainSWE_mm - CA_Total_EvapoTranspiration_Soil_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_StorageChange_Subsurface_mm - CA_Total_Runoff_NR_mm" << endl;
	WaterBudget << " = " << CA_Total_Precip_RainSWE_mm << " - " << CA_Total_EvapoTranspiration_Soil_mm << " - " << CA_Total_EvapoSublimation_Canopy_mm << " - " << CA_Total_EvapoSublimation_Ground_mm << " - " << CA_Total_StorageFinal_Canopy_mm << " - " << CA_Total_StorageFinal_Ground_mm << " - " << CA_Total_StorageChange_Subsurface_mm << " - " << CA_Total_Runoff_NR_mm << endl;
	WaterBudget << " = " << CA_Total_WaterBalance_R_mm << endl;
	WaterBudget << endl;

	WaterBudget << " Eq. 19.Runoff NonRoutedRouted vs  Balance (mm), relative to catchment area" << endl;
	WaterBudget << " CA_RunoffRouting_Balance_mm = (Total_Pervious_Flow_NonRouted_mm - Total_Pervious_Flow_Routed_mm) + (Total_Impervious_Flow_NonRouted_mm - Total_Impervious_Flow_Routed_mm) + (Total_SubsurfaceFlow_NonRouted_mm - Total_SubsurfaceFlow_Routed_mm) + (Total_Flow_NonRouted_mm - Total_Flow_Routed_mm)" << endl;
	WaterBudget << " = " << "(" << Total_Pervious_Flow_NonRouted_mm << " - " << Total_Pervious_Flow_Routed_mm << ")" << "+" << "(" << Total_Impervious_Flow_NonRouted_mm << " - " << Total_Impervious_Flow_Routed_mm << ")" << "+" << "(" << Total_SubsurfaceFlow_NonRouted_mm << " - " << Total_SubsurfaceFlow_Routed_mm << ")" << "+" << "("<< Total_Flow_NonRouted_mm << " - " << Total_Flow_Routed_mm<< ")" << endl;
	WaterBudget << " = " << CA_RunoffRouting_Balance_mm << endl;
	WaterBudget << endl;

	WaterBudget << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Topographic Index Calculation Approach (Exponential or Power Decay) and Parameters:" << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;

	//References for Topographic Index theory
	//Famiglietti, J.S., & Wood, E.F. (1994).Multiscale Modeling of Spatially Variable Waterand Energy Balance Processes.Water Resources Research, 30(11), 3061 - 3078.
	//Hornberger, G. M., Beven, K. J., Cosby, B. J., & Sappington, D. E. (1985). Shenandoah Watershed Study: Calibration of a Topography-Based, Variable Contributing Area Hydrological Model to a Small Forested Catchment. Water Resources Research, 21(12), 1841-1850.
	//TMOD9502.FOR TOPMODEL DEMONSTRATION PROGRAM VERSION 95.02, By Keith Beven or Eq given as misc.S_mean[0] = - params.m * log(misc.qs0 / misc.qss) in param.init.c https://cran.r-project.org/web/packages/topmodel/index.html
	//Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834.
	//Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 


	double SZQ, T0_m2_p_dt, TopographicIndex_Avg, AveSMD, m_ScalingParam, Q0_m_p_dt, n_PowerDecayParam;
	std::vector<double> localSMD(input->SimulationNumericalParams["TopographicIndexBins"]);
	Q0_m_p_dt = folder->ParamDict["Q0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	T0_m2_p_dt = folder->ParamDict["T0_m2ph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	TopographicIndex_Avg = input->AverageIndex;
	m_ScalingParam = folder->ParamDict["m"];

	//Infiltration type for decay function with hydraulic conductivity and computation of topographic index
	if (input->SimulationStringParams["Infiltration"] == "ExponentialDecay") {
		WaterBudget << "HydraulicConductivityDecay_function" << setw((70 - size("HydraulicConductivityDecay_function"))) << "ExponentialDecay" << endl;
		//SZQ(m) is maximum potential subsurface flow rate, when groundwater table is at the ground surface
		//Eq for SZQ similar to Eq 31b for Q0 in Famigliettiand Wood(1994)
		//Eq for SZQ below Eq 3 of Wang et al. (2005), S_bar = -m * ln(R / T0) - m * Lambda, and SZQ becomes T0* exp(-Lambda), where R = Q0
		SZQ = T0_m2_p_dt * exp(-1 * TopographicIndex_Avg);

		//Eq for AveSMD derived from Eq 4 in Horberger et al. (1985) Qb = SZQ * EXP(-S / SZM)
		//Eq for AveSMD Eq given as SBAR = -SZM * ALOG(Q0 / SZQ) in TMOD9502.FOR or param.init.c
		//Eq for AveSMD below Eq 3 of Wang et al. (2005), S_bar = -m * ln(R / T0) - m * Lambda
		//AveSMD computation uses Natural log of(Q0_m_p_dt / SZQ)
		//AveSMD adjusted to TotalPerviousCover_frac, which is fraction of catchment with soil moisture
		AveSMD = -1 * m_ScalingParam * log(Q0_m_p_dt / SZQ) * TotalPerviousCover_frac;
	}
	if (input->SimulationStringParams["Infiltration"] == "PowerDecay") {
		WaterBudget << "HydraulicConductivityDecay_function" << setw((70 - size("HydraulicConductivityDecay_function"))) << "PowerDecay" << endl;
		n_PowerDecayParam = folder->ParamDict["n"];
		//SZQ(m) is maximum potential subsurface flow rate, when groundwater table is at the ground surface
		//Eq for SZQ is RHS of Eq for S_bar, aveSMD, given in text below Eq 9 of Wang et al. (2006)
		SZQ = pow((Q0_m_p_dt / T0_m2_p_dt), (1 / n_PowerDecayParam)) * TopographicIndex_Avg;

		//Eq for aveSMD is below Eq 9 of Wang et al. (2006) and below Eq 5 of Wang(2005)
		//AveSMD (m) adjusted to TotalPerviousCover_frac, which is fraction of catchment with soil moisture
		AveSMD = m_ScalingParam * (1 - SZQ) * TotalPerviousCover_frac;
	}
	//Timestep for adjusting rate constants from HydroPlusConfig.xml
	WaterBudget << "TimeStep_sec (s)" << setw((70 - size("TimeStep_sec (s)"))) << folder->ParamDict["TimeStep_sec"] << endl;
	//Number of topographic index bins
	WaterBudget << "TopographicIndexBins (#)" << setw((70 - size("TopographicIndexBins (#)"))) << std::fixed << input->SimulationNumericalParams["TopographicIndexBins"] << endl;
	//Average topographic index value
	WaterBudget << "TopographicIndex_Avg" << setw((70 - size("TopographicIndex_Avg"))) << std::fixed << input->AverageIndex << endl;
	//Scaling parameter for adjusting soil moisture across topographic index
	WaterBudget << "m_ScalingParam" << setw((70 - size("m_ScalingParam"))) << std::scientific << folder->ParamDict["m"] << endl;
	//Power decay parameter for adjusting conductivity across topographic index
	WaterBudget << "n_PowerDecayParam" << setw((70 - size("n_PowerDecayParam"))) << std::fixed << folder->ParamDict["n"] << endl;
	//Transmissivity (m^2/dt) at catchment saturation
	WaterBudget << "T0_m2_p_dt (m/h)" << setw((70 - size("T0_m2_p_dt (m/h)"))) << std::scientific << folder->ParamDict["T0_m2ph"] * folder->ParamDict["TimeStep_sec"] / 3600 << endl;
	//Subsurface flow (m/dt) at start of simulation
	WaterBudget << "Q0_m_p_dt (m/h)" << setw((70 - size("Q0_m_p_dt (m/h)"))) << std::scientific << folder->ParamDict["Q0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600 << endl;
	//SZQ (m/hr) is maximum discharge from catchment pervious area when soil storage full saturated
	WaterBudget << "SZQ (m/ts)" << setw((70 - size("SZQ (m/ts)"))) << std::scientific << SZQ << endl;
	//AveSMD (m) is catchment average soil moisture deficit for water storage; multiplied by 1000 to convert m to mm for output display
	WaterBudget << "AveSMD (mm) TS 1st" << setw((70 - size("AveSMD (mm) TS 1st"))) << std::scientific << AveSMD * 1000 << endl;
	WaterBudget << endl;

	WaterBudget.precision(4);

	//Loop through all topographic index bins, write out TI values; histogram of TI bins ordered from maximum and wettest to minimum and driest
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Topographic Index Values for each Histogram Bin, sorted from Wettest to Driest" << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	for (int ia = 0; ia < input->IndexSize; ia++) {
		WaterBudget << "TopographicIndex_" << ia << setw((70 - size("TopographicIndex_"))) << std::fixed << input->Index[ia] << endl;
	}
	WaterBudget << endl;

	//Loop through all topographic index bins, write out TI value fractional areas; histogram of TI bins ordered from maximum and wettest to minimum and driest
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Topographic Index Value Fractional Areas for each Histogram Bin, sorted from Wettest to Driest" << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	for (int ia = 0; ia < input->IndexSize; ia++) {
		WaterBudget << "TopographicIndexArea_f_" << ia << setw((70 - size("TopographicIndexArea_f_00"))) << std::scientific << input->IndexArea[ia] << endl;
	}
	WaterBudget << endl;

	//Loop through all topographic index bins, write out localSMD values; histogram of TI bins ordered from maximum and wettest to minimum and driest
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	WaterBudget << "Local Soil Moisture Deficit Values for each Histogram Bin, sorted from Wettest to Driest" << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;
	for (int ia = 0; ia < input->IndexSize; ia++) {
		localSMD[ia] = AveSMD + m_ScalingParam * (TopographicIndex_Avg - input->Index[ia]);
		WaterBudget << "localSMD[" << ia << "]" << setw((70 - size("localSMD[00]"))) << std::scientific << localSMD[ia] << endl;
	}
	WaterBudget << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;

	//Groundwater_surficial_frac > 0.05 used to determine message about concern for excessive saturated area, using 0.05 as arbitrary threshold
	if (input->RepoVecDict["Groundwater_surficial_frac"][totalTimeSteps] >= 0.05) {
		WaterBudget << " Note: The simulation ended with " << std::fixed << input->RepoVecDict["Groundwater_surficial_frac"][totalTimeSteps] * 100 << "% of the catchment with groundwater at the soil surface, which is tracked by parameter TS_Last_SurficialGroundwater_frac in the Model Variable list above." <<
			" The above list of Local Soil Moisture Deficit Values for each Topographic Index (TI) bin contains negative values and may erroneously over-represent saturated areas and its runoff." << endl;
	}
	//localSMD[1] and input->IndexArea[0] used to determine message about concern some saturated area, using 0 to 0.0499 as arbitrary range for accurate simulation
	else if (localSMD[1] < 0 and input->IndexArea[1] > 0) {
		WaterBudget << " Note: The simulation ended with between 0 and 5% of the catchment with groundwater at the soil surface, which is tracked by parameter TS_Last_SurficialGroundwater_frac in the Model Variable list above." <<
			" The above list of Local Soil Moisture Deficit Values for each Topographic Index (TI) bin contains negative values and may accurately represent saturated areas and its runoff." << endl;
	}
	//localSMD[1] used to determine message about no saturated area, presuming some catchment area should be saturated (e.g., river channel area)
	else if (localSMD[1] >= 0) {
		WaterBudget << " Note: The simulation ended with 0% of the catchment with groundwater at the soil surface, which is tracked by parameter TS_Last_SurficialGroundwater_frac in the Model Variable list above." <<
			" The above list of Local Soil Moisture Deficit Values for each Topographic Index (TI) bin with any fractional area contain no negative values and may erroneously under-represent saturated areas and its runoff." << endl;
	}
	//Potential that localSMD[1] > 0 and localSMD[0] < 0 to determine message about uncertain but likely small saturated area, presuming some catchment area should be saturated (e.g., river channel area)
	else {
		WaterBudget << " Note: The simulation may have ended with little to none of the catchment with groundwater at the soil surface, which is tracked by parameter TS_Last_SurficialGroundwater_frac in the Model Variable list above." <<
			" The above list of Local Soil Moisture Deficit Values for each Topographic Index (TI) bin with any fractional area contain no negative values and may erroneously under-represent saturated areas and its runoff." << endl;
	}
	WaterBudget << endl;
	WaterBudget << " Note: If the you wish to adjust the fraction of catchment with surficial groundwater and saturation excess runoff by adjusting localSMD values, options include:" << endl;
	WaterBudget << "      1. Adjust T0_m2_p_dt; a larger T0_m2_p_dt input reduces the number TI bins with negative localSMD values, a smaller value increases the number TI bins with negative localSMD values; T0_m2_p_dt values can range order of magnitudes (e.g., 1E-3 to 1E3)." << endl;
	WaterBudget << "      2. Adjust TI bins; reducing the number of TI bins between the maximum and average TI values reduces number of TI bins with negative localSMD values, increasing has an opposite effect; the total fraction of area with negative or positive localSMD may remain the same." << endl;
	WaterBudget << "      3. Adjust DEM resolution; a larger DEM grid size may reduce the range between the maximum and average TI values, which reduces number of TI bins with negative localSMD values; finer resolution DEM grid sizes could have the opposite effect." << endl;
	WaterBudget << "	  4. Adjust m_ScalingParam; a smaller m_ScalingParam input reduces the local SMD value magnitude, but not the sign or number of TI bins with negative or positive localSMD values." << endl;
	WaterBudget << endl;
	WaterBudget << " --------------------------------------------------------------------------------------------------" << endl;

	WaterBudget << " References:" << endl;
	WaterBudget << " Famiglietti, J.S., & Wood, E.F. (1994).Multiscale Modeling of Spatially Variable Waterand Energy Balance Processes.Water Resources Research, 30(11), 3061 - 3078.:" << endl;
	WaterBudget << " Hornberger, G. M., Beven, K. J., Cosby, B. J., & Sappington, D. E. (1985). Shenandoah Watershed Study: Calibration of a Topography-Based, Variable Contributing Area Hydrological Model to a Small Forested Catchment. Water Resources Research, 21(12), 1841-1850." << endl;
	WaterBudget << " TMOD9502.FOR TOPMODEL DEMONSTRATION PROGRAM VERSION 95.02, By Keith Beven or Eq given as misc.S_mean[0] = - params.m * log(misc.qs0 / misc.qss) in param.init.c https://cran.r-project.org/web/packages/topmodel/index.html" << endl;
	WaterBudget << " Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. " << endl;
	WaterBudget << " Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. " << endl;
	WaterBudget.close();


}



void StatisticalOutputWriter::writeCaliQobs(Inputs *input, std::string indirectory){
	ofstream outfileCaliQobs(input->SimulationStringParams["OutputDirectory"] + "CaliQobs.dat");
	if (!outfileCaliQobs.good())
	{
		LogMessage::log("Could not create: " + input->SimulationStringParams["OutputDirectory"] + "CaliQobs.dat");
		return;
	}

	calibrate _qobs(indirectory, input->SimulationNumericalParams["StartDay"], input->SimulationNumericalParams["EndDay"]);
	std::vector<double>& caliQobs = _qobs.getQobs();
	caliQobs = convertToTimeStep(caliQobs, input->SimulationStringParams["CalibrationTimeStep"]);//chk

	outfileCaliQobs << caliQobs.size() << endl;
	for (unsigned int i = 0; i<caliQobs.size(); i++)
		outfileCaliQobs << setiosflags(ios::fixed)
		<< setiosflags(ios::showpoint)
		<< setprecision(9)
		<< caliQobs[i] << endl;

	outfileCaliQobs.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "CaliQobs.dat");
}

void StatisticalOutputWriter::writeCaliTotalQQ(Inputs *input){
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "CaliTotalQQ.dat");
	if (!outfile.good())
	{
		LogMessage::log("Could not create: "+ input->SimulationStringParams["OutputDirectory"] + "CaliTotalQQ.dat");
		return;
	}

	std::vector<double> caliTotalQQ = input->QQi;
	caliTotalQQ = convertToTimeStep(caliTotalQQ, input->SimulationStringParams["CalibrationTimeStep"]);

	outfile << caliTotalQQ.size() << endl;
	for (unsigned int i = 0; i<caliTotalQQ.size(); i++)
		outfile << setiosflags(ios::fixed)
		<< setiosflags(ios::showpoint)
		<< setprecision(9)
		<< caliTotalQQ[i] << endl;

	outfile.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "CaliTotalQQ.dat");
}

std::vector<double> StatisticalOutputWriter::convertToTimeStep(const std::vector<double>& val, const std::string timeStep){
	std::vector<double> tmpVal = val;
	if (timeStep == "Daily" || timeStep == "daily")
	{
		tmpVal = hourlyToDaily(tmpVal);
	}
	if (timeStep == "Weekly" || timeStep == "weekly")
	{
		tmpVal = hourlyToWeekly(tmpVal);
	}
	if (timeStep == "Monthly" || timeStep == "monthly")
	{
		tmpVal = hourlyToMonthly(tmpVal);
	}
	return tmpVal;
}

vector<double> StatisticalOutputWriter::hourlyToDaily(const vector<double>& input)
{
	int dailySize = input.size() / 24;
	vector<double> dailyValue(dailySize);

	for (int i = 0; i<dailySize; i++)
	{
		
		dailyValue[i] = hourlyToDailyVal(input, i);
	}

	return dailyValue;
}

double StatisticalOutputWriter::hourlyToDailyVal(const vector<double>& input, const int i)
{
		double dailySum = 0.0;
		for (int j = 0; j<24; j++)
		{
			dailySum = dailySum + input[i * 24 + j];
		}
		return dailySum;
}

vector<double> StatisticalOutputWriter::hourlyToWeekly(const vector<double>& input)
{
	int weeklySize = input.size() / (24 * 7);
	vector<double> weeklyValue(weeklySize);

	for (int i = 0; i<weeklySize; i++)
	{
		weeklyValue[i] = hourlyToWeeklyVal(input, i);
	}

	return weeklyValue;
}


double StatisticalOutputWriter::hourlyToWeeklyVal(const vector<double>& input, const int i)
{
	double weeklySum = 0.0;
	for (int j = 0; j < (24 * 7); j++)
	{
		weeklySum = weeklySum + input[i * 24 * 7 + j];
	}

	return weeklySum;
}

vector<double> StatisticalOutputWriter::hourlyToMonthly(const vector<double>& input)
{
	int monthlySize = input.size() / (24 * 30);
	vector<double> monthlyValue(monthlySize);
	for (int i = 0; i<monthlySize; i++)
	{
		monthlyValue[i] = hourlyToMonthlyVal(input, i);
	}
	return monthlyValue;
}

double StatisticalOutputWriter::hourlyToMonthlyVal(const vector<double>& input, const int i)
{
	double monthlySum = 0.0; 
	for (int j = 0; j < (24 * 30); j++)
	{
		monthlySum = monthlySum + input[i * 24 * 30 + j];
	}

	return monthlySum;
}

void StatisticalOutputWriter::writePollutionFile(std::string indirectory, Inputs *input)
{
	Pollution poll(input->SimulationNumericalParams["TotalTimeSteps"], input->SimulationNumericalParams["CatchmentArea_m2"], indirectory);
	if (!poll.PollutionDataFileExists)
	{
		LogMessage::log("Pollutiondata.dat can not be found/read. Should be in dir: "+indirectory);
		return;
	}
	std::vector<double> oQi, uQi;
	oQi = input->PAQi;
	uQi = input->DCIAQi;
	poll.calPollutionData(oQi, uQi);
	poll.writePollutiondata(oQi, uQi, input->SimulationStringParams["OutputDirectory"]);
	LogMessage::log("Created " + input->SimulationStringParams["OutputDirectory"] + "Pollution.dat");
}

void StatisticalOutputWriter::writeOutputFile(std::string indirectory,Inputs *input, DataFolder* BulkAreafolder)
{	
	std::string path;
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") { //RC,11/30/2018-in future rename to Output_BulkArea if isolated to BulkArea, then have Output.dat for whole prj area
		path = input->SimulationStringParams["OutputDirectory"] + "Output.dat"; //For now, naming Output.dat so when no GI is run we get output for prj, which GUI & tests expect
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_TreePit.dat";
	}
	//added by Li
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_InfilTrench.dat";
	}

	std::ofstream outfile(path);
	if (!outfile)
	{
		LogMessage::log("Could not create " + path);
		return;
	}

	double TotalTreeCover_frac = BulkAreafolder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + BulkAreafolder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double ShortVegCover_frac = BulkAreafolder->ParamDict["ShortVegetationCover_frac"];
	double TotalCatchmentArea_m2 = input->SimulationNumericalParams["CatchmentArea_m2"];

	double RainInterceptByTree_frac		=	(input->TotalRain_m - input->RepoDict["totalTreeThruFlow_m"])/input->TotalRain_m;
	double RainInterceptByShortVeg_frac	=	(input->TotalRain_m  - input->RepoDict["totalShortVegThruFlow_m"] ) / input->TotalRain_m;
	double RainOverTreeIntercepted_frac		=	(input->TotalRain_m  - input->RepoDict["totalTreeThruFlow_m"] ) * TotalTreeCover_frac/input->TotalRain_m;
	double RainOverShortVegIntercepted_frac	=	(input->TotalRain_m  - input->RepoDict["totalShortVegThruFlow_m"]) * ShortVegCover_frac /input->TotalRain_m ;
   	double RainInterceptByTree_m3		=	(input->TotalRain_m  - input->RepoDict["totalTreeThruFlow_m"] ) * TotalCatchmentArea_m2 * TotalTreeCover_frac;
	double RainInterceptByShortVeg_m3	=	(input->TotalRain_m  - input->RepoDict["totalShortVegThruFlow_m"]) * TotalCatchmentArea_m2 * ShortVegCover_frac ;

	outfile <<" Tree LAI:  "<< BulkAreafolder->ParamDict["TreeLAI"]<<"  Tree  %: " <<TotalTreeCover_frac*100.0<<" % "<<endl
		    <<" Shrub LAI: "<< BulkAreafolder->ParamDict["ShortVegLAI"] <<"  Shrub %: " <<ShortVegCover_frac*100.0 <<" % "<<endl<<endl;

	outfile <<" Total precipitation                                      :   " <<input->TotalPrecip_m*1000<<"mm " <<endl;
    outfile <<" Total rainfall                                           :   " <<input->TotalRain_m*1000<<"mm " <<endl;
    outfile <<" Total snow water equivalent                              :   " <<input->TotalSnow_m*1000<<"mm " <<endl;
    outfile <<" Total tree intercepted rain                              :   " <<input->RepoDict["total_InterceptionRain_viaTree_m"]*1000<<"mm " <<endl;
	outfile <<" Total short vegetation intercepted rain                  :   " <<input->RepoDict["total_InterceptionRain_viaSVeg_m"]*1000<<"mm " <<endl;
	outfile <<" Total tree intercepted snow water equivalent             :   " <<input->RepoDict["total_InterceptionSnowWE_viaTree_m"]*1000<<"mm " <<endl;
	outfile <<" Total short vegetation intercepted snow water equivalent :   " <<input->RepoDict["total_InterceptionSnowWE_viaSVeg_m"]*1000<<"mm " <<endl<<endl;

	std::vector<double> QH;
	std::vector<double> QD;
	std::vector<double> QW;
	std::vector<double> QM;

	// Addition of Timestep change code by Pallavi Iyengar
	if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration"){
		// Addition of Timestep change code by Pallavi Iyengar
		calibrate calQ(indirectory, input->SimulationNumericalParams["StartDay"], input->SimulationNumericalParams["EndDay"]);
		std::vector<double> qV = input->QQi;
		calQ.objFunRunoff(qV);
		// End of Timestep change code by Pallavi Iyengar
		std::vector<double> QHourly = input->QQi;
		std::vector<double> QDaily = hourlyToDaily(QHourly);
		std::vector<double> QWeekly = hourlyToWeekly(QHourly); 

		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector<double> QMonthly = hourlyToMonthly(QHourly);
		// End of Monthly Timestep change code by Isira Samarasekera

		std::vector<double>& QobsHourly = calQ.getQobs();
		std::vector<double>& QobsDaily = calQ.getDailyQobs();
		std::vector<double>& QobsWeekly = calQ.getWeeklyQobs();

		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector<double>& QobsMonthly = calQ.getMonthlyQobs();
		// End of Monthly Timestep change code by Isira Samarasekera
		QH = QobsHourly;
		QD = QobsDaily;
		QW = QobsWeekly;
		QM = QobsMonthly;

		//call calibrate:calibrate with Qobs
		calibrate calQHourly(QHourly, QobsHourly);
		double fCRF1 = calQHourly.getF1();
		double fCRF2 = calQHourly.getF2();
		double fCRF3 = calQHourly.getF3();
		double fVE = calQHourly.getVE();
		double fSE = calQHourly.getE();

		calibrate calQDaily(QDaily, QobsDaily);
		double dailyfCRF1 = calQDaily.getF1();
		double dailyfCRF2 = calQDaily.getF2();
		double dailyfCRF3 = calQDaily.getF3();
		double dailyfVE = calQDaily.getVE();
		double dailyfSE = calQDaily.getE();

		calibrate calQWeekly(QWeekly, QobsWeekly);
		double weeklyfCRF1 = calQWeekly.getF1();
		double weeklyfCRF2 = calQWeekly.getF2();
		double weeklyfCRF3 = calQWeekly.getF3();
		double weeklyfVE = calQWeekly.getVE();
		double weeklyfSE = calQWeekly.getE();

		// Addition of Monthly Timestep change code by Isira Samarasekera
		calibrate calQMonthly(QMonthly, QobsMonthly);
		double monthlyfCRF1 = calQMonthly.getF1();
		double monthlyfCRF2 = calQMonthly.getF2();
		double monthlyfCRF3 = calQMonthly.getF3();
		double monthlyfVE = calQMonthly.getVE();
		double monthlyfSE = calQMonthly.getE();
		// End of Monthly Timestep change code by Isira Samarasekera

		outfile << "Hourly fitness:  " << " VE = " << fVE << "   " << " CRF1 = " << fCRF1 << "   " << " CRF2 = " << fCRF2 << "   " << " CRF3 = " << fCRF3  <<endl;
		outfile << "Daily fitness:  " << " VE = " << dailyfVE << "   " << " CRF1 = " << dailyfCRF1 << "   " << " CRF2 = " << dailyfCRF2 << "   " << " CRF3 = " << dailyfCRF3 << endl;
		outfile << "Weekly fitness:  " << " VE = " << weeklyfVE << "   " << " CRF1 = " << weeklyfCRF1 << "   " << " CRF2 = " <<weeklyfCRF2 << "   " << " CRF3 = " <<weeklyfCRF3 << endl;
		outfile << "Monthly fitness:  " << " VE = " << monthlyfVE << "   " << " CRF1 = " << monthlyfCRF1 << "   " << " CRF2 = " << monthlyfCRF2 << "   " << " CRF3 = " << monthlyfCRF3 << endl << endl;
		// End of Monthly Timestep change code by Isira Samarasekera
		// End of Timestep change code by Pallavi Iyengar
	}
	else
	{
		outfile << " Observed Discharge is not available. The goodness of fit is %ofProjectArea TIValueBin." << endl;
	}

// Addition of Timestep change code by Pallavi Iyengar
	vector<int> dateVec = input->CalendarDates;
	vector<string> timeVec = input->TimeSeries;
	vector <double> QQ, BQi, PervQi, UrImpQQ, precip;
	QQ = input->QQi;
	BQi = input->BQi;
	PervQi = input->PAQi;
	UrImpQQ = input->DCIAQi;
	precip = input->Precip_mpdt;


	
		outfile << " Total runoff (TotalQ :mm)       :	" << input->RepoDict["total_Runoff_R_m"] * 1000 << "		Total pervious area surface flow (TotalPerviousAreaQ_mm :mm) :    " << input->RepoDict["total_Runoff_Pervious_R_m"] * 1000 << endl
			<< " Total Baseflow(BQ:mm)           :   " << input->RepoDict["total_Runoff_Subsurface_R_m"] * 1000 << "		Total Impervious impervious flow (TotalDCIAQ:mm)     :    " << input->RepoDict["total_Runoff_Impervious_R_m"] * 1000 << endl << endl;
		outfile << "  BQ/TotalQ: " << input->RepoDict["total_Runoff_Subsurface_R_m"] / input->RepoDict["total_Runoff_R_m"] << "   TotalDCIAQ_mm/TotalQ: " << input->RepoDict["total_Runoff_Impervious_R_m"] / input->RepoDict["total_Runoff_R_m"] << endl << endl;
		outfile << " The unit of the following time series output is : m/DT , DT: simulation time interval, such as 1 hour" << endl << endl;

		outfile << "  " << std::setw(10) << "yyyymmdd"
			<< "  " << std::setw(10) << "hh:mm:ss"
			<< "  " << std::setw(10) << "Precip "
			<< "  " << std::setw(13) << "TotalQ "
			<< "  " << std::setw(13) << "BaseQ "
			<< "  " << std::setw(13) << "PerviousQ "
			<< "  " << std::setw(13) << "DCIAQ ";
			if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
			{
				outfile << "  " << std::setw(13) << "Qobs ";
			}
			outfile<< endl << endl;//check this with previous Ver



	if (input->SimulationStringParams["OutputTimeStep"] == "Hourly")
	{
	
		for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
		{
			outfile << "  " << std::setw(10) << dateVec.at(i)
				<< "  " << std::setw(10) << timeVec.at(i)
				<< "  " << std::setw(10) << precip.at(i)
				<< "  " << std::setw(13) << QQ.at(i)
				<< "  " << std::setw(13) << BQi.at(i)
				<< "  " << std::setw(13) << PervQi.at(i)
				<< "  " << std::setw(13) << UrImpQQ.at(i);
				if(input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QH[i];
				}
				outfile<< endl;
		}
		outfile.close();
	}

	if (input->SimulationStringParams["OutputTimeStep"] == "Daily")
	{
		vector<int>dailydate = getDailyDate(dateVec);
		vector<string> dailyHHMMSS = getDailyHHMMSS(timeVec, dateVec);
		std::vector<double> dailyPrecip = hourlyToDaily(precip), dailyQQ = hourlyToDaily(QQ),
			dailyBQQ = hourlyToDaily(BQi),	
			dailyPervQi = hourlyToDaily(PervQi),
			dailyUrImpQQ = hourlyToDaily(UrImpQQ);

		//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
		int dailySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / 24);

		for (int i = 0; i < dailySize; i++)
		{
			outfile << "  " << std::setw(10) << dailydate.at(i)
				<< "  " << std::setw(10) << dailyHHMMSS.at(i)
				<< "  " << std::setw(10) << dailyPrecip.at(i)
				<< "  " << std::setw(13) << dailyQQ.at(i)
				<< "  " << std::setw(13) << dailyBQQ.at(i)
				<< "  " << std::setw(13) << dailyPervQi.at(i)
				<< "  " << std::setw(13) << dailyUrImpQQ.at(i);
				if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QD[i];
				}
			outfile << endl;
		}
		outfile.close();

	}

	if (input->SimulationStringParams["OutputTimeStep"] == "Weekly")
	{
		vector<int>weeklydate = getWeeklyDate(dateVec);
		vector<string> weeklyHHMMSS = getWeeklyHHMMSS(timeVec, dateVec);
		std::vector<double> weeklyQQ = hourlyToWeekly(QQ), weeklyBQQ = hourlyToWeekly(BQi),
			weeklyPervQi = hourlyToWeekly(PervQi),
			weeklyUrImpQQ = hourlyToWeekly(UrImpQQ),
			weeklyprecip = hourlyToWeekly(precip);

		//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
		int weeklySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / (24 * 7));

		for (int i = 0; i < weeklySize; i++)
			{
				outfile << "  " << std::setw(10) << weeklydate.at(i)
					<< "  " << std::setw(10) << weeklyHHMMSS.at(i)
					<< "  " << std::setw(10) << weeklyprecip.at(i)
					<< "  " << std::setw(13) << weeklyQQ.at(i)
					<< "  " << std::setw(13) << weeklyBQQ.at(i)
					<< "  " << std::setw(13) << weeklyPervQi.at(i)//does it correspong to pervious
					<< "  " << std::setw(13) << weeklyprecip.at(i);
				if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QW[i];
				}
			outfile << endl;
		}
			outfile.close();
	}
	
	// End of Timestep change code by Pallavi Iyengar

	// Addition of Monthly Timestep change code by Isira Samarasekera
	if (input->SimulationStringParams["OutputTimeStep"] == "Monthly")
	{
		vector<int> monthlyDate = getMonthlyDate(dateVec);
		vector<string> monthlyHHMMSS = getMonthlyHHMMSS(timeVec, dateVec);
		std::vector<double> monthlyQQ = hourlyToMonthly(QQ), monthlyBQQ = hourlyToMonthly(BQi),
			monthlyPervQi = hourlyToMonthly(PervQi),
			monthlyUrImpQQ = hourlyToMonthly(UrImpQQ),
			monthlyprecip = hourlyToMonthly(precip);

		//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
		//cte 20210715 Build function so integer 30 is replaced with variable of actual number of days in month
		int monthlySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / (24 * 30));

		for (int i = 0; i < monthlySize; i++)
		{
			outfile << "  " << std::setw(10) << monthlyDate.at(i)
				<< "  " << std::setw(10) << monthlyHHMMSS.at(i)
				<< "  " << std::setw(10) << monthlyprecip.at(i)
				<< "  " << std::setw(13) << monthlyQQ.at(i)
				<< "  " << std::setw(13) << monthlyBQQ.at(i)
				<< "  " << std::setw(13) << monthlyPervQi.at(i)
				<< "  " << std::setw(13) << monthlyUrImpQQ.at(i);
				if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QM[i];
				}
			outfile << endl;
		}
		outfile.close();
	}
	// End of Timestep change code by Isira Samarasekera
	outfile.close();
	LogMessage::log("Created " + path);

}

vector<int> StatisticalOutputWriter::getDailyDate(const std::vector<int> date)
{
	int size = date.size() / (24);
	std::vector<int> dailyDate(size);

	for (int i = 0; i<size; i++)
	{
		dailyDate[i] = date[i * 24];
	}
	return dailyDate;
}

vector<int> StatisticalOutputWriter::getWeeklyDate(const std::vector<int> date)
{
	int size = date.size() / (24 * 7);
	std::vector<int> weeklyDate(size);

	for (int i = 0; i<size; i++)
	{
		weeklyDate[i] = date[i * 24 * 7];
	}
	return weeklyDate;
}

vector<int> StatisticalOutputWriter::getMonthlyDate(const std::vector<int> date)
{
	int size = date.size() / (24 * 30);
	std::vector<int> monthlyDate(size);

	for (int i = 0; i<size; i++)
	{
		monthlyDate[i] = date[i * 24 * 30];
	}
	return monthlyDate;
}

vector<string> StatisticalOutputWriter::getDailyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / 24;
	std::vector<string> dailyHHMMSS(size);
	for (int i = 0; i<size; i++)
	{
		dailyHHMMSS[i] = HHMMSS[i * 24];
	}
	return dailyHHMMSS;
}

vector<string> StatisticalOutputWriter::getWeeklyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{ 
	int size = date.size() / (24 * 7);
	std::vector<string> weeklyHrMMSS(size);
	for (int i = 0; i<size; i++)
	{
		weeklyHrMMSS[i] = HHMMSS[i * 24 * 7];
	}
	return weeklyHrMMSS;
}

vector<string> StatisticalOutputWriter::getMonthlyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / (24 * 30);
	std::vector<string> monthlyHrMMSS(size);
	for (int i = 0; i<size; i++)
	{
		monthlyHrMMSS[i] = HHMMSS[i * 24 * 30];
	}
	return monthlyHrMMSS;
}
