#include "WriteOutputs.h"
///////////////////////////////////////////////////////////////
//  WriteOutputs.h   -  Calling various output writers       //
//  ver 1.0                                                  //
//                                                           //
//  Language:     Visual C++, VS 2019                        //
//  Application:  HydroPlus                                  //
//  Author:       Theodore Endreny, SUNY ESF                 //
//                (315)200-4006, te@esf.edu                  //
///////////////////////////////////////////////////////////////

void WriteOutputs::writeExtendedOutput(DataOrganizer* organizer, Inputs* input)
{
	std::string inputDirectory = input->SimulationStringParams["InputDirectory"];

	for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
	{
		for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
		{
			//Extended output files for water balance
			if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "BulkArea")
			{
				GIOutputWriter::writeVegCanopyFile_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeVegCanopyFile_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnPerAreaFile_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnPerAreaFile_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnImpAreaFile_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterOnImpAreaFile_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeEvap_ET_Infil_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeEvap_ET_Infil_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_FWB_BulkArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

				//Li 10242020 This is the noExtendedOutput result, but put under ExtendedOutput condition, only for bulk area; waterbalance of the last 2 arguments are same
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "BioRetention")
			{
				// BioRetention
				GIOutputWriter::writeWaterBalance_BioRetention(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RainGarden")
			{
				// Rain Garden
				GIOutputWriter::writeWaterBalance_RainGarden(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "InfilTrench")
			{
				// InfilTrench
				GIOutputWriter::writeWaterBalance_InfilTrench(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "Swale")
			{
				// Swale
				GIOutputWriter::writeWaterBalance_Swale(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "PermeablePavement")
			{
				// PermeablePavement
				GIOutputWriter::writeWaterBalance_PermeablePavement(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "GreenRoof")
			{
				//Li:green roof
				GIOutputWriter::writeWaterBalance_GreenRoof(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);

			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RoofDiscon")
			{
				//Li:rooftop Disconnection
				GIOutputWriter::writeWaterBalance_RoofDiscon(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
			else if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "RainBarrel")
			{
				//Li: Rain Barrel
				GIOutputWriter::writeWaterBalance_RainBarrel(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GIOutputWriter::writeWaterInRootZone_VWB_GIArea(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//Li 11/18/2020
				StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				//03162021 GI Water Quality
				GI_WaterQuality::Removal(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				GI_WaterQuality::Removal_TS(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			}
		}
	}
}

void WriteOutputs::writeNoExtendedOutput(DataOrganizer* organizer, Inputs* input)
{
	std::string inputDirectory = input->SimulationStringParams["InputDirectory"];

	for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
	{
		for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
		{
			StatisticalOutputWriter::writeWaterBalanceFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			StatisticalOutputWriter::writeOutputFile(inputDirectory, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
			StatisticalOutputWriter::writeCumulativeFile(input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
		}
	}
}

void WriteOutputs::writeStatisticalOutput(DataOrganizer* organizer, Inputs* input)
{
	std::string inputDirectory = input->SimulationStringParams["InputDirectory"];
	StatisticalOutputWriter::writeLAISeries(input);
	StatisticalOutputWriter::writePollutionFile(inputDirectory, input);
	if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
	{
		StatisticalOutputWriter::writeCaliQobs(input, inputDirectory);
		StatisticalOutputWriter::writeCaliTotalQQ(input);
	}
	if (input->SimulationStringParams["ExtendedOutputsTimeStep"] != "NoExtendedOutputs")
	{
		writeExtendedOutput(organizer, input);
	}
	else
	{
		writeNoExtendedOutput(organizer, input);
	}
}
