#include "PerviousDepressionStorageCalc.h"

//Compute perDepEvap (m) with functor calculatePerDepEvap, using spatial temperature approach 
void PerviousDepressionStorageCalc::calculatePerDepEvapTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
	//Latent heat of vaporization (J/kg)
	#define lamda 2257000.0
	folder->VarDict["perDepEvap"] = folder->VarDict["WaterLE_W_p_m2"] / lamda;
}

//Compute perDepEvap (m) with functor calculatePerDepEvap, using spatial temperature approach 
void PerviousDepressionStorageCalc::calculatePerDepEvapStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	if (folder->VarDict["perDepStor"] > 0.0) {
		folder->VarDict["perDepEvap"] = input->PE[timeStep] * (folder->VarDict["perDepStor"] / folder->VarDict["perDepStorMax"]);
	}
	else
		folder->VarDict["perDepEvap"] = 0.0;
}

void PerviousDepressionStorageCalc::calculate(Inputs* input, DataFolder* folder, int timeStep, std::map<std::string, void(*)(Inputs* input, DataFolder* folder, int timeStep)>& functors)
{

	// perviousStorageMax = param.dat pervious storage maximum value (in mm) * 0.001 -> to convert to meters
	folder->VarDict["perDepStorMax"] = 0.001 * folder->ParamDict["PerviousDepressionStorage_mm"];

	//perDepStorPrev (m) = impDepStor (m) to retain information on the prior timestep storage depth before addind precip and removing evaporation
	double perDepStorPrev = folder->VarDict["perDepStor"];

	//perPrecip (m), runon for recharge to soil, initialized to zero
	double perPrecip = 0;
	//incrPerDepStor (m), increment in pervious storage, initialized to zero
	double incrPerDepStor = 0.0;

	//Bulk Area Type defines perPrecip (m) as waterToPerDep (m)
	//cte 20210715 Rename waterToPerDep into PrecipAndRunonOnPervious_m and Runoff_BA_to_GI_m; rename perPrecip PrecipAndRunonOnPervious_m (extend to renaming snowOpen, perDepEvap, perDepStor)
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		//perPrecip (m) for BulkArea is from precipitation on pervious areas, and is considered for depression storage
		perPrecip = folder->VarDict["waterToPerDep"];
	}
	//GI Type defines perPrecip (m) as waterToPerDep_pre (m), which contains waterToPerDep (m) and Bulk Area runoff going to GI area
	//cte 20210715 Rename waterToPerDep_pre into two terms, PrecipAndRunonOnPervious_m and Runoff_BA_to_GI_m, combining three last terms below into one term; rename perPrecip PrecipAndRunonOnPervious_m (extend to renaming snowOpen, perDepEvap, perDepStor)
	else {
		//perPrecip (m) for GI is from precipitation on pervious areas + BulkArea upslope area to GI, and is considered for depression storage
		perPrecip = folder->VarDict["waterToPerDep_pre"];
	}

	//Compute perDepEvap (m) with functor calculatePerDepEvap, using spatial temperature or statistical approach 
	//perDepEvap (m) created by functor
	functors["calculatePerDepEvap"](input, folder, timeStep);

	//If snowOpen (m) > 0, there is snow on ground and liquid evaporation from storage is zero for this timestep
	if (folder->VarDict["snowOpen"] > 0.0) {
		folder->VarDict["perDepEvap"] = 0.0;
	}

	//If perDepStorPrev (m) <= 0, there was no previous storage, evaporation from storage is zero for this timestep
	if (perDepStorPrev <= 0.0) {
		folder->VarDict["perDepEvap"] = 0.0;
	}

	// Modify the (current)  pervious depression storage = pervious pression storage + increase in pervious depression storage
	//perDepStor (m) equals perDepStor (m) plus perPrecip (m) and minus perDepEvap (m)
	folder->VarDict["perDepStor"] = folder->VarDict["perDepStor"] + perPrecip - folder->VarDict["perDepEvap"];	// update interception storage

	//if perDepStor (m) negative, it is due to perDepEvap (m) > perPrecip (m), so perDepEvap (m) is limited to previous storage and perPrecip (m) 
	if (folder->VarDict["perDepStor"] < 0.0)
	{
		folder->VarDict["perDepEvap"] = perDepStorPrev + perPrecip;					// per dep evap = incoming plus stored water
		folder->VarDict["perDepStor"] = 0.0;
	}


	//cte 20210524 Test new algorithm that passes all water to infiltration as Water_forSoilRecharge_m, but retains any ponded water up to the depth of available storage StorageAvailable_Ponding_m
	//cte 20210524 In new algorithm, this available ponding depth StorageAvailable_Ponding_m would fill with infiltration excess runoff 
	//cte 20210524 In new algorithm, separate StorageAvailable_PerviousTotal_m is sum of StorageAvailable_Ponding_m and StorageAvailable_SatExcess_m, the latter filled only by saturation excess.
	//cte 20210524 StorageAvailable_PerviousTotal_m is limited based on user defined maximum storage
	//cte 20210524 StorageAvailable_Ponding_m is updated for entire pervious area, based on water on pervious, precipitation, evaporation, and can always infiltrate to soil
	//cte 20210524 StorageAvailable_SatExcess_m is updated for each TI bin in pervious area, based on saturation excess in TI bin and can only be deminished by evaporation or drainage when TI bin no longer saturated
	//cte 20210524 StorageAvailable_SatExcess_m would use a conditional, if previous time step StorageAvailable_SatExcess_m > 0 and bin has water table below surface, then send storage to folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia]

	//cte 20210524 In new algorithm, the following statements would be used:
	//folder->VarDict["Water_forSoilRecharge_m"] = folder->VarDict["perDepStor"];

	//cte 20210524 In new algorithm, the following statements might be used:
	//if (folder->ParamStringDict["Type"] != "BulkArea") {
		//folder->VarDict["Water_forSoilRecharge_m"] = perPrecip + folder->VarDict["surfaceDepth"];
	//}

	// If the current pervious storage < inputStorageTime (Pervious Storage Max)
	if (folder->VarDict["perDepStor"] < folder->VarDict["perDepStorMax"])
	{

		//cte 20210524 In new algorithm, the following statements would be used: 
		//folder->VarDict["StorageAvailable_Ponding_m"] = folder->VarDict["perDepStorMax"] - folder->VarDict["perDepStor"];

		//Water_forSoilRecharge_m (m) is zero due to the availability of pervious depression storage. 
		//cte 20210524 In new algorithm, the following statements would be removed: 
		folder->VarDict["Water_forSoilRecharge_m"] = 0.0;
	}
	//cte 20210524 Test using perDepStorPrev >= folder->VarDict["perDepStorMax"], which should be equivalent to the 2 statements of 1) perDepStorPrev > folder->VarDict["perDepStorMax"] and 2) else
	else if (folder->VarDict["perDepStor"] >= folder->VarDict["perDepStorMax"]) {

		//cte 20210524 In new algorithm, the following statements would be used: 
		//folder->VarDict["StorageAvailable_Ponding_m"] = 0;

		//Water_forSoilRecharge_m (m) equals perPrecip (m) minus water that can go into storage due to: (perDepStorMax-perDepStorPrev) and perDepEvap 
		//cte 20210524 In new algorithm, the following statements would be removed: 
		folder->VarDict["Water_forSoilRecharge_m"] = perPrecip - (folder->VarDict["perDepStorMax"] - perDepStorPrev) - folder->VarDict["perDepEvap"];

		//If GI Type, then return surfaceDepth (m) with perPrecip (m) as Water_forSoilRecharge_m
		if (folder->ParamStringDict["Type"] != "BulkArea") {
			//cte 20210524 In new algorithm, the following statements would be used: 
			//folder->VarDict["Water_forSoilRecharge_m"] = perPrecip + folder->VarDict["surfaceDepth"];

			//cte 20210524 In new algorithm, the following statements would be removed: 
			folder->VarDict["Water_forSoilRecharge_m"] = perPrecip - (folder->VarDict["perDepStorMax"] - perDepStorPrev) + folder->VarDict["surfaceDepth"]
				- folder->VarDict["perDepEvap"];

		}

		//cte 20210524 Test new algorithm that passes all water to infiltration as Water_forSoilRecharge_m, but retains any ponded water up to the depth of available storage
		//perDepStor (m) reset to perDepStorMax (m) having only passed excess storage 
		//cte 20210524 In new algorithm, the following statements would be removed: 
		folder->VarDict["perDepStor"] = folder->VarDict["perDepStorMax"];
	}

	//cte 20210524 In new algorithm, the following statements would be used: 
	//if (folder->VarDict["StorageAvailable_Ponding_m"] < 0.0) {
		//folder->VarDict["StorageAvailable_Ponding_m"] = 0;
	//}

	// Making sure that pervious dep runoff to infiltration  isn't < 0
		if (folder->VarDict["Water_forSoilRecharge_m"] < 0.0) {
			folder->VarDict["Water_forSoilRecharge_m"] = 0.0;
		}

	// Making sure that the amount currently stored in pervious depression storage isn't < 0
		if (folder->VarDict["perDepStor"] < 0.0) {
			folder->VarDict["perDepStor"] = 0.0;
		}

	storeValues(input, folder, timeStep);


	// Reza, for the GI unit Evap.rate variable based on the SWMM approach
	folder->VarDict["totalEvap"] = folder->VarDict["treeEvapW"] + folder->VarDict["treeEvapSWE"]
		+ folder->VarDict["treeEvap"] + folder->VarDict["perDepEvap"] + folder->VarDict["impEvap_m"];

}


void PerviousDepressionStorageCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalperDepEvap"] += folder->VarDict["perDepEvap"];
	folder->VarDict["totalrunoffToInfil"] += folder->VarDict["Water_forSoilRecharge_m"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		folder->VarVecDict["perDepEvap"].emplace_back(folder->VarDict["perDepEvap"]);
		folder->VarVecDict["perDepStor"].emplace_back(folder->VarDict["perDepStor"]);
		folder->VarVecDict["Water_forSoilRecharge_m"].emplace_back(folder->VarDict["Water_forSoilRecharge_m"]);
	}
}

