#include "StoreTimeStepTotalsCalc.h"

/****************************************************************************

*****************************************************************************/


void StoreTimeStepTotalsCalc::calculate(Inputs *input, int timeStep, std::vector<std::vector<DataFolder*> > &DataDrawers)
{
	storeorganizerValues(input, timeStep, DataDrawers);
}

//Continuity maintained when adjusting to catchment area from folder area (e.g., BulkArea). 
//		Depth_adjusted_Folder_m = Depth_unadjusted_Folder_m * Area_Folder_fraction = Depth_unadjusted_Folder_m * Area_Folder_m2 / Area_Catchment_m2 
//		Adjustment allows depths reported for catchment to balance with total inflow 
//		For a catchment with Area_BulkArea_01_fraction = 0.8 and Area_BulkArea_02_fraction = 0.2, each generating Depth_unadjusted_BulkArea_0#_m = 0.05 m of runoff. 
//		Depth_adjusted_BulkArea_01_m = Depth_unadjusted_BulkArea_01_m * Area_BulkArea_01_fraction and Depth_adjusted_BulkArea_02_m = Depth_unadjusted_BulkArea_02_m * Area_BulkArea_02_fraction

void StoreTimeStepTotalsCalc::storeorganizerValues(Inputs *input, int timeStep, std::vector<std::vector<DataFolder*> > &DataDrawers)
{
	double area = 0.0;              // area this folder makes up - THIS NEEDS TO CONSIDER THE CONTRIBUTING AREA OF EACH GI folder
	double fArea = 0.0;             // fraction of the watershed this folder makes up
	double watershedArea = input->SimulationNumericalParams["CatchmentArea_m2"];

	double Groundwater_surficial_m = 0.0;
	double Groundwater_surficial_frac = 0.0;

	double total_InterceptionRain_viaTree_m = 0.0;
	double total_InterceptionSnowWE_viaTree_m = 0.0;
	double total_InterceptionRain_viaSVeg_m = 0.0;
	double total_InterceptionSnowWE_viaSVeg_m = 0.0;
	double total_Evaporation_viaTree_m = 0.0;
	double total_Evaporation_viaSVeg_m = 0.0;
	double total_ThruFlow_viaTree_m = 0.0;
	double total_ThruFlow_viaSVeg_m = 0.0;
	double total_ThruFlowSnowWE_viaTree_m = 0.0;
	double total_ThruFlowSnowWE_viaSVeg_m = 0.0;

	double total_Snow_OpenArea_m = 0.0;
	double total_Snow_UnderTree_m = 0.0;
	double total_Snow_UnderSVeg_m = 0.0;
	double total_SnowSublimation_OpenArea_m = 0.0;
	double total_SnowSublimation_UnderTree_m = 0.0;
	double total_SnowSublimation_UnderSVeg_m = 0.0;
	double total_SnowMelt_TreeCanopy_m = 0.0;
	double total_SnowMelt_SVegCanopy_m = 0.0;

	double total_SnowMelt_UnderTree_m = 0.0;
	double total_SnowMelt_UnderSVeg_m = 0.0;
	double total_SnowMelt_OpenArea_m = 0.0;

	double totalWaterOnImperviousArea_m = 0.0;
	double totalImpDepEvap_m = 0.0;
	double totalImpDepStor_m = 0.0;
	double totalDCIAFlowToOutlet_m = 0.0;
	double totalImpFlowToSoil_m = 0.0;

	double totalWaterOnPerviousArea_m = 0.0;
	double totalWaterToSoil_m = 0.0;
	double totalPerDepEvap_m = 0.0;
	double totalPerDepStor_m = 0.0; 
	double total_Water_forSoilRecharge_m = 0.0;

	double total_Infiltration_m = 0.0;
	double total_Drainage_macroPore_m = 0.0;
	double total_InfilExInfil_m = 0.0;
	double total_SatExInfil_m = 0.0;

	double totalInfilExQ_m = 0.0;
	double totalSatExQ_m = 0.0;

	double totalRootZoneEvapoTranspiration_m = 0.0;
	double aveSMD_m = 0.0;
	double Storage_Used_RootZone_m = 0.0;
	double total_Drainage_RootZone_m = 0.0;

	double Storage_Used_UnsatZone_m = 0.0; 
	double total_unsatZoneFlow_m = 0.0;

	double total_Runoff_Pervious_NR_m = 0.0;
	double total_Runoff_Impervious_NR_m = 0.0;
	double total_Runoff_Subsurface_NR_m = 0.0;
	double total_Runoff_All_NR_m = 0.0;

	double totalPAQi = 0.0;
	double totalDCIAQi = 0.0;
	double totalBQi = 0.0;
	double totalQQi = 0.0;

	//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
	for (int i = 0; i < DataDrawers.size(); ++i)
	{
		//Loop through the DataFolders in each DataDrawer, where the upper bound is based on the specific DataDrawer[i] attribute .size()
		for (int j = 0; j < DataDrawers[i].size(); ++j)
		{
			//For DataFolders that have data, BulkArea or GI
			if (DataDrawers[i][j]->ParamStringDict["Type"]  != "NODATA")
			{
				DataFolder *folder = DataDrawers[i][j];

				//Note, multiplication of variables by fractional area below is to adjust depths from one analysis area to entire catchment area
				//	Rationale:	1. variables arrive as depths for an analysis area (e.g., Bulk Area or GI Area), which are not conserved when transfered to the catchment area
				//				2. multiplying these depths by their analysis area gives a volume, which are conserved when transfered between different areas
				//				3. multiplying these depths by their fractional area is equivalent to multiplying by their analysis area and dividing by the catchment area, getting depth for catchment area
				//				4. fArea multiplication below preserves depth for cover type area, which is addressed in StatisticalOutputWriter.cpp
				//area is DataFolder area (m2)
				area = folder->ParamDict["Area"];
				//fArea is fraction of DataFolder area (m2) to catchment area (m2)
				fArea = area / watershedArea;

				Groundwater_surficial_m += folder->VarDict["Groundwater_surficial_m"] * fArea;
				Groundwater_surficial_frac += folder->VarDict["Groundwater_surficial_frac"] * fArea;

				total_InterceptionRain_viaTree_m += folder->VarDict["treeIntW"] * fArea;
				total_InterceptionSnowWE_viaTree_m += folder->VarDict["treeIntSWE"] * fArea;

				total_InterceptionRain_viaSVeg_m += folder->VarDict["shortVegIntW"] * fArea;
				total_InterceptionSnowWE_viaSVeg_m += folder->VarDict["shortVegIntSWE"] * fArea;

				total_Evaporation_viaTree_m += folder->VarDict["treeEvap"] * fArea;
				total_Evaporation_viaSVeg_m += folder->VarDict["shortVegEvap"] * fArea;
				
				total_ThruFlow_viaTree_m += folder->VarDict["treeThruFlow"] * fArea;
				total_ThruFlowSnowWE_viaTree_m += folder->VarDict["treeThruFlowSWE"] * fArea;
				total_ThruFlow_viaSVeg_m += folder->VarDict["shortVegThroughQ"] * fArea;
				total_ThruFlowSnowWE_viaSVeg_m += folder->VarDict["shortVegThruFlowSWE"] * fArea;
				

				total_Snow_OpenArea_m += folder->VarDict["snowOpen"] * fArea;
				total_Snow_UnderTree_m += folder->VarDict["snowUnderTree"] * fArea;
				total_Snow_UnderSVeg_m += folder->VarDict["snowUnderShortVeg"] * fArea;

				total_SnowSublimation_OpenArea_m += folder->VarDict["snowEvapOpen"] * fArea;
				total_SnowSublimation_UnderTree_m += folder->VarDict["snowEvapUnderTree"] * fArea;
				total_SnowSublimation_UnderSVeg_m += folder->VarDict["snowEvapUnderShortVeg"] * fArea;
				total_SnowMelt_OpenArea_m += folder->VarDict["snowMeltOpen"] * fArea;
				total_SnowMelt_TreeCanopy_m += folder->VarDict["snowMeltTreeCanopy_m"] * fArea;
				total_SnowMelt_SVegCanopy_m += folder->VarDict["snowMeltSVegCanopy_m"] * fArea;
				total_SnowMelt_UnderTree_m += folder->VarDict["snowMeltUnderTree"] * fArea;
				total_SnowMelt_UnderSVeg_m += folder->VarDict["snowMeltUnderShortVeg"] * fArea;

				//cte 20210517 continue in next month renaming local variables, then folder variables
				totalWaterOnImperviousArea_m += folder->VarDict["waterOnImperviousArea"] * fArea;
				totalImpDepEvap_m += folder->VarDict["impEvap_m"] * fArea;
				totalImpDepStor_m += folder->VarDict["impDepStor_m"] * fArea;
				//Runoff_ImperviousToPervious_m, previously adjusted for catchment area with multiplication by ImpAreaNotDirectlyConnected2Outlet_frac in ImperviousDepressionStorageCalc.cpp
				totalImpFlowToSoil_m += folder->VarDict["Runoff_ImperviousToPervious_m"] * fArea;
				totalDCIAFlowToOutlet_m += folder->VarDict["Runoff_ImperviousToOutlet_m"] * fArea;

				//folder->VarDict["waterOnPerviousArea"] does not include folder->VarDict["Runoff_ImperviousToPervious_m"]
				totalWaterOnPerviousArea_m += folder->VarDict["waterOnPerviousArea"] * fArea;
				//folder->VarDict["waterToPerDep"] does include folder->VarDict["Runoff_ImperviousToPervious_m"] and hence can be larger than precipitation depth
				totalWaterToSoil_m += folder->VarDict["waterToPerDep"] * fArea;
				totalPerDepEvap_m += folder->VarDict["perDepEvap"] * fArea;
				totalPerDepStor_m += folder->VarDict["perDepStor"] * fArea;

				//folder->VarDict["Water_forSoilRecharge_m"] does include folder->VarDict["Runoff_ImperviousToPervious_m"] and hence can be larger than precipitation depth
				total_Water_forSoilRecharge_m += folder->VarDict["Water_forSoilRecharge_m"] * fArea;
				total_Drainage_macroPore_m += folder->VarDict["Drainage_macroPore_m"] * fArea;
				total_InfilExInfil_m += folder->VarDict["Infiltration_viaInfilEx_m"] * fArea;
				total_SatExInfil_m += folder->VarDict["Infiltration_viaSatEx_m"] * fArea;
				total_Infiltration_m += folder->VarDict["Infiltration_m"] * fArea;

				totalInfilExQ_m += folder->VarDict["Runoff_PervInfilEx_m"] * fArea;
				totalSatExQ_m += folder->VarDict["Runoff_SatExcess_m"] * fArea;

				totalRootZoneEvapoTranspiration_m += folder->VarDict["EvapoTranspiration_RootZone_m"] * fArea;
				aveSMD_m += folder->VarDict["aveSMD"] * fArea;
				Storage_Used_RootZone_m += folder->VarDict["Storage_Used_RootZone_m"] * fArea;
				total_Drainage_RootZone_m = folder->VarDict["Drainage_RootZone_m"] * fArea;

				Storage_Used_UnsatZone_m += folder->VarDict["Storage_Used_UnsatZone_m"] * fArea;
				total_unsatZoneFlow_m += folder->VarDict["Drainage_UnsatZone_m"] * fArea;

				//precursor of PerviousQ_NR_m, (SatExQ + infilExQ), adjusted to folder area depth in FlowSumCalc.cpp with multiplication of PAQ with PerviousAreaOfFolder_frac
				total_Runoff_Pervious_NR_m += folder->VarDict["PerviousQ_NR_m"] * fArea;
				//precursor of ImperviousQ_NR_m, folder->VarDict["Runoff_ImperviousToOutlet_m"], adjusted to folder area depth in in ImperviousDepressionStorageCalc.cpp with multiplication of PAQ with folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] 
				total_Runoff_Impervious_NR_m += folder->VarDict["ImperviousQ_NR_m"] * fArea;
				//folder->VarDict["BaseQ_NR_m"] generated for entire catchment, but may need to be reduced to pervious area depending on reducing topographic index 
				total_Runoff_Subsurface_NR_m += folder->VarDict["BaseQ_NR_m"] * fArea;
				total_Runoff_All_NR_m += folder->VarDict["TotalQ_NR_m"] * fArea;
			}
		}

	}

	//RepoVecDict and RepoDict pass between folders
	input->RepoVecDict["Groundwater_surficial_m"].push_back(Groundwater_surficial_m);
	input->RepoVecDict["Groundwater_surficial_frac"].push_back(Groundwater_surficial_frac);

	input->RepoVecDict["totalTreeIntRain_TS"].push_back(total_InterceptionRain_viaTree_m);
	input->RepoDict["total_InterceptionRain_viaTree_m"] += total_InterceptionRain_viaTree_m;
	input->RepoVecDict["totalTreeIntSWE_TS"].push_back(total_InterceptionSnowWE_viaTree_m);
	input->RepoDict["total_InterceptionSnowWE_viaTree_m"] += total_InterceptionSnowWE_viaTree_m;

	input->RepoVecDict["totalShortVegIntRain_TS"].push_back(total_InterceptionRain_viaSVeg_m);
	input->RepoDict["total_InterceptionRain_viaSVeg_m"] += total_InterceptionRain_viaSVeg_m;
	input->RepoVecDict["totalShortVegIntSWE_TS"].push_back(total_InterceptionSnowWE_viaSVeg_m);
	input->RepoDict["total_InterceptionSnowWE_viaSVeg_m"] += total_InterceptionSnowWE_viaSVeg_m;

	input->RepoVecDict["totalTreeEvap_TS"].push_back(total_Evaporation_viaTree_m);
	input->RepoDict["total_Evaporation_viaTree_m"] += total_Evaporation_viaTree_m;
	input->RepoVecDict["totalShortVegEvap_TS"].push_back(total_Evaporation_viaSVeg_m);
	input->RepoDict["total_Evaporation_viaSVeg_m"] += total_Evaporation_viaSVeg_m;

	input->RepoVecDict["totalTreeThruQ_TS"].push_back(total_ThruFlow_viaTree_m);
	input->RepoDict["totalTreeThruFlow_m"] += total_ThruFlow_viaTree_m;
	input->RepoVecDict["totalShortVegThruQ_TS"].push_back(total_ThruFlow_viaSVeg_m);
	input->RepoDict["totalShortVegThruFlow_m"] += total_ThruFlow_viaSVeg_m;
	input->RepoDict["totalTreeThruFlowSnowWE_m"] += total_ThruFlowSnowWE_viaTree_m;
	input->RepoDict["totalShortVegThruFlowSnowWE_m"] += total_ThruFlowSnowWE_viaSVeg_m;

	input->RepoVecDict["totalSnowOpen_TS_m"].push_back(total_Snow_OpenArea_m);
	input->RepoDict["total_Snow_OpenArea_m"] += total_Snow_OpenArea_m;
	input->RepoVecDict["totalSnowUnderTree_TS_m"].push_back(total_Snow_UnderTree_m);
	input->RepoDict["total_Snow_UnderTree_m"] += total_Snow_UnderTree_m;
	input->RepoVecDict["totalSnowUnderShortVeg_TS_m"].push_back(total_Snow_UnderSVeg_m);
	input->RepoDict["total_Snow_UnderSVeg_m"] += total_Snow_UnderSVeg_m;

	input->RepoVecDict["totalSnowEvapOpen_TS"].push_back(total_SnowSublimation_OpenArea_m);
	input->RepoDict["total_SnowSublimation_OpenArea_m"] += total_SnowSublimation_OpenArea_m;
	input->RepoVecDict["totalSnowEvapUnderTree_TS"].push_back(total_SnowSublimation_UnderTree_m);
	input->RepoDict["total_SnowSublimation_UnderTree_m"] += total_SnowSublimation_UnderTree_m;
	input->RepoVecDict["totalSnowEvapUnderShortVeg_TS"].push_back(total_SnowSublimation_UnderSVeg_m);
	input->RepoDict["total_SnowSublimation_UnderSVeg_m"] += total_SnowSublimation_UnderSVeg_m;
	input->RepoDict["total_SnowMelt_TreeCanopy_m"] += total_SnowMelt_TreeCanopy_m;
	input->RepoDict["total_SnowMelt_SVegCanopy_m"] += total_SnowMelt_SVegCanopy_m;
	input->RepoDict["total_SnowMelt_UnderTree_m"] += total_SnowMelt_UnderTree_m;
	input->RepoDict["total_SnowMelt_UnderSVeg_m"] += total_SnowMelt_UnderSVeg_m;
	input->RepoDict["total_SnowMelt_OpenArea_m"] += total_SnowMelt_OpenArea_m;

	input->RepoVecDict["totalWaterOnImperviousArea_TS"].push_back(totalWaterOnImperviousArea_m);
	input->RepoDict["totalWaterOnImperviousArea_m"] += totalWaterOnImperviousArea_m;
	input->RepoVecDict["totalImpDepEvap_TS"].push_back(totalImpDepEvap_m);
	input->RepoDict["totalImpDepEvap_m"] += totalImpDepEvap_m;
	input->RepoVecDict["totalImpDepStor_TS_m"].push_back(totalImpDepStor_m);
	input->RepoDict["totalImpDepStor_m"] += totalImpDepStor_m;
	//totalDCIAFlowToOutlet_m (m) is catchment area depth, transformed from pervious area depth in FlowSumCalc.cpp
	input->RepoVecDict["totalFlowToDCIA_TS"].push_back(totalDCIAFlowToOutlet_m);
	input->RepoDict["totalDCIAFlowToOutlet_m"] += totalDCIAFlowToOutlet_m;
	//totalImpFlowToSoil_m (m) is catchment area depth, transformed from impervious area depth as Runoff_ImperviousToPervious_m in ImperviousDepressionStorageCalc.cpp
	input->RepoVecDict["totalImpFlowToSoil_TS"].push_back(totalImpFlowToSoil_m);
	input->RepoDict["totalImpFlowToSoil_m"] += totalImpFlowToSoil_m;

	//input->RepoDict["totalWaterOnPerviousArea_m"] does not include folder->VarDict["Runoff_ImperviousToPervious_m"] 
	input->RepoVecDict["totalWaterOnPerviousArea_TS"].push_back(totalWaterOnPerviousArea_m);
	input->RepoDict["totalWaterOnPerviousArea_m"] += totalWaterOnPerviousArea_m;
	//input->RepoDict["totalWaterToSoil_m"] does include folder->VarDict["Runoff_ImperviousToPervious_m"] and hence can be larger than precipitation depth
	input->RepoVecDict["totalWaterToSoil_TS"].push_back(totalWaterToSoil_m);
	input->RepoDict["totalWaterToSoil_m"] += totalWaterToSoil_m;
	input->RepoVecDict["totalPerDepEvap_TS"].push_back(totalPerDepEvap_m);
	input->RepoDict["totalPerDepEvap_m"] += totalPerDepEvap_m;
	input->RepoVecDict["totalPerDepStore_TS_m"].push_back(totalPerDepStor_m);
	input->RepoDict["totalPerDepStor_m"] += totalPerDepStor_m;

	//input->RepoDict["total_Water_forSoilRecharge_m"] does include folder->VarDict["Runoff_ImperviousToPervious_m"] and hence can be larger than precipitation depth
	input->RepoVecDict["totalPerFlowToInfil_TS"].push_back(total_Water_forSoilRecharge_m);
	input->RepoDict["total_Water_forSoilRecharge_m"] += total_Water_forSoilRecharge_m;
	input->RepoDict["total_Drainage_macroPore_m"] += total_Drainage_macroPore_m;
	input->RepoVecDict["totalpmacroflow_TS"].push_back(total_Drainage_macroPore_m);
	input->RepoDict["total_InfilExInfil_m"] += total_InfilExInfil_m;
	input->RepoDict["total_SatEx_Infil_m"] += total_SatExInfil_m;
	input->RepoDict["total_Infiltration_m"] += total_Infiltration_m;

	input->RepoVecDict["totalInfilExQ_TS"].push_back(totalInfilExQ_m);
	input->RepoDict["totalInfilExQ_m"] += totalInfilExQ_m;
	input->RepoVecDict["totalSatExQ_TS"].push_back(totalSatExQ_m);
	input->RepoDict["totalSatExQ_m"] += totalSatExQ_m;

	input->RepoDict["totalRootZoneEvapoTranspiration_m"] += totalRootZoneEvapoTranspiration_m;
	input->RepoVecDict["aveSMD_m"].push_back(aveSMD_m);
	input->RepoVecDict["Storage_Used_RootZone_m"].push_back(Storage_Used_RootZone_m);
	input->RepoDict["total_Drainage_RootZone_m"] += total_Drainage_RootZone_m;

	input->RepoVecDict["Storage_Used_UnsatZone_m"].push_back(Storage_Used_UnsatZone_m);
	//total_unsatZoneFlow_m (m) is catchment area depth, transformed from pervious area depth in FlowSumCalc.cpp
	input->RepoDict["total_unsatZoneFlow_m"] += total_unsatZoneFlow_m;

	input->RepoDict["total_Runoff_Pervious_NR_m"] += total_Runoff_Pervious_NR_m;
	input->RepoDict["total_Runoff_Impervious_NR_m"] += total_Runoff_Impervious_NR_m;
	input->RepoDict["total_Runoff_Subsurface_NR_m"] += total_Runoff_Subsurface_NR_m;
	input->RepoDict["total_Runoff_All_NR_m"] += total_Runoff_All_NR_m;
}


