#include "WaterOnImperviousAreaCalc.h"
#include <iostream>


void WaterOnImperviousAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	double TotalImperviousCover_frac;
	//BulkArea computing totalImperviousArea for folder 
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		//Fraction computed within BuildDataOrganizer.cpp
		TotalImperviousCover_frac = folder->ParamDict["TotalImperviousCover_frac"];
	}
	//GI area computing totalImperviousArea for folder 
	//12/23/2020 Li from Robbie: folder->ParamDict["SurfaceWaterIA_frac"] was added to be able to track and modify the amount of surface water, this variable was defined in the StatHydro config file as a bulk area land cover term
	//12/23/2020 Keep this term for GI in case we need it later. If surfaceWaterIA was not defined in config files (in GI cases), folder->ParamDict["SurfaceWaterIA_frac"] ==0, in other cases, we can set it to 0.
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		TotalImperviousCover_frac = folder->ParamDict["SurfaceWaterIA_frac"]+ folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	}
	//Initialize variable
	double WaterOnImperviousAreaVal = 0.0;
	
	//If totalImperviousArea is > 0, gather water from various catchment features
 	if(TotalImperviousCover_frac)
	{
		//Continuity maintained when adjusting to folder area from cover type area (e.g., pervious or impervious). 
		//		This adjustment typically uses multiplication by a fraction to reduce the depth from the cover type area to the BulkArea
		//		ex. Depth_adjusted_CoverType_m = Depth_unadjusted_CoverType_m * Area_CoverType_fraction = Depth_unadjusted_CoverType_m * Area_CoverType_m2 / Area_Folder_m2
		//Continuity maintained when adjusting to cover type area (e.g., pervious or impervious) from folder area.
		//		This adjustment typically uses division by a fraction to increase the depth from BulkArea avg to cover type area.
		//		ex. Below the computation: 1st starts with depth on impervious area with tree cover, 
		//			2nd converts to depth on BulkArea, 3rd converts to depth on impervious area with and without trees		
		//WaterOnImperviousAreaVal accumulates water under tree
		//Compute water on impervious area (m), weighted for entire impervious area, by getting of (water under tree) * (fraction of under tree area that is impervious) / (total impervious fraction)
		WaterOnImperviousAreaVal = folder->VarDict["waterUnderTree"] *
			folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] / TotalImperviousCover_frac;

			//Continuity maintained when adjusting to folder area from cover type area (e.g., pervious or impervious). 
			//		This adjustment typically uses multiplication by a fraction to reduce the depth from the cover type area to the BulkArea
			//		ex. Depth_adjusted_CoverType_m = Depth_unadjusted_CoverType_m * Area_CoverType_fraction = Depth_unadjusted_CoverType_m * Area_CoverType_m2 / Area_Folder_m2
			//Continuity maintained when adjusting to cover type area (e.g., pervious or impervious) from folder area.
			//		This adjustment typically uses division by a fraction to increase the depth from BulkArea avg to cover type area.
			//		ex. Below the computation: 1st starts with depth on impervious area without tree cover, 
			//			2nd converts to depth on BulkArea, 3rd converts to depth on impervious area with and without trees
			//WaterOnImperviousAreaVal accumulates water in open area
			//Compute water on impervious area (m), weighted for entire impervious area, by adding (water in open area) * (fraction of open area that is impervious) / (total impervious fraction)
			WaterOnImperviousAreaVal += folder->VarDict["waterOpenArea"] *
			(folder->ParamDict["SurfaceWaterIA_frac"] + folder->ParamDict["ImperviousCover_frac"]) / TotalImperviousCover_frac;

		//waterOnImperviousArea (m) will have the fraction (1-DCIA) sent to pervious area for processing 
		//For BulkArea with no GI, waterOnImperviousArea (m) will have the fraction DCIA sent to outlet 
		//For BulkArea with GI, waterOnImperviousArea (m) may have a fraction of the fraction DCIA sent to GI
		//For GI, waterOnImperviousArea (m) is contributed to inflow from BulkArea 
		folder->VarDict["waterOnImperviousArea"] = WaterOnImperviousAreaVal;
	}
	//No impervious area, so waterOnImperviousArea = 0
	else
	{
		folder->VarDict["waterOnImperviousArea"] = 0.0;
	}

	storeValues(input, folder, timeStep);
}

void WaterOnImperviousAreaCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		folder->VarVecDict["waterOnImpArea"].emplace_back(folder->VarDict["waterOnImperviousArea"]);
	}
}
