# Title: TestToCompareHPOutputToStandards_PostFailure.py

# This program is a modified version of TestToCompareHPOutputToStandards.py for case when the test cases failed due to empty folders. 
# This program is used as follows:  
# 1. manually run the model for each empty output folder in the directory Testing_%Y_%m_%d_%H_%M_%S, using the associated Testing_%Y_%m_%d_%H_%M_%S input folder; e.g. HydroPlus.exe Testing_%Y_%m_%d_%H_%M_%S\Hydro_GI_bioRetention\input 
# 2. delete the results.txt file from the directory Testing_%Y_%m_%d_%H_%M_%S. 
# 3. run a new Python script TestToCompareHPOutputToStandards_PostFailure.py, stored in the directory HydroPlus\TestingFilesAndScript. This TestToCompareHPOutputToStandards_PostFailure.py takes as command line input the directory name Testing_%Y_%m_%d_%H_%M_%S used in step 1; e.g., python TestToCompareHPOutputToStandards_PostFailure.py Testing_%Y_%m_%d_%H_%M_%S 
# ===============================================================================

#import Python libraries
import subprocess
import os
import pandas as pd
import time
from random import uniform
import numpy as np
import xml.etree.ElementTree as ET
import csv 
import shutil
import glob
from filecmp import dircmp
import sys

#Folder name based on time format: time.strftime("Testing_%Y_%m_%d_%H_%M_%S")
timeN = sys.argv[1]

#define test_file as input ListOfTests.txt
test_file = "ListOfTests.txt"

#open test_file as f
with open(test_file) as f:
	#define data for each line in f with tuple command, reading objects separated by commas
    data=[tuple(line) for line in csv.reader(f)]
	
#define outputFile as results.txt within timeN folder
outputFile = timeN+"\\results.txt"

#define dictTests as strings to keep track of results
dictTests = {"Overall Test Results": "Passed"}

#for each testCase in data
for testCase in data:
     if testCase[0] == "Executable":
            #name HydroPlus executable
            hydro = testCase[1]
            continue
	#if 1st variable of data is "TestName" then treat as header file and continue to top of loop, and read in next line
     if testCase[0] == "TestName":
            continue
	#define foldName within timeN folder as 1st variable in data testCase, the unique test case name 
     foldName = time.strftime(timeN+"\\"+testCase[0])
	#define newoutpath within foldName folder with name for output
     newoutpath = os.path.join(os.path.dirname(__file__),foldName+'\\output')
	#define expoutpath within foldName folder with name for expectedoutput
     expoutpath = os.path.join(os.path.dirname(__file__),foldName+'\\expectedoutput')
	#define newinpath within foldName folder with name for input
     newinpath = os.path.join(os.path.dirname(__file__),foldName+'\\input')
	#make newoutpath if it does not exit
     if not os.path.exists(newoutpath):
        os.makedirs(newoutpath)
	#make newinpath if it does not exit
     if not os.path.exists(newinpath):
        os.makedirs(newinpath)
	#make expoutpath if it does not exit
     if not os.path.exists(expoutpath):
        os.makedirs(expoutpath)
		
	#define tc as input path, 2nd variable in data testCase, as upper case string
     tc = testCase[1].upper()

	#define tc as expectedoutput path, 3nd variable in data testCase, as upper case string
     tc = testCase[2].upper()
	 #for each file in the expectedoutput path +/*.*, finding all file paths with glob command

	#define fc1 as output from dircmp direct comparison between newoutpath and expoutpath files
     fc1 = dircmp(newoutpath, expoutpath)
	
	#open outputFile results.txt as f
     with open(outputFile,'a') as f:
        #define as default, dictTests for each testCase 1st variable as Passed
         dictTests[testCase[0]] =  "Passed"
        #write testCase name
         f.write("Test Name:"+testCase[0]+"\n")
        #write testCase path of output string and actual path
         f.write("Path of test generated output:")
         f.write(newoutpath)
         f.write("\n")
        #write testCase path of expectedoutput string and actual path
         f.write("Path of expected/standard output:")
         f.write(expoutpath)
         f.write("\n")
        #for each instance of the right_list, expectedoutput, of fc1 dircmp object
         for r in fc1.right_list:
            #if the instance of the right_list is not in the left_list, change default from Passed to Failed
             if r not in fc1.left_list:
                #write test case failed due to missing file outputs, and list missing file
                f.write("\t Test case failed.\n The following file is in the expected outputs, but missing from test dir.\n\t")
                #write right_list file
                f.write(r)
                f.write("\n")
                #write test results failed as result
                dictTests["Overall Test Results"]="Failed"
                dictTests[testCase[0]] = "Failed"
		#if diff_files function has found different files in output and expected output, change default from Passed to Failed
         if len(fc1.diff_files) != 0:
			#write why test case failed
             f.write("Test case failed! These file(s) are different:")
			#write overall test results as failed
             dictTests["Overall Test Results"]= "Failed"
			#write specific test case result as failed
             dictTests[testCase[0]] =  "Failed"
			#for diff in list of different files
             for diff in fc1.diff_files:
                #write diff file to f
                f.write(diff+ " ")
                f.write("\n")
         f.write("==============================================================================\n")
         #close outputFile
         f.close()
		 
#print to screen out dictTests
print(dictTests)

#open outputFile results.txt as f
with open(outputFile,'a') as f:
    #for each testCase result in dictTests
  for testRes in dictTests:
         #write result of testCase
         f.write(testRes + ":\t"+ str(dictTests[testRes]))
         
         f.write("\n")
f.close()

