# Purpose: to copy output files from TestingFilesAndScript\Testing_YYYY_MM_DD_HH_MM_SS folders to TestingFilesAndScript\TestCases ExpectedOutput folders
#           This is required after making updates to UnifiedHydro that cause test cases to fail, and before the SVN Commit. 
#           This should only be used when it is known the new UnifiedHydro output is correct, and the expectedoutput is now incorrect.
# Requires: File ListOfTests.txt (which has paths to ExpectedOutput directories) and command line argument Testing_YYYY_MM_DD_HH_MM_SS 
# Run from the directory TestingFilesAndScript.
# At command prompt, give the path to the folder containing the new test results, Testing_YYYY_MM_DD_HH_MM_SS 
# Example of the command prompt C:\iTree\UnifiedHydro\TestingFilesAndScript>python z_Copy_TestCaseOutput_to_ExpectedOutput.py Testing_2020_11_11_07_08_09
# Created by ESF student Li Zhang, working with Professor T. Endreny in fall 2020
# Ver1 11/15/2020 


#import Python libraries
import subprocess
import os
import time
from random import uniform
import xml.etree.ElementTree as ET
import csv 
import shutil
import glob
from filecmp import dircmp
import sys

#folder containing model outputs that will be copied into expectedoutput, read from command line >python z_Copy_TestCaseOutput_to_ExpectedOutput.py Testing_2020_11_11_07_08_09
Testresultfolder = sys.argv[1]

#path to working directory 
Testpath = os.path.dirname(__file__)

#Create path to folder containing model outputs that will be copied into expectedoutput
Testresultpath=os.path.join(Testpath,Testresultfolder)

#while running in IDLE, use these 3 lines
# Testpath ="C:\\iTree\\UnifiedHydro\\TestingFilesAndScript\\"
# Testresultfolder="Testing_2020_11_04_13_32_31"
# Testresultpath= Testpath + Testresultfolder

test_file = "ListOfTests.txt"

#open ListOfTests.txt as f, read each line into a tuple
with open(Testpath + test_file, "r") as f:
	#define data for each line in f with tuple command, reading objects separated by commas
    data=[tuple(line) for line in csv.reader(f)]
    
#For each line in ListOfTests.txt, creating testCase as the line of input as a vector
for testCase in data:
     #skip over the first two lines of the ListOfTests.txt if the first word [0] is a key word for other than the directory
     if testCase[0] == "Executable":
            continue
     if testCase[0] == "TestName":
            continue
     #Define foldName as the unique folder name containing the new output directory
     foldName = "\\"+ str(testCase[0])
     #Define newoutput_path as the path and directory containing the new outputs that will be copied into expectedoutputs
     newoutput_path = os.path.join(Testresultpath + foldName + '\\output')
       
     #Define expectedoutput_path as the path and directory containing the expected outputs that will be overwritten, replaced by new outputs
     expectedoutput_path = str(testCase[2])
    
     
    
    #For each file in the old expected output folder, we delete all files
     for file in glob.glob(expectedoutput_path+'/*.*'):
         try:
             os.remove(file)            
         except Exception as e:
             print('Failed to delete %s. Reason: %s' % (file, e))
    
    #For every file in the new output directory 
     for file in glob.glob(newoutput_path+'/*.*'):
         try:
             #Use shutil function to copy the file to the expectedoutput_path
             shutil.copy(file, expectedoutput_path)            
         except Exception as e:
             print('Failed to copy %s. Reason: %s' % (file, e))
    
     #Messages: delete and copy
     print("Deleting files in: {} ".format(expectedoutput_path))
     print("Copying files from: {} to: {}".format(newoutput_path, expectedoutput_path))

