#coding=windows-1252
# This first line, #coding=windows-1252, is required to run the program.

# Purpose: to graph in Python the surface contour plot of average air temperature metric across 11 values of tree cover TC and impervious cover IC, a total of 121 unique IC and TC combinations
# Requires: i-Tree Cool Air model output in the form of Blockgroup results (daily and hourly), with a unique block group for each of the 121 unique TC and IC combinations.
# Requires: Seems to need at least 5 days of outputs to construct averages
# At command prompt, give the path to the folder containing the Blockgroup results
# Example of the command prompt C:\iTree\projects>python z_Map_CoolAirTair_11x11_TC-IC_combinations.py C:\iTree\UnifiedHydro\TestingFilesAndScript\TestCases\SpatialCoolAir\LCScale_11x11Grid\expectedoutput

# Created by ESF student Li Zhang, working with Professor T. Endreny in spring 2020
# V4 11/18/2020

# Requires python 3 interpreter to use matplotlib (and associated mpl_toolkits)
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
import array
import matplotlib.pyplot as plt
from matplotlib import cm
from matplotlib.ticker import LinearLocator, FormatStrFormatter
import numpy as np
import linecache
import pandas as pd
import os
from os import path
import sys
# from matplotlib.mlab import griddata #has been removed
from scipy.interpolate import griddata

mod = sys.modules[__name__]

#file path to model inputs, read from command line >python z_Map_CoolAirTair_11x11_TC-IC_combinations.py C:\iTree\UnifiedHydro\TestingFilesAndScript\TestCases\SpatialCoolAir\LCScale_11x11Grid\expectedoutput
#backslash is added to path input
fpath = sys.argv[1]  + '\\'

#change directory to file path with model inputs and where outputs are written
os.chdir(fpath)
path_out = fpath

# ask user
r = input(
    'Select an i-Tree Cool Air output data option:  Hourly Air Temperature (H) or Daily Metrics of Air Temperature (D):')
if r == 'H':
    r2 = input(
        'Select the frequency of Air Temp vs IC and TC plots: Year (Y), Seasonal (S), Month (M), Day (D), or Hour (H):')
if r == 'D':
    r2 = None

# if user wants to have hourly, daily, monthly yearly and seasonally plots, read the BlockGroup 1-121 files into matrix
if r == "H":
    # read multiple files
    a = "BlockGroup"
    c = ".txt"

    # to know the date and time
    timeseries = np.loadtxt(path_out + "BlockGroup1.txt", skiprows=1, usecols=[0])
    timeseries = timeseries.astype(int)
    starttime = int(timeseries[0])
    endtime = int(timeseries[-1])
    print("start time", starttime, "end time", endtime)

    year = timeseries // 1000000
    month = (timeseries // 10000) % 10
    day = (timeseries // 100) % 100
    time = timeseries % 100

    uniq_year = np.unique(year)

    # create a matrix to save all the temperature data (each hour 121 data in a row)
    matrix = np.empty([len(timeseries), 121])
    for b in range(0, 121):
        file_name = a + str(b + 1) + c
        temp = path_out + file_name
        tempT = np.loadtxt(temp, skiprows=1, usecols=[1])
        matrix[:, b] = tempT

    # create a matrix with time, with column Year, Month, Day, Time, 121 data)
    matrix_w_date = np.column_stack((year, month, day, time, matrix))

    print('Finish reading BlockGroup files')

# if user wants to have max or min plots, read the BlockGroupDaily 1-121 files into matrix2
if (r == "D"):
    # read multiple files
    a = "BlockGroupDaily"
    c = ".txt"

    # to know the date and time
    days = np.loadtxt(path_out + "BlockGroupDaily1.txt", skiprows=1, usecols=[0])
    # if just 1 day, the syntax days[0] will cause error, so we separate it into 2 senario,=1 or >1
    if days.size == 1:
        days = np.array([days.astype(int)])
    if days.size > 1:
        days = np.array(days.astype(int))
    startdate = days[0]
    enddate = days[-1]
    print("start date", startdate, "end date", enddate)

    year = days // 10000
    month = (days // 100) % 10
    day = days % 100

    uniq_year = np.unique(year)

    # create a matrix2 to save all the temperature data (121 data in a row)
    # a 121 values
    # b 121 values
    # ...
    matrix2 = np.empty([11, 121])
    # save 11 max or min dates
    maxmindate = np.empty([121, 11])

    for b in range(0, 121):
        file_name = a + str(b + 1) + c
        temp = path_out + file_name
        # read each BlockGroupDaily file and put it in TempT (no header)
        tempT = np.loadtxt(temp, skiprows=1, usecols=range(1, 12))

        for i in range(0, 11):
            if days.size == 1:
                tempatok = tempT[i]
                matrix2[i, b] = tempatok
                maxmindate[b, i] = days

            if days.size > 1:
                # get the Max of these columns and put them into rows in matrix2
                if (i == 0) | (i == 2) | (i == 3) | (i == 4) | (i == 5) | (i == 6) | (i == 7) | (i == 8):
                    tempatok = np.amax(tempT[:, i])
                    maxmindate[b, i] = days[np.argmax(tempT[:, i])]
                # get the Min of these columns and put them into rows in matrix2
                if (i == 1) | (i == 9) | (i == 10):
                    tempatok = np.amin(tempT[:, i])
                    maxmindate[b, i] = days[np.argmin(tempT[:, i])]
                matrix2[i, b] = tempatok
    # matrix2_w_date=np.column_stack((days,matrix2))
    maxmindate.astype('int')
    print('Finish reading BlockGroupDaily files')
    # txtpath=path_out+"/%s.txt" % ("matrix2")
    # np.savetxt(txtpath,matrix2,fmt="%f")
########################################
# create plots for each hour
if (r == 'H') & (r2 == 'H'):
    # create folders to save the plots
    # Leaf directory
    directory = "hourly"
    # Path
    hr_outputpath = os.path.join(path_out, directory)
    # check if the path is existing
    isdir = os.path.isdir(hr_outputpath)
    # Create the directory
    if not isdir:
        os.makedirs(hr_outputpath)
        print("Directory '% s' created" % directory)

    # create plots for each hour in floder hourlyplot
    for i in range(0, len(timeseries)):
        # to find the data for that hour in the matrix
        def f(a, b):
            y = (b / 10) * 11 + a / 10
            y = int(y)
            z = matrix[i, y]
            z.astype(float)
            return z


        # create a new matrix to save the 11*11 2d data
        Z_m = np.empty([11, 11])
        for X in range(0, 110, 10):
            for Y in range(0, 110, 10):
                Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
        # begin plotting
        Xarr = np.arange(0, 110, 10)
        Yarr = np.arange(0, 110, 10)
        X, Y = np.meshgrid(Xarr, Yarr)
        Z = Z_m

        fig = plt.figure()
        ax = fig.gca(projection='3d')
        surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
        ax.set_xlabel('TC')
        ax.set_ylabel('IC')
        ax.set_zlabel('Temperature')

        # Customize the z axis.
        ax.zaxis.set_major_locator(LinearLocator(10))
        ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
        fig.colorbar(surf, shrink=0.5, aspect=5)

        figpath = hr_outputpath + "/%s.png" % (timeseries[i])
        plt.savefig(figpath, format='png')

        # txtpath=hr_outputpath+"/%s.txt" % (timeseries[i])
        # np.savetxt(txtpath,Z_m)
        # print('txt Saved')
        # plt.show()
        # print('Showed')
        plt.close()
    print('Saved plots for each hour')

########################################
# create plots for each year
if (r == 'H') & (r2 == 'Y'):
    # Leaf directory
    directory = "yearly"
    # Path
    y_outputpath = os.path.join(path_out, directory)
    # check if the path is existing
    isdir = os.path.isdir(y_outputpath)
    # Create the directory
    if not isdir:
        os.makedirs(y_outputpath)
        print("Directory '% s' created" % directory)

    for j in range(0, len(uniq_year)):
        matrix_filter = matrix_w_date[matrix_w_date[:, 0] == uniq_year[j]]
        if len(matrix_filter) != 0:
            matrix_year = np.empty([1, 121])
            for k in range(0, 121):
                matrix_year[:, k] = np.mean(matrix_filter[:, k + 4])


            def f(a, b):
                y = (b / 10) * 11 + a / 10
                y = int(y)
                z = matrix_year[0, y]
                z.astype(float)
                return z


            Z_m = np.empty([11, 11])
            for X in range(0, 110, 10):
                for Y in range(0, 110, 10):
                    Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
            # begin plotting
            Xarr = np.arange(0, 110, 10)
            Yarr = np.arange(0, 110, 10)
            X, Y = np.meshgrid(Xarr, Yarr)
            Z = Z_m

            fig = plt.figure()
            ax = fig.gca(projection='3d')
            surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            ax.set_xlabel('TC')
            ax.set_ylabel('IC')
            ax.set_zlabel('Mean Temperature')

            # Customize the z axis.
            ax.zaxis.set_major_locator(LinearLocator(10))
            ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
            fig.colorbar(surf, shrink=0.5, aspect=5)

            figpath = y_outputpath + "/%s.png" % (uniq_year[j])
            plt.savefig(figpath, format='png')

            # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
            # np.savetxt(txtpath,Z_m)
            # print('txt Saved')
            # plt.show()
            # print('Showed')
            plt.close()
    print('Saved plots for each year (averaged)')

####################################
# create plots for each month
if (r == 'H') & (r2 == 'M'):
    # Leaf directory
    directory = "monthly"
    # Path
    m_outputpath = os.path.join(path_out, directory)
    # check if the path is existing
    isdir = os.path.isdir(m_outputpath)
    # Create the directory
    if not isdir:
        os.makedirs(m_outputpath)
        print("Directory '% s' created" % directory)

    for i in range(0, len(uniq_year)):
        for j in range(0, 12):
            matrix_filter = matrix_w_date[(matrix_w_date[:, 0] == uniq_year[i]) & (matrix_w_date[:, 1] == j + 1)]
            if len(matrix_filter) != 0:
                matrix_month = np.empty([1, 121])
                for k in range(0, 121):
                    matrix_month[:, k] = np.mean(matrix_filter[:, k + 4])


                # a function for finding the data for each blockgroup in matrix
                def f(a, b):
                    y = (b / 10) * 11 + a / 10
                    y = int(y)
                    z = matrix_month[0, y]
                    z.astype(float)
                    return z


                # put the data we found in a new 11*11 martix Z_m for plotting
                Z_m = np.empty([11, 11])
                for X in range(0, 110, 10):
                    for Y in range(0, 110, 10):
                        Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
                # begin plotting
                Xarr = np.arange(0, 110, 10)
                Yarr = np.arange(0, 110, 10)
                X, Y = np.meshgrid(Xarr, Yarr)
                Z = Z_m

                fig = plt.figure()
                ax = fig.gca(projection='3d')
                surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
                ax.set_xlabel('TC')
                ax.set_ylabel('IC')
                ax.set_zlabel('Mean Temperature')

                # Customize the z axis.
                ax.zaxis.set_major_locator(LinearLocator(10))
                ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
                fig.colorbar(surf, shrink=0.5, aspect=5)

                figpath = m_outputpath + "/%s_%s.png" % (uniq_year[i], (j + 1))
                plt.savefig(figpath, format='png')

                # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
                # np.savetxt(txtpath,Z_m)
                # print('txt Saved')
                # plt.show()
                # print('Showed')
                plt.close()
    print('Saved plots for each month (averaged)')

####################################
# create plots for each day
# create matrix to save averaged data for each year
if (r == 'H') & (r2 == 'D'):
    # Leaf directory
    directory = "daily"
    # Path
    d_outputpath = os.path.join(path_out, directory)
    # check if the path is existing
    isdir = os.path.isdir(d_outputpath)
    # Create the directory
    if not isdir:
        os.makedirs(d_outputpath)
        print("Directory '% s' created" % directory)
    # set up the filter to find the day that year and month
    for i in range(0, len(uniq_year)):
        for j in range(0, 12):
            for k in range(0, 31):
                matrix_filter = matrix_w_date[(matrix_w_date[:, 0] == uniq_year[i]) & (matrix_w_date[:, 1] == j + 1) & (
                            matrix_w_date[:, 2] == k + 1)]
                if len(matrix_filter) != 0:
                    matrix_day = np.empty([1, 121])
                    for l in range(0, 121):
                        matrix_day[0, l] = np.mean(matrix_filter[:, l + 4])


                    # a function for finding the data for each blockgroup in matrix
                    def f(a, b):
                        y = (b / 10) * 11 + a / 10
                        y = int(y)
                        z = matrix_day[0, y]
                        z.astype(float)
                        return z


                    # put the data we found in a new 11*11 martix Z_m for plotting
                    Z_m = np.empty([11, 11])
                    for X in range(0, 110, 10):
                        for Y in range(0, 110, 10):
                            Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
                    # begin plotting
                    Xarr = np.arange(0, 110, 10)
                    Yarr = np.arange(0, 110, 10)
                    X, Y = np.meshgrid(Xarr, Yarr)
                    Z = Z_m

                    fig = plt.figure()
                    ax = fig.gca(projection='3d')
                    surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
                    ax.set_xlabel('TC')
                    ax.set_ylabel('IC')
                    ax.set_zlabel('Mean Temperature')

                    # Customize the z axis.
                    ax.zaxis.set_major_locator(LinearLocator(10))
                    ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
                    fig.colorbar(surf, shrink=0.5, aspect=5)

                    figpath = d_outputpath + "/%s_%s_%s.png" % (uniq_year[i], j + 1, k + 1)
                    plt.savefig(figpath, format='png')

                    # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
                    # np.savetxt(txtpath,Z_m)
                    # print('txt Saved')
                    # plt.show()
                    # print('Showed')
                    plt.close()
    print('Saved plots for each day (averaged)')

####################################
# create plots for each season
# create matrix to save averaged data for each year
if (r == 'H') & (r2 == 'S'):
    # Leaf directory
    directory = "season"
    # Path
    s_outputpath = os.path.join(path_out, directory)
    # check if the path is existing
    isdir = os.path.isdir(s_outputpath)
    # Create the directory
    if not isdir:
        os.makedirs(s_outputpath)
        print("Directory '% s' created" % directory)
    # set up the filter to find the day that year and month
    for i in range(0, len(uniq_year)):
        matrix_filter = matrix_w_date[(matrix_w_date[:, 0] == uniq_year[i]) & (
                    (matrix_w_date[:, 1] == 3) | (matrix_w_date[:, 1] == 4) | (matrix_w_date[:, 1] == 5))]
        if len(matrix_filter) != 0:
            matrix_season = np.empty([1, 121])
            for k in range(0, 121):
                matrix_season[:, k] = np.mean(matrix_filter[:, k + 4])


            # a function for finding the data for each blockgroup in matrix
            def f(a, b):
                y = (b / 10) * 11 + a / 10
                y = int(y)
                z = matrix_season[0, y]
                z.astype(float)
                return z


            # put the data we found in a new 11*11 martix Z_m for plotting
            Z_m = np.empty([11, 11])
            for X in range(0, 110, 10):
                for Y in range(0, 110, 10):
                    Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
            # begin plotting
            Xarr = np.arange(0, 110, 10)
            Yarr = np.arange(0, 110, 10)
            X, Y = np.meshgrid(Xarr, Yarr)
            Z = Z_m

            fig = plt.figure()
            ax = fig.gca(projection='3d')
            surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            ax.set_xlabel('TC')
            ax.set_ylabel('IC')
            ax.set_zlabel('Mean Temperature')

            # Customize the z axis.
            ax.zaxis.set_major_locator(LinearLocator(10))
            ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
            fig.colorbar(surf, shrink=0.5, aspect=5)

            figpath = s_outputpath + "/%s_%s.png" % (uniq_year[i], 'spring')
            plt.savefig(figpath, format='png')

            # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
            # np.savetxt(txtpath,Z_m)
            # print('txt Saved')
            # plt.show()
            # print('Showed')
            plt.close()
            print("Saved plots for '%s' spring" % uniq_year[i])

    for i in range(0, len(uniq_year)):
        matrix_filter = matrix_w_date[(matrix_w_date[:, 0] == uniq_year[i]) & (
                    (matrix_w_date[:, 1] == 6) | (matrix_w_date[:, 1] == 7) | (matrix_w_date[:, 1] == 8))]
        if len(matrix_filter) != 0:
            matrix_season = np.empty([1, 121])
            for k in range(0, 121):
                matrix_season[:, k] = np.mean(matrix_filter[:, k + 4])


            # a function for finding the data for each blockgroup in matrix
            def f(a, b):
                y = (b / 10) * 11 + a / 10
                y = int(y)
                z = matrix_season[0, y]
                z.astype(float)
                return z


            # put the data we found in a new 11*11 martix Z_m for plotting
            Z_m = np.empty([11, 11])
            for X in range(0, 110, 10):
                for Y in range(0, 110, 10):
                    Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
            # begin plotting
            Xarr = np.arange(0, 110, 10)
            Yarr = np.arange(0, 110, 10)
            X, Y = np.meshgrid(Xarr, Yarr)
            Z = Z_m

            fig = plt.figure()
            ax = fig.gca(projection='3d')
            surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            ax.set_xlabel('TC')
            ax.set_ylabel('IC')
            ax.set_zlabel('Mean Temperature')

            # Customize the z axis.
            ax.zaxis.set_major_locator(LinearLocator(10))
            ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
            fig.colorbar(surf, shrink=0.5, aspect=5)

            figpath = s_outputpath + "/%s_%s.png" % (uniq_year[i], 'summer')
            plt.savefig(figpath, format='png')

            # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
            # np.savetxt(txtpath,Z_m)
            # print('txt Saved')
            # plt.show()
            # print('Showed')
            plt.close()
            print("Saved plots for '%s' summer" % uniq_year[i])

    for i in range(0, len(uniq_year)):
        matrix_filter = matrix_w_date[(matrix_w_date[:, 0] == uniq_year[i]) & (
                    (matrix_w_date[:, 1] == 9) | (matrix_w_date[:, 1] == 10) | (matrix_w_date[:, 1] == 11))]
        if len(matrix_filter) != 0:
            matrix_season = np.empty([1, 121])
            for k in range(0, 121):
                matrix_season[:, k] = np.mean(matrix_filter[:, k + 4])


            # a function for finding the data for each blockgroup in matrix
            def f(a, b):
                y = (b / 10) * 11 + a / 10
                y = int(y)
                z = matrix_season[0, y]
                z.astype(float)
                return z


            # put the data we found in a new 11*11 martix Z_m for plotting
            Z_m = np.empty([11, 11])
            for X in range(0, 110, 10):
                for Y in range(0, 110, 10):
                    Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
            # begin plotting
            Xarr = np.arange(0, 110, 10)
            Yarr = np.arange(0, 110, 10)
            X, Y = np.meshgrid(Xarr, Yarr)
            Z = Z_m

            fig = plt.figure()
            ax = fig.gca(projection='3d')
            surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            ax.set_xlabel('TC')
            ax.set_ylabel('IC')
            ax.set_zlabel('Mean Temperature')

            # Customize the z axis.
            ax.zaxis.set_major_locator(LinearLocator(10))
            ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
            fig.colorbar(surf, shrink=0.5, aspect=5)

            figpath = s_outputpath + "/%s_%s.png" % (uniq_year[i], 'autumn')
            plt.savefig(figpath, format='png')

            # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
            # np.savetxt(txtpath,Z_m)
            # print('txt Saved')
            # plt.show()
            # print('Showed')
            plt.close()
            print("Saved plots for '%s' autumn" % uniq_year[i])

    for i in range(0, len(uniq_year)):
        matrix_filter = matrix_w_date[(matrix_w_date[:, 0] == uniq_year[i]) & (
                    (matrix_w_date[:, 1] == 12) | (matrix_w_date[:, 1] == 1) | (matrix_w_date[:, 1] == 2))]
        if len(matrix_filter) != 0:
            matrix_season = np.empty([1, 121])
            for k in range(0, 121):
                matrix_season[:, k] = np.mean(matrix_filter[:, k + 4])


            # a function for finding the data for each blockgroup in matrix
            def f(a, b):
                y = (b / 10) * 11 + a / 10
                y = int(y)
                z = matrix_season[0, y]
                z.astype(float)
                return z


            # put the data we found in a new 11*11 martix Z_m for plotting
            Z_m = np.empty([11, 11])
            for X in range(0, 110, 10):
                for Y in range(0, 110, 10):
                    Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))
            # begin plotting
            Xarr = np.arange(0, 110, 10)
            Yarr = np.arange(0, 110, 10)
            X, Y = np.meshgrid(Xarr, Yarr)
            Z = Z_m

            fig = plt.figure()
            ax = fig.gca(projection='3d')
            surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            ax.set_xlabel('TC')
            ax.set_ylabel('IC')
            ax.set_zlabel('Mean Temperature')

            # Customize the z axis.
            ax.zaxis.set_major_locator(LinearLocator(10))
            ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
            fig.colorbar(surf, shrink=0.5, aspect=5)

            figpath = s_outputpath + "/%s_%s.png" % (uniq_year[i], 'winter')
            plt.savefig(figpath, format='png')

            # txtpath=y_outputpath+"/%s.txt" % (uniq_year[j])
            # np.savetxt(txtpath,Z_m)
            # print('txt Saved')
            # plt.show()
            # print('Showed')
            plt.close()
            print("Saved plots for '%s' winter" % uniq_year[i])

########################################
# plot for file BlockGroupDaily#.txt

if (r == "D"):
    directory = "Daily_Metrics"
    name = np.array(['RawTairMax.txt', 'RawTairMin.txt', 'RawTairAvg.txt', 'RawTdewAvg.txt', 'HeatIndexMax_NWS.txt',
                     'HeatIndexAvg_NWS.txt', 'HeatIndex2Max_Lin.txt', 'HtIdx2AvgDlyHI_Lin.txt',
                     'HtIdx2DlyAvgTaTd_Lin.txt', 'WindchillMin.txt', 'WindchillAvg.txt'])
    atok_outputpath = os.path.join(path_out, directory)
    # check if the path is existing
    isdir = os.path.isdir(atok_outputpath)
    # Create the directory
    if not isdir:
        os.makedirs(atok_outputpath)
        print("Directory '% s' created" % directory)

    for i in range(0, 11):
        def f(a, b):
            y = (b / 10) * 11 + a / 10
            y = int(y)
            z = matrix2[i, y]
            z.astype(float)
            return z


        Z_m = np.empty([11, 11])
        for X in range(0, 110, 10):
            for Y in range(0, 110, 10):
                Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))

        # begin plotting
        Xarr = np.arange(0, 110, 10)
        Yarr = np.arange(0, 110, 10)
        X, Y = np.meshgrid(Xarr, Yarr)
        Z = Z_m

        fig = plt.figure()
        ax = fig.gca(projection='3d')
        surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
        ax.set_xlabel('TC')
        ax.set_ylabel('IC')
        ax.set_zlabel('Temperature')

        # Customize the z axis.
        ax.zaxis.set_major_locator(LinearLocator(10))
        ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
        fig.colorbar(surf, shrink=0.5, aspect=5)

        figpath = atok_outputpath + "/%s.png" % (name[i])
        plt.savefig(figpath, format='png')
        # txtpath=hr_outputpath+"/%s.txt" % (i+starttime)
        # np.savetxt(txtpath,Z_m)
        # print('txt Saved')
        # plt.show()
        # print('Showed')
        plt.close()
    print("Saved plots for '% s'" % directory)

    #########################################
    # generate txt files
    header = ['Block', 'RawTairMax', 'RawTairMin', 'RawTairAvg', 'RawTdewAvg', 'HeatIndexMax_NWS', 'HeatIndexAvg_NWS',
              'HeatIndex2Max_Lin', 'HtIdx2AvgDlyHI_Lin', 'HtIdx2DlyAvgTaTd_Lin', 'WindchillMin', 'WindchillAvg']
    header_v = np.array([np.arange(1, 122, dtype=np.int)])
    matrix_date = np.column_stack((header_v.T, maxmindate.astype('int')))
    matrix_date = np.vstack((header, matrix_date))

    info_headers = "a.RawTairMax  plot the maximum\r\nb.RawTairMin  plot the minimum\r\nc.RawTairAvg  plot the maximum\r\nd.RawTdewAvg  plot the maximum\r\ne.HeatIndexMax_NWS  plot the maximum\r\nf.HeatIndexAvg_NWS  plot the maximum\r\ng.HeatIndex2Max_Lin  plot the maximum\r\nh.HtIdx2AvgDlyHI_Lin  plot the maximum\r\ni.HtIdx2DlyAvgTaTd_Lin  plot the maximum\r\nj.WindchillMin  plot the minimum\r\nk.WindchillAvg  plot the minimum\r\n"

    txtpath = atok_outputpath + "/%s.txt" % (directory)
    np.savetxt(txtpath, matrix_date, fmt="%s", header=info_headers, comments='')
    print("Date output file saved for 11 variables")

    info_date = np.empty([11, 2])
    for i in range(0, 11):
        repeatedmax = 0
        datemax = 0
        uniquedate = np.unique(maxmindate[:, i])
        for j in range(0, len(uniquedate)):
            temp = maxmindate[:, i]
            # temp.astype('int')
            if np.count_nonzero(temp == uniquedate[j]) > repeatedmax:
                repeatedmax = np.count_nonzero(temp == uniquedate[j])
                datemax = uniquedate[j]
        info_date[i, 0] = datemax
        info_date[i, 1] = repeatedmax

    info = np.empty([11, 3])

    info[:, 0] = np.repeat(len(days), 11)
    info[:, 1] = info_date[:, 0]
    info[:, 2] = info_date[:, 1]
    MaxorMin = np.array(['Max', 'Min', 'Max', 'Max', 'Max', 'Max', 'Max', 'Max', 'Max', 'Min', 'Min'])
    header_v2 = np.array(
        ['RawTairMax.png', 'RawTairMin.png', 'RawTairAvg.png', 'RawTdewAvg.png', 'HeatIndexMax_NWS.png',
         'HeatIndexAvg_NWS.png', 'HeatIndex2Max_Lin.png', 'HtIdx2AvgDlyHI_Lin.png', 'HtIdx2DlyAvgTaTd_Lin.png',
         'WindchillMin.png', 'WindchillAvg.png'])
    header2 = [["*Number of blocks reached the max/min T on that day", "", "", "", ""],
               ["filename", "Uniquedates", "YYYYMMDD", "Blocks", "Type"]]

    matrix_info = np.column_stack((header_v2.T, info.astype('int'), MaxorMin.T))
    matrix_info = np.vstack((header2, matrix_info))
    txtpath = atok_outputpath + "/%s.txt" % ('AirTempMetric-Info')
    np.savetxt(txtpath, matrix_info, fmt="%s", comments='')
    print('AirTempMetric-Info.txt saved')

    flag = True
    while flag == True:
        r3 = input('Do you want to rotate a 3D plot and save it manually? (Y/N):')
        if r3 == "Y":
            flag = True
            r4 = input(
                'Please choose:\r\nnMaximun RawTairMax(a)\r\nMinimun RawTairMin(b)\r\nMaximun RawTairAvg(c)\r\nMaximum RawTdewAvg(d)\r\nMaximum HeatIndexMax_NWS(e)\r\nMaximum HeatIndexAvg_NWS(f)\r\nMaximum HeatIndex2Max_Lin(g)\r\nMaximum HtIdx2AvgDlyHI_Lin(h)\r\nMaximum HtIdx2DlyAvgTaTd_Lin(i)\r\nMinimum WindchillMin(j)/Minimum WindchillAvg(k)\r\n:')
            if r4 == 'a':
                rowatok = 0
            if r4 == 'b':
                rowatok = 1
            if r4 == 'c':
                rowatok = 2
            if r4 == 'd':
                rowatok = 3
            if r4 == 'e':
                rowatok = 4
            if r4 == 'f':
                rowatok = 5
            if r4 == 'g':
                rowatok = 6
            if r4 == 'h':
                rowatok = 7
            if r4 == 'i':
                rowatok = 8
            if r4 == 'j':
                rowatok = 9
            if r4 == 'k':
                rowatok = 10


            def f(a, b):
                y = (b / 10) * 11 + a / 10
                y = int(y)
                z = matrix2[rowatok, y]
                z.astype(float)
                return z


            Z_m = np.empty([11, 11])
            for X in range(0, 110, 10):
                for Y in range(0, 110, 10):
                    Z_m[int(Y / 10), int(X / 10)] = f(int(X), int(Y))

            # begin plotting
            Xarr = np.arange(0, 110, 10)
            Yarr = np.arange(0, 110, 10)
            X, Y = np.meshgrid(Xarr, Yarr)
            Z = Z_m

            fig = plt.figure()
            ax = fig.gca(projection='3d')
            surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            ax.set_xlabel('TC')
            ax.set_ylabel('IC')
            ax.set_zlabel('Temperature')

            # Customize the z axis.
            ax.zaxis.set_major_locator(LinearLocator(10))
            ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))
            fig.colorbar(surf, shrink=0.5, aspect=5)

            # figpath=atok_outputpath+"/%s.png" % (directory)
            # plt.savefig(figpath,format='png')

            plt.show()
            # print('Showed')
            # plt.close()
            # print("Saved plots for '% s'"% directory)
        else:
            flag = False
