# Purpose: Water budget calculation using terms in CumWaterBalance.dat; all variable names and values are assigned from the file.
# Requires: command line argument with the directory containing CumWaterBalance.dat 
# Example of the command prompt C:\iTree\UnifiedHydro\TestingFilesAndScript>python WaterbalanceTest.py C:\iTree\projects\case1\output
# Created by ESF student Li Zhang, modified by Professor T. Endreny in fall 2020
# Ver1 12/27/2020 

from scipy.interpolate import griddata
import subprocess
import os
import time
from random import uniform
import xml.etree.ElementTree as ET
import csv 
import shutil
import glob
from filecmp import dircmp
import sys
#for exponential command
import math

#Test result folder name
Testresult_path = sys.argv[1]
#cte 20210715 Consider 2nd CLI argument to designate if Input file is for CumWaterBalance.dat or CumWaterBalance_[GIType].dat
#HP_Output_file contains the water balance terms written by HydroPlus.exe 
HP_Output_file="CumWaterBalance.dat" #"CumWaterBalance_BioRetention.dat" or "CumWaterBalance.dat"
Testresult_filepath=os.path.join(Testresult_path,HP_Output_file)

def filereader(file_path):
    #Python with open read command, naming file file_input
    with open(file_path, "r") as file_input:
        #Python csv.reader command creates and populates file_database from file_input, using comma delimiter
        file_database=csv.reader(file_input,delimiter=',')
        #initiate file_lines as as empty list
        file_lines=[]
        #loop through each line in the file_database, reading in file_line
        for file_line in file_database:
            #append file_line to file_lines list, which contains 2 items for each element, variable_name and variable_value
            file_lines.append(file_line)
    #close function by returning file_lines list
    return file_lines
    
#read in HP_Output_file using filereader function
file_lines=filereader(Testresult_filepath)

#initiate variable name and value vectors as empty lists 
variable_names_list=[]
variable_values_list=[]

#initiate water_balance as empty list
water_balance_list=[]

#initiate counter
i=0
#loop through each line of file_lines database
print('Reading input data: {}'.format(Testresult_filepath))
for each_line in file_lines:
    #if the header line, then print headers without appending data to lists with variable names and values 
    if (i == 0):
        print('{}. {} = {}'.format(i, file_lines[i][0], file_lines[i][1]))
    #if after the header line, then append data to lists with variable names and values 
    else:
        #assign variable name as string (location 0) the variable value as float (location 1)
        globals()[str(file_lines[i][0])] = float(file_lines[i][1])
        #append variable name to list as string
        variable_names_list.append (str(file_lines[i][0]))
        #append variable value to list as float
        variable_values_list.append (float(file_lines[i][1]))
        #print to xTerm the variable name and value
        print('{}. {} = {}'.format(i, file_lines[i][0], file_lines[i][1]))
        #print('{}.'.format(i))
    i=i+1

##Analyze average and local soil moisture deficit, and depth to water table


##Define terms used in water balance calculations, using sub-sections numbers 1 to 20, each a compartment of the water balance
##  Terms may be transformed from depths for watershed area to depth for fractional area, such as impervious or pervious area (IA or PA)
##  Total_Precip_RainSWE_mm is evenly distributed across all catchment areas

#Compute fraction relative to catchment area CA, pervious area PA, or impervious area IA
CA_TC_frac = Area_ImperviousTree_frac + Area_PerviousTree_frac
CA_SV_frac = Area_ShortVegCover_frac
CA_NoTC_frac = 1 - CA_TC_frac
PA_NoTC_frac = Area_PerviousAll_frac - Area_PerviousTree_frac
PA_TC_frac = Area_PerviousTree_frac
IA_NoTC_frac = Area_ImperviousAll_frac - Area_ImperviousTree_frac
IA_TC_frac = Area_ImperviousTree_frac
CA_Open_frac = 1 - CA_TC_frac - Area_ShortVegCover_frac
CA_PA_frac = 1 - Area_ImperviousAll_frac

##1 Tree Canopy water balance (mm), relative to tree canopy area
if (CA_TC_frac > 0):
    TC_Total_Precip_RainSWE_mm = Total_Precip_RainSWE_mm
    TC_Evap_mm = Total_TreeEvap_RainSWE_mm / CA_TC_frac
    TC_StorageFinal_Canopy_mm = (TS_Last_Tree_LeafStorage_Rain_mm + TS_Last_Tree_LeafStorage_SWE_mm) / CA_TC_frac
    TC_ThroughFall_mm = Total_TreeThruFlow_RainSWE_mm / CA_TC_frac
else:
    TC_Total_Precip_RainSWE_mm = 0
    TC_Evap_mm = 0
    TC_StorageFinal_Canopy_mm = 0
    TC_ThroughFall_mm = 0
##  Balance is simple input = change in storage + output(s)
TC_WaterBalance_mm = TC_Total_Precip_RainSWE_mm - TC_StorageFinal_Canopy_mm - TC_Evap_mm - TC_ThroughFall_mm
water_balance_list.append(TC_WaterBalance_mm)

##2 Short Veg Canopy water balance (mm), relative to short vegetation area
if (CA_SV_frac > 0):
    SV_Total_Precip_RainSWE_mm = Total_Precip_RainSWE_mm
    SV_Evap_mm = Total_ShortVegEvap_RainSWE_mm / CA_SV_frac
    SV_StorageFinal_Canopy_mm = (TS_Last_ShortVeg_LeafStorage_Rain_mm + TS_Last_ShortVeg_LeafStorage_SWE_mm)/ CA_SV_frac
    SV_ThroughFall_mm = Total_ShortVegThruFlow_RainSWE_mm / CA_SV_frac
else:
    SV_Total_Precip_RainSWE_mm = 0
    SV_Evap_mm = 0
    SV_StorageFinal_Canopy_mm = 0
    SV_ThroughFall_mm = 0
##  Balance is simple input = change in storage + output(s)
SV_WaterBalance_mm = SV_Total_Precip_RainSWE_mm - SV_StorageFinal_Canopy_mm - SV_Evap_mm - SV_ThroughFall_mm
water_balance_list.append(SV_WaterBalance_mm)
    
##3 Canopy Water balance (mm), relative to catchment area
TC_SV_Total_Precip_RainSWE_mm = Total_Precip_RainSWE_mm * (CA_TC_frac + CA_SV_frac)
CA_TC_SV_Evap_mm = Total_TreeEvap_RainSWE_mm + Total_ShortVegEvap_RainSWE_mm
CA_TC_SV_StorageFinal_Canopy_mm = (TS_Last_Tree_LeafStorage_Rain_mm + TS_Last_Tree_LeafStorage_SWE_mm) + (TS_Last_ShortVeg_LeafStorage_Rain_mm + TS_Last_ShortVeg_LeafStorage_SWE_mm) 
CA_TC_SV_ThroughFall_mm = Total_TreeThruFlow_RainSWE_mm + Total_ShortVegThruFlow_RainSWE_mm
##  Balance is simple input = change in storage + output(s)
CA_TC_SV_WaterBalance_mm = TC_SV_Total_Precip_RainSWE_mm - CA_TC_SV_StorageFinal_Canopy_mm - CA_TC_SV_Evap_mm - CA_TC_SV_ThroughFall_mm
water_balance_list.append(CA_TC_SV_WaterBalance_mm)

##4 Impervious Area Water Atmospheric Inputs balance (mm), relative to catchment area
CA_IA_WaterOn_mm = Total_WaterOnImpArea_mm
CA_IA_NoTC_Precip_mm = IA_NoTC_frac * Total_Precip_RainSWE_mm
#if area with trees, then reduce by impervious area with trees
if (CA_TC_frac > 0):
    CA_IA_TC_Throughfall_mm = IA_TC_frac * Total_TreeThruFlow_RainSWE_mm / CA_TC_frac
    CA_IA_TC_Sublimation_SnowWE_mm = IA_TC_frac * Total_SnowWE_Sub_UnderTree_mm / CA_TC_frac
    CA_IA_TC_StorageFinal_Ground_SnowWE_mm = IA_TC_frac * TS_Last_SnowWE_UnderTree_mm / CA_TC_frac
else:
    CA_IA_TC_Throughfall_mm = 0
    CA_IA_TC_Sublimation_SnowWE_mm = 0
    CA_IA_TC_StorageFinal_Ground_SnowWE_mm = 0  
#if area without trees and short veg, then reduce by impervious area without trees
if (CA_Open_frac > 0):
    CA_IA_NoTC_Sublimation_SnowWE_mm = IA_NoTC_frac * Total_SnowWE_Sub_OpenArea_mm / CA_Open_frac
    CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm = IA_NoTC_frac * TS_Last_SnowWE_Open_mm / CA_Open_frac
else:
    CA_IA_NoTC_Sublimation_SnowWE_mm = 0
    CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm = 0
##  Balance is simple input = change in storage + output(s) [Snow melt is included in CA_IA_WaterOn_mm]
CA_IA_WaterBalance_Input_mm = CA_IA_TC_Throughfall_mm + CA_IA_NoTC_Precip_mm - CA_IA_TC_Sublimation_SnowWE_mm - CA_IA_TC_StorageFinal_Ground_SnowWE_mm - CA_IA_NoTC_Sublimation_SnowWE_mm - CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm - CA_IA_WaterOn_mm
water_balance_list.append(CA_IA_WaterBalance_Input_mm)

##5 Impervious Area Water Storage + Atmospheric Outputs balance (mm), relative to catchment area
CA_IA_WaterOn_mm = Total_WaterOnImpArea_mm
CA_IA_StorageFinal_Depression_mm = TS_Last_ImpDepStor_mm
CA_IA_Evaporation_Depression_mm = Total_ImperDepEvap_mm
if (Area_ImperviousAll_frac > 0):
    CA_IA_Runoff_To_PA_mm = Total_ImperFlowToSoil_mm
    CA_IA_Runoff_To_DCIA_mm = Total_ImpFlowToOutlet_mm
else:
    CA_IA_Runoff_To_PA_mm = 0
    CA_IA_Runoff_To_DCIA_mm = 0
##  Balance is simple input = change in storage + output(s) [Snow melt is included in WaterOn]
CA_IA_WaterBalance_OnOut_mm = CA_IA_WaterOn_mm - CA_IA_StorageFinal_Depression_mm - CA_IA_Evaporation_Depression_mm - CA_IA_Runoff_To_PA_mm - CA_IA_Runoff_To_DCIA_mm
water_balance_list.append(CA_IA_WaterBalance_OnOut_mm)

##6 Impervious Area non-routed Runoff balance (mm), relative to catchment area
CA_IA_Runoff_NR_mm =  Total_Impervious_Flow_NonRouted_mm
if (Area_ImperviousAll_frac > 0):
    CA_IA_Runoff_To_DCIA_mm = Total_ImpFlowToOutlet_mm
else:
    CA_IA_Runoff_To_DCIA_mm = 0
##  Balance is simple input = change in storage + output(s)
CA_IA_WaterBalance_RunOff_mm = CA_IA_Runoff_To_DCIA_mm - CA_IA_Runoff_NR_mm
water_balance_list.append(CA_IA_WaterBalance_RunOff_mm)

##7 Pervious Area Water Atmospheric Inputs balance (mm), relative to catchment area
CA_PA_WaterOn_mm = Total_WaterOnPerArea_mm
CA_PA_NoTC_NoSV_Precip_mm = (PA_NoTC_frac - CA_SV_frac) * Total_Precip_RainSWE_mm
CA_SV_ThroughFall_mm = Total_ShortVegThruFlow_RainSWE_mm
CA_SV_Sublimation_SnowWE_mm = Total_SnowWE_Sub_UnderShortVeg_mm
CA_SV_StorageFinal_Ground_SnowWE_mm = TS_Last_SnowWE_UnderSVeg_mm
#if area with trees, then reduce by pervious area with trees
if (CA_TC_frac > 0):
    CA_PA_TC_ThroughFall_mm = PA_TC_frac * Total_TreeThruFlow_RainSWE_mm / CA_TC_frac
    CA_PA_TC_Sublimation_SnowWE_mm = PA_TC_frac * Total_SnowWE_Sub_UnderTree_mm / CA_TC_frac
    CA_PA_TC_StorageFinal_Ground_SnowWE_mm = PA_TC_frac * TS_Last_SnowWE_UnderTree_mm / CA_TC_frac
else:
    CA_PA_TC_ThroughFall_mm = 0
    CA_PA_TC_Sublimation_SnowWE_mm = 0
    CA_PA_TC_StorageFinal_Ground_SnowWE_mm = 0  
#if area without trees and short veg, then reduce by pervious area without vegetation
if (CA_Open_frac > 0):
    CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm = (PA_NoTC_frac - CA_SV_frac) * Total_SnowWE_Sub_OpenArea_mm / CA_Open_frac
    CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm = (PA_NoTC_frac - CA_SV_frac) * TS_Last_SnowWE_Open_mm / CA_Open_frac
else:
    CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm =  0
    CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm = 0  
##  Balance is simple input = change in storage + output(s)
CA_PA_Water_To_StorageFinal_Depression_mm = (CA_PA_TC_ThroughFall_mm + CA_SV_ThroughFall_mm + CA_PA_NoTC_NoSV_Precip_mm) - CA_PA_TC_Sublimation_SnowWE_mm - CA_PA_TC_StorageFinal_Ground_SnowWE_mm - CA_SV_Sublimation_SnowWE_mm - CA_SV_StorageFinal_Ground_SnowWE_mm - CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm -  CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm - CA_PA_WaterOn_mm 
water_balance_list.append(CA_PA_Water_To_StorageFinal_Depression_mm)

##8 Pervious Area Water Atmospheric + Runon Inputs balance (mm), relative to catchment area
CA_PA_WaterOn_mm = Total_WaterOnPerArea_mm
CA_PA_Water_To_StorageFinal_Depression_mm = Total_WaterToPerDep_mm
if (Area_ImperviousAll_frac > 0):
    CA_PA_Runon_From_IA_mm = Total_ImperFlowToSoil_mm
else:
    CA_PA_Runon_From_IA_mm = 0
##  Balance is simple input = change in storage + output(s)
CA_PA_IAro_Water_To_StorageFinal_Depression_mm = CA_PA_Water_To_StorageFinal_Depression_mm - CA_PA_WaterOn_mm - CA_PA_Runon_From_IA_mm
water_balance_list.append(CA_PA_IAro_Water_To_StorageFinal_Depression_mm)

##9 Pervious Area Water Storage + Atmospheric Outputs + Infiltration balance (mm), relative to catchment area
CA_PA_Water_To_Infil_mm = Total_FlowToInfil_mm 
CA_PA_Evaporation_Depression_mm = Total_PerDepEvap_mm
CA_PA_StorageFinal_Depression_mm = TS_Last_PerDepStor_mm
##  Balance is simple input = change in storage + output(s)
CA_PA_WaterToInfilt_WaterBalance_mm = CA_PA_Water_To_StorageFinal_Depression_mm - CA_PA_StorageFinal_Depression_mm - CA_PA_Evaporation_Depression_mm - CA_PA_Water_To_Infil_mm 
water_balance_list.append(CA_PA_WaterToInfilt_WaterBalance_mm)

##10 Pervious Area Water to Infiltration + Runoff + Macroporeflow balance (mm), relative to catchment area
CA_PA_Runoff_InfilExcess_mm = Total_Infil_Excess_RO_mm
CA_PA_Runoff_SatExcess_mm = Total_Sat_Excess_RO_mm
CA_PA_Infiltration_InfilExcess_mm = Total_InfilExInfil_mm
CA_PA_Infiltration_SatExcess_mm = Total_SatEx_Infil_mm
CA_PA_Drainage_Macropore_mm = Total_PMacroFlow_mm
##  Balance is simple input = change in storage + output(s)
CA_PA_Infiltration_WaterBalance_mm = CA_PA_Water_To_Infil_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm - CA_PA_Drainage_Macropore_mm 
water_balance_list.append(CA_PA_Infiltration_WaterBalance_mm)

##11 Pervious Area Infiltration routines balance (mm), relative to catchment area
CA_PA_Infiltration_InfilExcess_mm = Total_InfilExInfil_mm
CA_PA_Infiltration_SatExcess_mm = Total_SatEx_Infil_mm
CA_PA_Infiltration_mm = Total_Infiltration_mm 
##  Balance is simple input = change in storage + output(s)
CA_PA_InfiltComponents_WaterBalance_mm = CA_PA_Infiltration_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm
water_balance_list.append(CA_PA_InfiltComponents_WaterBalance_mm)

##12 Pervious Area non-routed Runoff routines (mm), relative to catchment area
CA_PA_Runoff_NR_mm = Total_Pervious_Flow_NonRouted_mm
CA_PA_Runoff_InfilExcess_mm = Total_Infil_Excess_RO_mm
CA_PA_Runoff_SatExcess_mm = Total_Sat_Excess_RO_mm
##  Balance is simple input = change in storage + output(s)
CA_PA_WaterToInfilt_WaterBalance_mm = CA_PA_Runoff_NR_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm
water_balance_list.append(CA_PA_WaterToInfilt_WaterBalance_mm)

##13 Pervious Area Root Zone balance (mm), relative to catchment area
CA_PA_Infiltration_mm = Total_Infiltration_mm 
CA_PA_Drainage_Macropore_mm = Total_PMacroFlow_mm
CA_PA_EvapTranspiration_RootZone_mm = Total_RootZoneET_mm
CA_PA_StorageChange_RootZone_mm = (TS_Last_RootZoneStor_mm - TS_1st_RootZoneStor_mm) 
CA_PA_Drainage_RootZone_mm = Total_RootZone_Flow_mm
##  Balance is simple input = change in storage + output(s)
CA_PA_RootZone_WaterBalance_mm = CA_PA_Infiltration_mm + CA_PA_Drainage_Macropore_mm - CA_PA_StorageChange_RootZone_mm - CA_PA_EvapTranspiration_RootZone_mm - CA_PA_Drainage_RootZone_mm
water_balance_list.append(CA_PA_RootZone_WaterBalance_mm)

##14 Pervious Area Unsaturated Zone balance (mm), relative to catchment area
CA_PA_StorageChange_UnsatZone_mm = (TS_Last_UnsatZoneStor_mm - TS_1st_UnsatZoneStor_mm)
CA_PA_Drainage_UnsatZone_mm = Total_UnsatZone_Flow_mm
CA_PA_Drainage_RootZone_mm = Total_RootZone_Flow_mm
##  Balance is simple input = change in storage + output(s)   
CA_PA_UnsatZone_WaterBalance_mm = CA_PA_Drainage_RootZone_mm - CA_PA_Drainage_UnsatZone_mm - CA_PA_StorageChange_UnsatZone_mm
water_balance_list.append(CA_PA_UnsatZone_WaterBalance_mm)

##15 Pervious Area Saturated Zone balance (mm), relative to catchment area
#Theory is in HydroPlus.sln SemiAveSMDCalc.cpp, aveSMD = aveSMD - Drainage_UnsatZone_m + Runoff_Subsurface_m
CA_PA_Drainage_UnsatZone_mm = Total_UnsatZone_Flow_mm
CA_PA_Runoff_Subsurface_mm = Total_SubsurfaceFlow_NonRouted_mm
CA_PA_StorageDeficit_AvgSoil_1st_mm = TS_1st_AveSMD_mm 
CA_PA_StorageDeficit_AvgSoil_Last_mm = TS_Last_AveSMD_mm
##  Balance is simple input = change in storage + output(s)
CA_PA_SubsurfaceFlow_WaterBalance_mm = CA_PA_StorageDeficit_AvgSoil_1st_mm - Total_UnsatZone_Flow_mm + CA_PA_Runoff_Subsurface_mm - CA_PA_StorageDeficit_AvgSoil_Last_mm
water_balance_list.append(CA_PA_SubsurfaceFlow_WaterBalance_mm)

##16 Total Snow Water Equivalent balance (mm), relative to catchment area
CA_Precip_SnowWE_mm = Total_SnowWE_mm 
CA_Total_Sublimation_Ground_SnowWE_mm = Total_SnowWE_Sub_UnderTree_mm + Total_SnowWE_Sub_UnderShortVeg_mm + Total_SnowWE_Sub_OpenArea_mm
CA_Total_Sublimation_Canopy_SnowWE_mm = Total_SnowWE_Sub_Tree_mm + Total_SnowWE_Sub_SVeg_mm
CA_Total_Melt_Ground_SnowWE_mm = Total_SnowWE_Melt_UnderTree_mm +  Total_SnowWE_Melt_UnderShortVeg_mm +  Total_SnowWE_Melt_OpenArea_mm
CA_Total_Melt_Canopy_SnowWE_mm = Total_SnowWE_Melt_TreeCanopy_mm + Total_SnowWE_Melt_SVegCanopy_mm
CA_Total_Storage_Ground_SnowWE_mm = TS_Last_SnowWE_UnderTree_mm + TS_Last_SnowWE_UnderSVeg_mm + TS_Last_SnowWE_Open_mm
CA_TS_Last_Canopy_SnowWE_mm = TS_Last_Tree_LeafStorage_SWE_mm + TS_Last_ShortVeg_LeafStorage_SWE_mm
##  Balance is simple input = change in storage + output(s)   
CA_SnowWE_WaterBalance_mm = CA_Precip_SnowWE_mm - CA_Total_Sublimation_Ground_SnowWE_mm - CA_Total_Sublimation_Canopy_SnowWE_mm - CA_Total_Melt_Ground_SnowWE_mm - CA_Total_Melt_Canopy_SnowWE_mm - CA_Total_Storage_Ground_SnowWE_mm - CA_TS_Last_Canopy_SnowWE_mm
water_balance_list.append(CA_SnowWE_WaterBalance_mm)

##17 Total Surface Water balance NonRouted (mm), relative to catchment area
CA_Total_Precip_RainSWE_mm = Total_Precip_RainSWE_mm
CA_Total_EvapoSublimation_Canopy_mm = Total_TreeEvap_RainSWE_mm + Total_ShortVegEvap_RainSWE_mm 
CA_Total_EvapoSublimation_Ground_mm = Total_ImperDepEvap_mm + Total_PerDepEvap_mm + Total_SnowWE_Sub_UnderTree_mm + Total_SnowWE_Sub_UnderShortVeg_mm + Total_SnowWE_Sub_OpenArea_mm
CA_Total_StorageFinal_Canopy_mm = (TS_Last_Tree_LeafStorage_Rain_mm + TS_Last_Tree_LeafStorage_SWE_mm) + (TS_Last_ShortVeg_LeafStorage_Rain_mm + TS_Last_ShortVeg_LeafStorage_SWE_mm)
CA_Total_StorageFinal_Ground_mm = TS_Last_ImpDepStor_mm + TS_Last_PerDepStor_mm + TS_Last_SnowWE_Open_mm + TS_Last_SnowWE_UnderTree_mm + TS_Last_SnowWE_UnderSVeg_mm
CA_Total_Drainage_ToSoil_mm = Total_PMacroFlow_mm + Total_Infiltration_mm
CA_Total_Runoff_Surface_NR_mm = Total_Pervious_Flow_NonRouted_mm + Total_Impervious_Flow_NonRouted_mm 
##  Balance is simple input = change in storage + output(s)
CA_Total_WaterBalance_NR_mm = CA_Total_Precip_RainSWE_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_Drainage_ToSoil_mm - CA_Total_Runoff_Surface_NR_mm
water_balance_list.append(CA_Total_WaterBalance_NR_mm)

##18 Total Surface and Subsurface Water balance NonRouted (mm), relative to catchment area
CA_Total_Precip_RainSWE_mm = Total_Precip_RainSWE_mm
CA_Total_EvapoTranspiration_Soil_mm = Total_RootZoneET_mm
CA_Total_EvapoSublimation_Canopy_mm = Total_TreeEvap_RainSWE_mm + Total_ShortVegEvap_RainSWE_mm 
CA_Total_EvapoSublimation_Ground_mm = Total_ImperDepEvap_mm + Total_PerDepEvap_mm + Total_SnowWE_Sub_UnderTree_mm + Total_SnowWE_Sub_UnderShortVeg_mm + Total_SnowWE_Sub_OpenArea_mm
CA_Total_StorageFinal_Canopy_mm = (TS_Last_Tree_LeafStorage_Rain_mm + TS_Last_Tree_LeafStorage_SWE_mm) + (TS_Last_ShortVeg_LeafStorage_Rain_mm + TS_Last_ShortVeg_LeafStorage_SWE_mm)
CA_Total_StorageFinal_Ground_mm = TS_Last_ImpDepStor_mm + TS_Last_PerDepStor_mm + TS_Last_SnowWE_Open_mm + TS_Last_SnowWE_UnderTree_mm + TS_Last_SnowWE_UnderSVeg_mm
CA_Total_StorageChange_Subsurface_mm = (TS_Last_RootZoneStor_mm - TS_1st_RootZoneStor_mm) + (TS_Last_UnsatZoneStor_mm - TS_1st_UnsatZoneStor_mm) + (TS_1st_AveSMD_mm - TS_Last_AveSMD_mm)
CA_Total_Runoff_NR_mm = Total_Flow_NonRouted_mm
##  Balance is simple input = change in storage + output(s)
CA_Total_WaterBalance_R_mm = CA_Total_Precip_RainSWE_mm - CA_Total_EvapoTranspiration_Soil_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_StorageChange_Subsurface_mm - CA_Total_Runoff_NR_mm
water_balance_list.append(CA_Total_WaterBalance_R_mm)

##19.Runoff NonRoutedRouted vs  Balance (mm), relative to catchment area
#cte issue in HydroPlus with creating or destroying water with diffusion hydrograph
Total_Pervious_Flow_NonRouted_mm
Total_Impervious_Flow_NonRouted_mm
Total_SubsurfaceFlow_NonRouted_mm
Total_Flow_NonRouted_mm
Total_Pervious_Flow_Routed_mm
Total_Impervious_Flow_Routed_mm
Total_SubsurfaceFlow_Routed_mm
Total_Flow_Routed_mm
##  Balance is simple input = (non-routed - routed) 
CA_RunoffRouting_Balance_mm = (Total_Pervious_Flow_NonRouted_mm - Total_Pervious_Flow_Routed_mm) + (Total_Impervious_Flow_NonRouted_mm - Total_Impervious_Flow_Routed_mm) + (Total_SubsurfaceFlow_NonRouted_mm - Total_SubsurfaceFlow_Routed_mm) + (Total_Flow_NonRouted_mm - Total_Flow_Routed_mm)
water_balance_list.append(CA_RunoffRouting_Balance_mm)

##20.Other Balance? (mm), relative to catchment area
#OtherBalance = 0
#water_balance_list.append(OtherBalance)

#write results
timeN = time.strftime("Testing_%Y_%m_%d_%H_%M_%S")

##Name of output file is WB_Test.dat
outputFile = Testresult_path+"\\WB_Test.dat"

if os.path.exists(outputFile):
  os.remove(outputFile)

##Open output file for writing
with open(outputFile,'a') as f:

    #Write the time of the test
    f.write("Test time: " + timeN +"\n")

    #Write out variables read from HP_Output_file
    f.write("==============================================================================\n")
    f.write("Writing content of UH Output file: {}\n\n".format(HP_Output_file))

    #Write the header for all variables read from input
    f.write("Variable:                                         " + "Value" +"\n")
    f.write("------------------------------------------------------------------------------\n")
     
    #Loop through all variables in list of variable values
    for x in range(0, len(variable_values_list)):
        #Format output using spacenum, a space number for indentation that changes with length of variable name, with a maximum of 50 spaces
        spacenum=50-len(variable_names_list[x])
        #Check if variable magnitude < 0.1 but > 0 to determine if writing output in scientific notation
        if (variable_values_list[x]*10 < 1 and variable_values_list[x] > 0):
            #Write output in scientific notation, and adjust blank spaces for length of variable name with " "*spacenum
            f.write("{}:{}{:.2e} \n".format(variable_names_list[x]," "*spacenum,variable_values_list[x]))
        else:
            #Write output in float notation, and adjust blank spaces for length of variable name with " "*spacenum
            f.write("{}:{}{:.2f} \n".format(variable_names_list[x]," "*spacenum,variable_values_list[x]))

    #Write out local soil moisture deficit for each bin of topographic index
    #References:
    #Famiglietti, J. S., & Wood, E. F. (1994). Multiscale Modeling of Spatially Variable Water and Energy Balance Processes. Water Resources Research, 30(11), 3061-3078. 
    #Hornberger, G. M., Beven, K. J., Cosby, B. J., & Sappington, D. E. (1985). Shenandoah Watershed Study: Calibration of a Topography-Based, Variable Contributing Area Hydrological Model to a Small Forested Catchment. Water Resources Research, 21(12), 1841-1850.
    #TMOD9502.FOR TOPMODEL DEMONSTRATION PROGRAM VERSION 95.02, By Keith Beven or Eq given as misc.S_mean[0] = - params.m * log(misc.qs0 / misc.qss) in param.init.c https://cran.r-project.org/web/packages/topmodel/index.html
    #Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
    #Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
    
    f.write("==============================================================================\n")
    f.write("Local soil moisture deficit at 1st time step of simulation \n\n")
    f.write("TopographicIndex_Avg: {:.2f} \n".format(TopographicIndex_Avg))
    #T0 is maximum transmissivity when groundwater table is at the ground surface
    f.write("T0_m2_p_dt: {:.2e} m^2/dt \n".format(T0_m2_p_dt))
    #Q0 is the drainage from the basin at the start of the simulation
    f.write("Q0_m_p_dt: {:.2e} m/dt \n".format(Q0_m_p_dt))
    f.write("m_ScalingParam: {:.2f} \n".format(m_ScalingParam))
    f.write("n_PowerDecayParam: {} \n".format(n_PowerDecayParam))
    f.write("TS_1st_SurficialGroundwater_frac: {:.2f} frac \n".format(TS_1st_SurficialGroundwater_frac))
    f.write("TS_Last_SurficialGroundwater_frac: {:.2f} frac \n".format(TS_Last_SurficialGroundwater_frac))
    f.write("TS_1st_SurficialGroundwater_mm: {:.2f} mm \n".format(TS_1st_SurficialGroundwater_mm))
    f.write("TS_Last_SurficialGroundwater_mm: {:.2f} mm \n".format(TS_Last_SurficialGroundwater_mm))
    #NAC is number of topographic index bins
    f.write("NAC: {} \n".format(NAC))
    #cte update this to be sure it is start and stop of TI
    for x in range(0, len(variable_values_list)):
        if (variable_names_list[x] == "TopographicIndex_0"):
            row_ti_start = x
            #row_ti_stop = int(x + NAC - 1)
            row_ti_stop = int(x + NAC)
 
    #select if Decay function is ExponentialDecay or PowerDecay, which use different computations for AveSMD and localSMD
    if (HydraulicConductivityDecay_function == 0):
        f.write("HydraulicConductivityDecay_function: ExponentialDecay \n")

        #SZQ (m) is maximum potential subsurface flow rate, when groundwater table is at the ground surface
        #Eq for SZQ similar to Eq 31b for Q0 in Famiglietti and Wood (1994)
        #Eq for SZQ below Eq 3 of Wang et al. (2005), S_bar = -m*ln(R/T0) -m*Lambda, and SZQ becomes T0 * exp(-Lambda), where R = Q0
        SZQ = T0_m2_p_dt * math.exp(-1*TopographicIndex_Avg)
        f.write("SZQ: {:.2e} \n".format(SZQ))
        
        #Eq for AveSMD derived from Eq 4 in Horberger et al. (1985) Qb = SZQ * EXP(-S/SZM)
        #Eq for AveSMD Eq given as SBAR=-SZM*ALOG(Q0/SZQ) in TMOD9502.FOR or param.init.c 
        #Eq for AveSMD below Eq 3 of Wang et al. (2005), S_bar = -m*ln(R/T0) -m*Lambda
        #AveSMD computation uses Natural log of (Q0_m_p_dt / SZQ)
        AveSMD = -1 * m_ScalingParam * math.log(Q0_m_p_dt / SZQ)
        f.write("AveSMD: {:.2e} \n".format(AveSMD))
        
        #initialize list of length NAC, and counter to 0
        localSMD = [0] * int(NAC)
        i = 0
        #loop through all TI bins
        for x in range(row_ti_start , row_ti_stop):
            localSMD[i] = AveSMD + m_ScalingParam * (TopographicIndex_Avg - variable_values_list[x])
            f.write("localSMD[{}]: {:.2e} m \n".format(i,localSMD[i]))
            i = i + 1
            
    elif (HydraulicConductivityDecay_function == 1):
        f.write("HydraulicConductivityDecay_function: PowerDecay \n")

        #SZQ (m) is maximum potential subsurface flow rate, when groundwater table is at the ground surface
        #Eq for SZQ is RHS of Eq for S_bar, aveSMD, given in text below Eq 9 of Wang et al. (2006) 
        SZQ = pow((Q0_m_p_dt/T0_m2_p_dt),(1/n_PowerDecayParam))*TopographicIndex_Avg
        f.write("SZQ: {:.2e} \n".format(SZQ))

        #Eq for aveSMD is below Eq 9 of Wang et al. (2006) and below Eq 5 of Wang (2005)
        #AveSMD computation 
        AveSMD = m_ScalingParam * (1 - SZQ)
        f.write("AveSMD: {:.2e} \n".format(AveSMD))

        #initialize list of length NAC, and counter to 0
        localSMD = [0] * int(NAC)
        i = 0
        #loop through all TI bins
        for x in range(row_ti_start , row_ti_stop):
            localSMD[i] = AveSMD + m_ScalingParam * (pow((Q0_m_p_dt/T0_m2_p_dt),(1/n_PowerDecayParam))*(TopographicIndex_Avg - variable_values_list[x]))
            f.write("localSMD[{}]: {:.2e} m \n".format(i,localSMD[i]))
            i = i + 1
    f.write("\n\n")
    #cte 20210313 Write the total fraction that is negative
    f.write("Note: If the above localSMD list contains many negative values, the model may be erroneously over-representing a saturated areas and its runoff. It may also cause Eq 15 below (e.g., Pervious Area Saturated Zone balance) to not balance.\n")
    f.write("Note: If the you wish to reduce the negative values from the localSMD, options include:\n")
    f.write("Note:      1. The T0_m2_p_dt input value can be increased to remove the negative localSMD; T0_m2_p_dt values can go into 1000s.\n")
    f.write("Note:      2. The spread between the maximum and average topographic index (TI) values can be reduced to remove the negative localSMD. This might be approached by reducing the number of TI bins, or using a coarser DEM resolution.\n \n")


    #Write out specific water balances for each component of vertical system
    f.write("==============================================================================\n")
    f.write("Water Balance Calculations \n\n")
    
    f.write("1.Tree Canopy balance, over tree cover area (mm)\n")
    f.write("Balance_mm =\t TC_Total_Precip_RainSWE_mm - TC_StorageFinal_Canopy_mm - TC_Evap_mm - TC_ThroughFall_mm \n" )
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} - {:.2f} mm \n".format(TC_Total_Precip_RainSWE_mm, TC_StorageFinal_Canopy_mm, TC_Evap_mm, TC_ThroughFall_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[0]))
    
    f.write("2.Short Veg Canopy balance, over short vegetation area (mm)\n")
    f.write("Balance_mm =\t SV_Total_Precip_RainSWE_mm - SV_StorageFinal_Canopy_mm - SV_Evap_mm - SV_ThroughFall_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} - {:.2f} mm \n". format(SV_Total_Precip_RainSWE_mm, SV_StorageFinal_Canopy_mm, SV_Evap_mm, SV_ThroughFall_mm))
    f.write( "Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[1]))
    
    f.write("3.Tree & Short Veg Canopy balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t TC_SV_Total_Precip_RainSWE_mm - CA_TC_SV_StorageFinal_Canopy_mm - CA_TC_SV_Evap_mm - CA_TC_SV_ThroughFall_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} - {:.2f} mm\n". format(TC_SV_Total_Precip_RainSWE_mm, CA_TC_SV_StorageFinal_Canopy_mm, CA_TC_SV_Evap_mm, CA_TC_SV_ThroughFall_mm))
    f.write( "Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[2]))

    f.write("4.Impervious Area Atmospheric Inputs balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_IA_TC_Throughfall_mm + CA_IA_NoTC_Precip_mm - CA_IA_TC_Sublimation_SnowWE_mm - CA_IA_TC_StorageFinal_Ground_SnowWE_mm - CA_IA_NoTC_Sublimation_SnowWE_mm - CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm - CA_IA_WaterOn_mm \n")
    f.write("Balance_mm = \t {:.2f} mm + {:.2f} mm - {:.2f} - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_IA_TC_Throughfall_mm,CA_IA_NoTC_Precip_mm, CA_IA_TC_Sublimation_SnowWE_mm, CA_IA_TC_StorageFinal_Ground_SnowWE_mm, CA_IA_NoTC_Sublimation_SnowWE_mm, CA_IA_NoTC_StorageFinal_Ground_SnowWE_mm, CA_IA_WaterOn_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[3]))

    f.write("5.Impervious Area Storage + Atmospheric Outputs balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_IA_WaterOn_mm - CA_IA_StorageFinal_Depression_mm - CA_IA_Evaporation_Depression_mm - CA_IA_Runoff_To_PA_mm - CA_IA_Runoff_To_DCIA_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} - {:.2f} mm - {:.2f} mm \n".format(CA_IA_WaterOn_mm,CA_IA_StorageFinal_Depression_mm,CA_IA_Evaporation_Depression_mm,CA_IA_Runoff_To_PA_mm,CA_IA_Runoff_To_DCIA_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[4]))

    f.write("6.Impervious Area non-routed Runoff balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_IA_Runoff_To_DCIA_mm - CA_IA_Runoff_NR_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm \n".format(CA_IA_Runoff_To_DCIA_mm,CA_IA_Runoff_NR_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n". format(water_balance_list[5]))

    f.write("7.Pervious Area Atmospheric Inputs balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t (CA_PA_TC_ThroughFall_mm + CA_SV_ThroughFall_mm + CA_PA_NoTC_NoSV_Precip_mm) - CA_PA_TC_Sublimation_SnowWE_mm - CA_PA_TC_StorageFinal_Ground_SnowWE_mm - CA_SV_Sublimation_SnowWE_mm - CA_SV_StorageFinal_Ground_SnowWE_mm - CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm -  CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm - CA_PA_WaterOn_mm \n")
    f.write("Balance_mm =\t ({:.2f} mm + {:.2f} mm + {:.2f} mm) - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_PA_TC_ThroughFall_mm, CA_SV_ThroughFall_mm, CA_PA_NoTC_NoSV_Precip_mm, CA_PA_TC_Sublimation_SnowWE_mm,CA_PA_TC_StorageFinal_Ground_SnowWE_mm, CA_SV_Sublimation_SnowWE_mm, CA_SV_StorageFinal_Ground_SnowWE_mm, CA_PA_NoTC_NoSV_Sublimation_SnowWE_mm,CA_PA_NoTC_NoSV_StorageFinal_Ground_SnowWE_mm, CA_PA_WaterOn_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[6]))

    f.write("8.Pervious Area Atmospheric + Runon Inputs balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Water_To_StorageFinal_Depression_mm = CA_PA_WaterOn_mm + CA_PA_Runon_From_IA_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_PA_Water_To_StorageFinal_Depression_mm,CA_PA_WaterOn_mm,CA_PA_Runon_From_IA_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[7]))

    f.write("9.Pervious Area Storage + Atmospheric Outputs + Infiltration balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Water_To_StorageFinal_Depression_mm - CA_PA_StorageFinal_Depression_mm - CA_PA_Evaporation_Depression_mm - CA_PA_Water_To_Infil_mm\n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_PA_Water_To_StorageFinal_Depression_mm,CA_PA_StorageFinal_Depression_mm,CA_PA_Evaporation_Depression_mm,CA_PA_Water_To_Infil_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[8]))

    f.write("10.Pervious Area Water to Infiltration + Runoff + Macroporeflow balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Water_To_Infil_mm - CA_PA_Runoff_InfilExcess_mm - CA_PA_Runoff_SatExcess_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm - CA_PA_Drainage_Macropore_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} \n".format(CA_PA_Water_To_Infil_mm,CA_PA_Runoff_InfilExcess_mm,CA_PA_Runoff_SatExcess_mm,CA_PA_Infiltration_InfilExcess_mm,CA_PA_Infiltration_SatExcess_mm,CA_PA_Drainage_Macropore_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[9]))

    f.write("11.Pervious Area Infiltration routines balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Infiltration_mm - CA_PA_Infiltration_InfilExcess_mm - CA_PA_Infiltration_SatExcess_mm \n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_PA_Infiltration_mm,CA_PA_Infiltration_InfilExcess_mm,CA_PA_Infiltration_SatExcess_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[10]))

    f.write("12.Pervious Area non-routed Runoff routines balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Runoff_InfilExcess_mm + CA_PA_Runoff_SatExcess_mm - CA_PA_Runoff_NR_mm \n")
    f.write("Balance_mm =\t {:.2f} mm + {:.2f} mm - {:.2f} mm \n".format(CA_PA_Runoff_InfilExcess_mm,CA_PA_Runoff_SatExcess_mm,CA_PA_Runoff_NR_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[11]))

    f.write("13.Pervious Area Root Zone balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Infiltration_mm + CA_PA_Drainage_Macropore_mm - CA_PA_StorageChange_RootZone_mm - CA_PA_EvapTranspiration_RootZone_mm - CA_PA_Drainage_RootZone_mm \n")
    f.write("Balance_mm =\t {:.2f} mm + {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_PA_Infiltration_mm,CA_PA_Drainage_Macropore_mm,CA_PA_StorageChange_RootZone_mm,CA_PA_EvapTranspiration_RootZone_mm,CA_PA_Drainage_RootZone_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[12]))

    f.write("14.Pervious Area Unsaturated Zone balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_PA_Drainage_RootZone_mm - CA_PA_Drainage_UnsatZone_mm - CA_PA_StorageChange_UnsatZone_mm\n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_PA_Drainage_RootZone_mm,CA_PA_Drainage_UnsatZone_mm,CA_PA_StorageChange_UnsatZone_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[13]))

    f.write("15.Pervious Area Saturated Zone balance, over catchment area (mm)\n")
    f.write("Balance_mm = \t CA_PA_StorageDeficit_AvgSoil_1st_mm - Total_UnsatZone_Flow_mm + CA_PA_Runoff_Subsurface_mm - CA_PA_StorageDeficit_AvgSoil_Last_mm\n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm + {:.2f} mm - {:.2f} mm \n".format(CA_PA_StorageDeficit_AvgSoil_1st_mm,Total_UnsatZone_Flow_mm,CA_PA_Runoff_Subsurface_mm,CA_PA_StorageDeficit_AvgSoil_Last_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[14]))

    f.write("16.Total Snow Water Equivalent balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_Precip_SnowWE_mm - CA_Total_Sublimation_Ground_SnowWE_mm - CA_Total_Sublimation_Canopy_SnowWE_mm - CA_Total_Melt_Ground_SnowWE_mm - CA_Total_Melt_Canopy_SnowWE_mm - CA_Total_Storage_Ground_SnowWE_mm - CA_TS_Last_Canopy_SnowWE_mm\n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm \n".format(CA_Precip_SnowWE_mm,CA_Total_Sublimation_Ground_SnowWE_mm,CA_Total_Sublimation_Canopy_SnowWE_mm,CA_Total_Melt_Ground_SnowWE_mm,CA_Total_Melt_Canopy_SnowWE_mm,CA_Total_Storage_Ground_SnowWE_mm,CA_TS_Last_Canopy_SnowWE_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[15]))

    f.write("17.Total Surface Water NonRouted balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_Total_Precip_RainSWE_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_Drainage_ToSoil_mm - CA_Total_Runoff_Surface_NR_mm\n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm\n".format(CA_Total_Precip_RainSWE_mm,CA_Total_EvapoSublimation_Canopy_mm,CA_Total_EvapoSublimation_Ground_mm, CA_Total_StorageFinal_Canopy_mm,CA_Total_StorageFinal_Ground_mm,CA_Total_Drainage_ToSoil_mm,CA_Total_Runoff_Surface_NR_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[16]))

    f.write("18.Total Water NonRouted balance, over catchment area (mm)\n")
    f.write("Balance_mm =\t CA_Total_Precip_RainSWE_mm - CA_Total_EvapoTranspiration_Soil_mm - CA_Total_EvapoSublimation_Canopy_mm - CA_Total_EvapoSublimation_Ground_mm - CA_Total_StorageFinal_Canopy_mm - CA_Total_StorageFinal_Ground_mm - CA_Total_StorageChange_Subsurface_mm - CA_Total_Runoff_NR_mm\n")
    f.write("Balance_mm =\t {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm - {:.2f} mm\n".format(CA_Total_Precip_RainSWE_mm,CA_Total_EvapoTranspiration_Soil_mm,CA_Total_EvapoSublimation_Canopy_mm,CA_Total_EvapoSublimation_Ground_mm, CA_Total_StorageFinal_Canopy_mm,CA_Total_StorageFinal_Ground_mm,CA_Total_StorageChange_Subsurface_mm,CA_Total_Runoff_NR_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[17]))

    f.write("19.Runoff NonRouted vs Routed Balance, over catchment area (mm) #cte issue in HydroPlus\n")
    f.write("Balance_mm =\t (Total_Pervious_Flow_NonRouted_mm - Total_Pervious_Flow_Routed_mm) + (Total_Impervious_Flow_NonRouted_mm - Total_Impervious_Flow_Routed_mm) + (Total_SubsurfaceFlow_NonRouted_mm - Total_SubsurfaceFlow_Routed_mm) + (Total_Flow_NonRouted_mm - Total_Flow_Routed_mm) \n")
    f.write("Balance_mm =\t ({:.2f} mm - {:.2f} mm) + ({:.2f} mm - {:.2f} mm) + ({:.2f} mm - {:.2f} mm) + ({:.2f} mm - {:.2f} mm) \n".format(Total_Pervious_Flow_NonRouted_mm,Total_Pervious_Flow_Routed_mm,Total_Impervious_Flow_NonRouted_mm,Total_Impervious_Flow_Routed_mm,Total_SubsurfaceFlow_NonRouted_mm,Total_SubsurfaceFlow_Routed_mm,Total_Flow_NonRouted_mm,Total_Flow_Routed_mm))
    f.write("Balance_mm =\t {:.2f} mm \n\n\n".format(water_balance_list[18]))

#close the output file
f.close()



